@@ -55,14 +55,15 @@ pub use crate::actions::{
     AccountAction::{self, *},
     Action::{self, *},
     ComposeAction::{self, *},
+    FlagAction,
     ListingAction::{self, *},
     MailingListAction::{self, *},
     TabAction::{self, *},
-    TagAction::{self, *},
+    TagAction,
     ViewAction::{self, *},
 };
 
-/// Helper macro to convert an array of tokens into a TokenStream
+/// Helper macro to convert an array of tokens into a `TokenStream`
 macro_rules! to_stream {
     ($token: expr) => {
         TokenStream {
@@ -111,7 +112,7 @@ pub fn quoted_argument(input: &[u8]) -> IResult<&[u8], &str> {
     }
 }
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Clone, Copy, Debug)]
 pub struct TokenStream {
     tokens: &'static [TokenAdicity],
 }
@@ -142,6 +143,11 @@ impl TokenStream {
                             tokens.append(&mut m);
                         }
                     }
+                    AlternativeStrings(v) => {
+                        for t in v.iter() {
+                            sugg.insert(format!("{}{}", if s.is_empty() { " " } else { "" }, t));
+                        }
+                    }
                     Seq(_s) => {}
                     RestOfStringValue => {
                         sugg.insert(String::new());
@@ -195,6 +201,20 @@ impl TokenStream {
                         *s = "";
                     }
                 }
+                AlternativeStrings(v) => {
+                    for lit in v.iter() {
+                        if lit.starts_with(*s) && lit.len() != s.len() {
+                            sugg.insert(lit[s.len()..].to_string());
+                            tokens.push((s, *t.inner()));
+                            return tokens;
+                        } else if s.starts_with(lit) {
+                            tokens.push((&s[..lit.len()], *t.inner()));
+                            *s = &s[lit.len()..];
+                        } else {
+                            return vec![];
+                        }
+                    }
+                }
                 Seq(_s) => {
                     return vec![];
                 }
@@ -225,7 +245,7 @@ impl TokenStream {
 
 /// `Token` wrapper that defines how many times a token is expected to be
 /// repeated
-#[derive(Debug, Copy, Clone)]
+#[derive(Clone, Copy, Debug)]
 pub enum TokenAdicity {
     ZeroOrOne(Token),
     ZeroOrMore(Token),
@@ -245,11 +265,12 @@ impl TokenAdicity {
 }
 
 /// A token encountered in the UI's command execution bar
-#[derive(Debug, Copy, Clone)]
+#[derive(Clone, Copy, Debug)]
 pub enum Token {
     Literal(&'static str),
     Filepath,
     Alternatives(&'static [TokenStream]),
+    AlternativeStrings(&'static [&'static str]),
     Seq(&'static [TokenAdicity]),
     AccountName,
     MailboxPath,
@@ -339,6 +360,11 @@ define_commands!([
                   tokens: &[One(Literal("search")), One(RestOfStringValue)],
                   parser: parser::search
                 },
+                { tags: ["clear-selection"],
+                  desc: "clear-selection",
+                  tokens: &[One(Literal("clear-selection"))],
+                  parser: parser::select
+                },
                 { tags: ["select"],
                   desc: "select <TERM>, selects envelopes matching with given term",
                   tokens: &[One(Literal("select")), One(RestOfStringValue)],
@@ -482,6 +508,18 @@ Alternatives(&[to_stream!(One(Literal("add-attachment")), One(Filepath)), to_str
                   tokens: &[One(Literal("manage-mailboxes"))],
                   parser: parser::manage_mailboxes
                 },
+                { tags: ["man"],
+                  desc: "read documentation",
+                  tokens: {
+                      #[cfg(feature = "cli-docs")]
+                      {
+                          &[One(Literal("man")), One(AlternativeStrings(crate::manpages::POSSIBLE_VALUES))]
+                      }
+                      #[cfg(not(feature = "cli-docs"))]
+                      { &[] }
+                  },
+                  parser: parser::view_manpage
+                },
                 { tags: ["manage-jobs"],
                   desc: "view and manage jobs",
                   tokens: &[One(Literal("manage-jobs"))],
@@ -502,7 +540,7 @@ Alternatives(&[to_stream!(One(Literal("add-attachment")), One(Filepath)), to_str
 /// Get command suggestions for input
 pub fn command_completion_suggestions(input: &str) -> Vec<String> {
     use crate::melib::ShellExpandTrait;
-    let mut sugg = Default::default();
+    let mut sugg: HashSet<String> = Default::default();
     for (_tags, _desc, tokens, _) in COMMAND_COMPLETION.iter() {
         let _m = tokens.matches(&mut &(*input), &mut sugg);
         if _m.is_empty() {
@@ -527,17 +565,15 @@ mod tests {
         let mut input = "sort".to_string();
         macro_rules! match_input {
             ($input:expr) => {{
-                let mut sugg = Default::default();
-                let mut vec = vec![];
+                let mut sugg: HashSet<String> = Default::default();
                 //print!("{}", $input);
                 for (_tags, _desc, tokens, _) in COMMAND_COMPLETION.iter() {
-                    //println!("{:?}, {:?}, {:?}", _tags, _desc, tokens);
-                    let m = tokens.matches(&mut $input.as_str(), &mut sugg);
-                    if !m.is_empty() {
-                        vec.push(tokens);
-                        //print!("{:?} ", desc);
-                        //println!(" result = {:#?}\n\n", m);
-                    }
+                    //    //println!("{:?}, {:?}, {:?}", _tags, _desc, tokens);
+                    let _ = tokens.matches(&mut $input.as_str(), &mut sugg);
+                    //    if !m.is_empty() {
+                    //        //print!("{:?} ", desc);
+                    //        //println!(" result = {:#?}\n\n", m);
+                    //    }
                 }
                 //println!("suggestions = {:#?}", sugg);
                 sugg.into_iter()
@@ -586,7 +622,7 @@ mod tests {
             match io::stdin().read_line(&mut input) {
                 Ok(_n) => {
                     println!("Input is {:?}", input.as_str().trim());
-                    let mut sugg = Default::default();
+                    let mut sugg: HashSet<String> = Default::default();
                     let mut vec = vec![];
                     //print!("{}", input);
                     for (_tags, _desc, tokens, _) in COMMAND_COMPLETION.iter() {
@@ -652,9 +688,15 @@ mod tests {
         assert_eq!(
             parse_command(b"set foo").unwrap_err().to_string(),
             BadValue {
-                inner: "Bad argument for `set`. Accepted arguments are [seen, unseen, plain, \
-                        threaded, compact, conversations]."
-                    .into(),
+                inner: "foo".into(),
+                suggestions: Some(&[
+                    "seen",
+                    "unseen",
+                    "plain",
+                    "threaded",
+                    "compact",
+                    "conversations"
+                ])
             }
             .to_string(),
         );
