@@ -24,22 +24,37 @@
 use super::*;
 use crate::command::{argcheck::*, error::*};
 
+const FLAG_SUGGESTIONS: &[&str] = &[
+    "passed",
+    "replied",
+    "seen or read",
+    "junk or trash or trashed",
+    "draft",
+    "flagged",
+];
+
 macro_rules! command_err {
-    (nom $b:expr, $input: expr, $msg:literal) => {{
+    (nom $b:expr, $input: expr, $msg:expr, $suggs:expr) => {{
         let evaluated: IResult<&'_ [u8], _> = { $b };
         match evaluated {
             Err(_) => {
-                let err = CommandError::BadValue { inner: $msg.into() };
+                let err = CommandError::BadValue {
+                    inner: $msg.into(),
+                    suggestions: $suggs,
+                };
                 return Ok(($input, Err(err)));
             }
             Ok(v) => v,
         }
     }};
-    ($b:expr, $input: expr, $msg:literal) => {{
+    ($b:expr, $input: expr, $msg:expr, $suggs:expr) => {{
         let evaluated = { $b };
         match evaluated {
             Err(_) => {
-                let err = CommandError::BadValue { inner: $msg.into() };
+                let err = CommandError::BadValue {
+                    inner: $msg.into(),
+                    suggestions: $suggs,
+                };
                 return Ok(($input, Err(err)));
             }
             Ok(v) => v,
@@ -101,6 +116,7 @@ pub fn listing_action(input: &[u8]) -> IResult<&[u8], Result<Action, CommandErro
         open_in_new_tab,
         export_mbox,
         _tag,
+        flag,
     ))(input)
 }
 
@@ -123,7 +139,7 @@ pub fn view(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
 }
 
 pub fn new_tab(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
-    alt((manage_mailboxes, manage_jobs, compose_action))(input)
+    alt((manage_mailboxes, manage_jobs, compose_action, view_manpage))(input)
 }
 
 pub fn parse_command(input: &[u8]) -> Result<Action, CommandError> {
@@ -154,6 +170,125 @@ pub fn parse_command(input: &[u8]) -> Result<Action, CommandError> {
     .and_then(|(_, v)| v)
 }
 
+/// Set/unset a flag.
+///
+/// # Example
+///
+/// ```
+/// # use meli::{melib::Flag, command::{Action,ListingAction, FlagAction, parser}};
+///
+/// let (rest, parsed) = parser::flag(b"flag set junk").unwrap();
+/// assert_eq!(rest, b"");
+/// assert!(
+///     matches!(
+///         parsed,
+///         Ok(Action::Listing(ListingAction::Flag(FlagAction::Set(
+///             Flag::TRASHED
+///         ))))
+///     ),
+///     "{:?}",
+///     parsed
+/// );
+///
+/// let (rest, parsed) = parser::flag(b"flag unset junk").unwrap();
+/// assert_eq!(rest, b"");
+/// assert!(
+///     matches!(
+///         parsed,
+///         Ok(Action::Listing(ListingAction::Flag(FlagAction::Unset(
+///             Flag::TRASHED
+///         ))))
+///     ),
+///     "{:?}",
+///     parsed
+/// );
+///
+/// let (rest, parsed) = parser::flag(b"flag set draft").unwrap();
+/// assert_eq!(rest, b"");
+/// assert!(
+///     matches!(
+///         parsed,
+///         Ok(Action::Listing(ListingAction::Flag(FlagAction::Set(
+///             Flag::DRAFT
+///         ))))
+///     ),
+///     "{:?}",
+///     parsed
+/// );
+///
+/// let (rest, parsed) = parser::flag(b"flag set xunk").unwrap();
+/// assert_eq!(rest, b"");
+/// assert_eq!(
+///     &parsed.unwrap_err().to_string(),
+///     "Bad value/argument: xunk is not a valid flag name. Possible values are: passed, replied, \
+///      seen or read, junk or trash or trashed, draft, flagged"
+/// );
+/// ```
+pub fn flag<'a>(input: &'a [u8]) -> IResult<&'a [u8], Result<Action, CommandError>> {
+    use melib::Flag;
+
+    fn parse_flag(s: &str) -> Option<Flag> {
+        match s {
+            o if o.eq_ignore_ascii_case("passed") => Some(Flag::PASSED),
+            o if o.eq_ignore_ascii_case("replied") => Some(Flag::REPLIED),
+            o if o.eq_ignore_ascii_case("seen") => Some(Flag::SEEN),
+            o if o.eq_ignore_ascii_case("read") => Some(Flag::SEEN),
+            o if o.eq_ignore_ascii_case("junk") => Some(Flag::TRASHED),
+            o if o.eq_ignore_ascii_case("trash") => Some(Flag::TRASHED),
+            o if o.eq_ignore_ascii_case("trashed") => Some(Flag::TRASHED),
+            o if o.eq_ignore_ascii_case("draft") => Some(Flag::DRAFT),
+            o if o.eq_ignore_ascii_case("flagged") => Some(Flag::FLAGGED),
+            _ => None,
+        }
+    }
+
+    preceded(
+        tag("flag"),
+        alt((
+            |input: &'a [u8]| -> IResult<&'a [u8], Result<Action, CommandError>> {
+                let mut check = arg_init! { min_arg:2, max_arg: 2, flag};
+                let (input, _) = tag("set")(input.trim())?;
+                arg_chk!(start check, input);
+                let (input, _) = is_a(" ")(input)?;
+                arg_chk!(inc check, input);
+                let (input, flag) = quoted_argument(input.trim())?;
+                arg_chk!(finish check, input);
+                let (input, _) = eof(input)?;
+                let Some(flag) = parse_flag(flag) else {
+                    return Ok((
+                        b"",
+                        Err(CommandError::BadValue {
+                            inner: format!("{flag} is not a valid flag name").into(),
+                            suggestions: Some(FLAG_SUGGESTIONS),
+                        }),
+                    ));
+                };
+                Ok((input, Ok(Listing(Flag(FlagAction::Set(flag))))))
+            },
+            |input: &'a [u8]| -> IResult<&'a [u8], Result<Action, CommandError>> {
+                let mut check = arg_init! { min_arg:2, max_arg: 2, flag};
+                let (input, _) = tag("unset")(input.trim())?;
+                arg_chk!(start check, input);
+                let (input, _) = is_a(" ")(input)?;
+                arg_chk!(inc check, input);
+                let (input, flag) = quoted_argument(input.trim())?;
+                arg_chk!(finish check, input);
+                let (input, _) = eof(input)?;
+                let Some(flag) = parse_flag(flag) else {
+                    return Ok((
+                        b"",
+                        Err(CommandError::BadValue {
+                            inner: format!("{flag} is not a valid flag name").into(),
+                            suggestions: Some(FLAG_SUGGESTIONS),
+                        }),
+                    ));
+                };
+                Ok((input, Ok(Listing(Flag(FlagAction::Unset(flag))))))
+            },
+        )),
+    )(input.trim())
+}
+
 pub fn set(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     fn toggle(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
         let mut check = arg_init! { min_arg:1, max_arg: 1, set};
@@ -173,9 +308,13 @@ pub fn set(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
         let (input, _) = is_a(" ")(input)?;
         arg_chk!(inc check, input);
         let (input, ret) = command_err!(nom
-                                   alt((map(tag("seen"), |_| Listing(SetSeen)), map(tag("unseen"), |_| Listing(SetUnseen))))(input),
+                                   alt((
+                                           map(tag("seen"), |_| Listing(SetSeen)),
+                                           map(tag("unseen"), |_| Listing(SetUnseen)
+                                   )))(input),
                                    input,
-                                   "Bad argument for `set`. Accepted arguments are [seen, unseen, plain, threaded, compact, conversations].");
+                                   String::from_utf8_lossy(input.trim()).to_string(),
+                                   Some(&["seen", "unseen", "plain", "threaded", "compact", "conversations"]));
         arg_chk!(finish check, input);
         let (input, _) = eof(input)?;
         Ok((input, Ok(ret)))
@@ -280,7 +419,8 @@ pub fn goto(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     let (input, nth) = command_err!(nom
                                usize_c(input),
                                input,
-                               "Argument must be an integer.");
+                               "Argument must be an integer.",
+                               None);
     arg_chk!(finish check, input);
     let (input, _) = eof(input)?;
     Ok((input, Ok(Action::ViewMailbox(nth))))
@@ -336,6 +476,26 @@ pub fn search(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     Ok((input, Ok(Listing(Search(String::from(string))))))
 }
 pub fn select(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
+    #[inline]
+    fn clear_selection(input: &[u8]) -> Option<IResult<&[u8], Result<Action, CommandError>>> {
+        if !input.trim().starts_with(b"clear-selection") {
+            return None;
+        }
+        #[inline]
+        fn inner(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
+            let mut check = arg_init! { min_arg:0, max_arg: 0, clear_selection};
+            let (input, _) = tag("clear-selection")(input.ltrim())?;
+            arg_chk!(start check, input);
+            arg_chk!(finish check, input);
+            let (input, _) = eof(input)?;
+            Ok((input, Ok(Listing(ListingAction::ClearSelection))))
+        }
+        Some(inner(input))
+    }
+    if let Some(retval) = clear_selection(input) {
+        return retval;
+    }
+
     let mut check = arg_init! { min_arg:1, max_arg: {u8::MAX}, select};
     let (input, _) = tag("select")(input.trim())?;
     arg_chk!(start check, input);
@@ -433,7 +593,8 @@ pub fn mailto(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     let (input, val) = command_err!(
         parser(val.as_bytes()),
         val.as_bytes(),
-        "Could not parse mailto value. If the value is valid, please report this bug."
+        "Could not parse mailto value. If the value is valid, please report this bug.",
+        None
     );
     Ok((input, Ok(Compose(Mailto(val)))))
 }
@@ -708,6 +869,19 @@ pub fn add_addresses_to_contacts(input: &[u8]) -> IResult<&[u8], Result<Action,
     let (input, _) = eof(input)?;
     Ok((input, Ok(View(AddAddressesToContacts))))
 }
+
+/// Set/unset a tag.
+///
+/// # Example
+///
+/// ```
+/// # use meli::command::{Action,ListingAction, TagAction, parser::_tag};
+///
+/// let (rest, parsed) = _tag(b"tag add newsletters").unwrap();
+/// println!("parsed is {:?}", parsed);
+/// assert_eq!(rest, b"");
+/// assert!(matches!(parsed, Ok(Action::Listing(ListingAction::Tag(TagAction::Add(ref tagname)))) if tagname == "newsletters"), "{:?}", parsed);
+/// ```
 pub fn _tag<'a>(input: &'a [u8]) -> IResult<&'a [u8], Result<Action, CommandError>> {
     preceded(
         tag("tag"),
@@ -721,7 +895,7 @@ pub fn _tag<'a>(input: &'a [u8]) -> IResult<&'a [u8], Result<Action, CommandErro
                 let (input, tag) = quoted_argument(input.trim())?;
                 arg_chk!(finish check, input);
                 let (input, _) = eof(input)?;
-                Ok((input, Ok(Listing(Tag(Add(tag.to_string()))))))
+                Ok((input, Ok(Listing(Tag(TagAction::Add(tag.to_string()))))))
             },
             |input: &'a [u8]| -> IResult<&'a [u8], Result<Action, CommandError>> {
                 let mut check = arg_init! { min_arg:2, max_arg: 2, tag};
@@ -732,11 +906,12 @@ pub fn _tag<'a>(input: &'a [u8]) -> IResult<&'a [u8], Result<Action, CommandErro
                 let (input, tag) = quoted_argument(input.trim())?;
                 arg_chk!(finish check, input);
                 let (input, _) = eof(input)?;
-                Ok((input, Ok(Listing(Tag(Remove(tag.to_string()))))))
+                Ok((input, Ok(Listing(Tag(TagAction::Remove(tag.to_string()))))))
             },
         )),
     )(input.trim())
 }
+
 pub fn print_account_setting(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     let mut check = arg_init! { min_arg:2, max_arg: 2, print};
     let (input, _) = tag("print")(input.trim())?;
@@ -796,7 +971,8 @@ pub fn toggle(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
             return Ok((
                 input,
                 Err(CommandError::BadValue {
-                    inner: "Valid toggle values are thread_snooze, mouse, sign, encrypt.".into(),
+                    inner: String::from_utf8_lossy(input).to_string().into(),
+                    suggestions: Some(&["thread_snooze", "mouse", "sign", "encrypt"]),
                 }),
             ));
         }
@@ -823,6 +999,39 @@ pub fn manage_jobs(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>>
     let (input, _) = eof(input)?;
     Ok((input, Ok(Tab(ManageJobs))))
 }
+
+#[cfg(feature = "cli-docs")]
+pub fn view_manpage(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
+    let mut check = arg_init! { min_arg:1, max_arg: 1, view_manpage };
+    let (input, _) = tag("man")(input.trim())?;
+    arg_chk!(start check, input);
+    let (input, _) = is_a(" ")(input)?;
+    arg_chk!(inc check, input);
+    let (input, manpage) = map_res(not_line_ending, std::str::from_utf8)(input.trim())?;
+    let (input, _) = eof(input)?;
+    arg_chk!(finish check, input);
+    match crate::manpages::parse_manpage(manpage) {
+        Ok(m) => Ok((input, Ok(Tab(Man(m))))),
+        Err(err) => Ok((
+            input,
+            Err(CommandError::BadValue {
+                inner: err.to_string().into(),
+                suggestions: Some(crate::manpages::POSSIBLE_VALUES),
+            }),
+        )),
+    }
+}
+
+#[cfg(not(feature = "cli-docs"))]
+pub fn view_manpage(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
+    Ok((
+        input,
+        Err(CommandError::Other {
+            inner: "this meli binary has not been compiled with the cli-docs feature".into(),
+        }),
+    ))
+}
+
 pub fn quit(input: &[u8]) -> IResult<&[u8], Result<Action, CommandError>> {
     let mut check = arg_init! { min_arg:0, max_arg: 0, quit};
     let (input, _) = tag("quit")(input.trim())?;
