@@ -29,9 +29,14 @@ use std::{
     collections::HashSet,
     io::Read,
     process::{Command, Stdio},
+    sync::Arc,
 };
 
-use melib::{backends::TagHash, search::Query, SortField, SortOrder, StderrLogger};
+use melib::{
+    backends::{MailboxHash, TagHash},
+    search::Query,
+    ShellExpandTrait, SortField, SortOrder, StderrLogger,
+};
 
 use crate::{
     conf::deserializers::non_empty_opt_string,
@@ -62,7 +67,7 @@ use std::{
 
 use indexmap::IndexMap;
 use melib::{
-    conf::{AccountSettings, MailboxConf, ToggleFlag},
+    conf::{AccountSettings, ActionFlag, MailboxConf, ToggleFlag},
     error::*,
 };
 use pager::PagerSettings;
@@ -113,7 +118,7 @@ macro_rules! mailbox_settings {
     }};
 }
 
-#[derive(Default, Debug, Clone, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct MailUIConf {
     #[serde(default)]
@@ -136,7 +141,7 @@ pub struct MailUIConf {
     pub pgp: PGPSettingsOverride,
 }
 
-#[derive(Debug, Default, Clone, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 #[serde(default)]
 pub struct FileMailboxConf {
     #[serde(flatten)]
@@ -158,14 +163,20 @@ impl FileMailboxConf {
 }
 
 use crate::conf::deserializers::extra_settings;
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct FileAccount {
     pub root_mailbox: String,
+    /// The mailbox that is the default to open / view for this account. Must be
+    /// a valid mailbox path.
+    ///
+    /// If not specified, the default is [`Self::root_mailbox`].
+    #[serde(default = "none", skip_serializing_if = "Option::is_none")]
+    pub default_mailbox: Option<String>,
     pub format: String,
     pub identity: String,
     #[serde(default)]
     pub extra_identities: Vec<String>,
-    #[serde(default = "none")]
+    #[serde(default = "none", skip_serializing_if = "Option::is_none")]
     pub display_name: Option<String>,
 
     #[serde(default = "false_val")]
@@ -180,14 +191,18 @@ pub struct FileAccount {
     pub order: (SortField, SortOrder),
     #[serde(default = "false_val")]
     pub manual_refresh: bool,
-    #[serde(default = "none")]
+    #[serde(default = "none", skip_serializing_if = "Option::is_none")]
     pub refresh_command: Option<String>,
     #[serde(flatten)]
     pub conf_override: MailUIConf,
     #[serde(flatten)]
-    #[serde(deserialize_with = "extra_settings")]
-    pub extra: IndexMap<String, String>, /* use custom deserializer to convert any given value
-                                          * (eg bool, number, etc) to string */
+    #[serde(
+        deserialize_with = "extra_settings",
+        skip_serializing_if = "IndexMap::is_empty"
+    )]
+    /// Use custom deserializer to convert any given value (eg `bool`, number,
+    /// etc) to `String`.
+    pub extra: IndexMap<String, String>,
 }
 
 impl FileAccount {
@@ -195,16 +210,12 @@ impl FileAccount {
         &self.mailboxes
     }
 
-    pub fn mailbox(&self) -> &str {
-        &self.root_mailbox
-    }
-
     pub fn search_backend(&self) -> &SearchBackend {
         &self.search_backend
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct FileSettings {
     pub accounts: IndexMap<String, FileAccount>,
@@ -227,9 +238,11 @@ pub struct FileSettings {
     pub log: LogSettings,
 }
 
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Clone, Debug, Default, Serialize)]
 pub struct AccountConf {
     pub account: AccountSettings,
+    pub default_mailbox: Option<MailboxHash>,
+    pub sent_mailbox: Option<MailboxHash>,
     pub conf: FileAccount,
     pub conf_override: MailUIConf,
     pub mailbox_confs: IndexMap<String, FileMailboxConf>,
@@ -279,8 +292,10 @@ impl From<FileAccount> for AccountConf {
         };
 
         let mailbox_confs = x.mailboxes.clone();
-        AccountConf {
+        Self {
             account: acc,
+            default_mailbox: None,
+            sent_mailbox: None,
             conf_override: x.conf_override.clone(),
             conf: x,
             mailbox_confs,
@@ -289,24 +304,23 @@ impl From<FileAccount> for AccountConf {
 }
 
 pub fn get_config_file() -> Result<PathBuf> {
-    let xdg_dirs = xdg::BaseDirectories::with_prefix("meli").map_err(|err| {
-        Error::new(format!(
-            "Could not detect XDG directories for user: {}",
-            err
-        ))
-        .set_source(Some(std::sync::Arc::new(Box::new(err))))
-    })?;
-    match env::var("MELI_CONFIG") {
-        Ok(path) => Ok(PathBuf::from(path)),
-        Err(_) => Ok(xdg_dirs
-            .place_config_file("config.toml")
-            .chain_err_summary(|| {
-                format!(
-                    "Cannot create configuration directory in {}",
-                    xdg_dirs.get_config_home().display()
-                )
-            })?),
+    if let Ok(path) = env::var("MELI_CONFIG") {
+        return Ok(PathBuf::from(path).expand());
     }
+    let xdg_dirs = xdg::BaseDirectories::with_prefix("meli").map_err(|err| {
+        Error::new("Could not detect XDG directories for user")
+            .set_source(Some(std::sync::Arc::new(Box::new(err))))
+            .set_kind(ErrorKind::NotSupported)
+    })?;
+    xdg_dirs
+        .place_config_file("config.toml")
+        .chain_err_summary(|| {
+            format!(
+                "Cannot create configuration directory in {}",
+                xdg_dirs.get_config_home().display()
+            )
+        })
+        .chain_err_kind(ErrorKind::OSError)
 }
 
 pub fn get_included_configs(conf_path: PathBuf) -> Result<Vec<PathBuf>> {
@@ -322,7 +336,7 @@ changequote(`"', `"')dnl
     let mut contents = String::new();
     while let Some((parent, p)) = stack.pop() {
         if !p.exists() || p.is_dir() {
-            return Err(format!(
+            return Err(Error::new(format!(
                 "Path {}{included}{in_parent} {msg}.",
                 p.display(),
                 included = if parent.is_some() {
@@ -340,8 +354,8 @@ changequote(`"', `"')dnl
                 } else {
                     "is a directory, not a text file"
                 }
-            )
-            .into());
+            ))
+            .set_kind(ErrorKind::Configuration));
         }
         contents.clear();
         let mut file = std::fs::File::open(&p)?;
@@ -354,11 +368,18 @@ changequote(`"', `"')dnl
             .spawn()
         {
             Ok(handle) => handle,
-            Err(error) => match error.kind() {
+            Err(err) => match err.kind() {
                 io::ErrorKind::NotFound => {
-                    return Err("`m4` executable not found. Please install.".into())
+                    return Err(Error::new(
+                        "`m4` executable not found in PATH. Please provide an m4 binary.",
+                    )
+                    .set_kind(ErrorKind::Platform))
+                }
+                _ => {
+                    return Err(Error::new("Could not process configuration with `m4`")
+                        .set_source(Some(Arc::new(err)))
+                        .set_kind(ErrorKind::OSError))
                 }
-                _ => return Err(error.into()),
             },
         };
 
@@ -393,7 +414,7 @@ define(`include', `builtin_include(substr($1,1,decr(decr(len($1)))))dnl')dnl
     file.read_to_string(&mut contents)?;
 
     let mut handle = Command::new("m4")
-        .current_dir(conf_path.parent().unwrap_or(Path::new("/")))
+        .current_dir(conf_path.parent().unwrap_or_else(|| Path::new("/")))
         .stdin(Stdio::piped())
         .stdout(Stdio::piped())
         .stderr(Stdio::piped())
@@ -407,12 +428,13 @@ define(`include', `builtin_include(substr($1,1,decr(decr(len($1)))))dnl')dnl
 }
 
 impl FileSettings {
-    pub fn new() -> Result<FileSettings> {
+    pub fn new() -> Result<Self> {
         let config_path = get_config_file()?;
         if !config_path.exists() {
             let path_string = config_path.display().to_string();
             if path_string.is_empty() {
-                return Err(Error::new("No configuration found."));
+                return Err(Error::new("Given configuration path is empty.")
+                    .set_kind(ErrorKind::Configuration));
             }
             #[cfg(not(test))]
             let ask = Ask {
@@ -424,33 +446,34 @@ impl FileSettings {
             #[cfg(not(test))]
             if ask.run() {
                 create_config_file(&config_path)?;
-                return Err(Error::new(
-                    "Edit the sample configuration and relaunch meli.",
-                ));
+                return Err(
+                    Error::new("Edit the sample configuration and relaunch meli.")
+                        .set_kind(ErrorKind::Configuration),
+                );
             }
             #[cfg(test)]
-            return Ok(FileSettings::default());
+            return Ok(Self::default());
             #[cfg(not(test))]
-            return Err(Error::new("No configuration file found."));
+            return Err(
+                Error::new("No configuration file found.").set_kind(ErrorKind::Configuration)
+            );
         }
 
-        FileSettings::validate(config_path, true, false)
+        Self::validate(config_path, true, false)
     }
 
     pub fn validate(path: PathBuf, interactive: bool, clear_extras: bool) -> Result<Self> {
         let s = pp::pp(&path)?;
-        let map: toml::map::Map<String, toml::value::Value> =
-            toml::from_str(&s).map_err(|err| {
-                Error::new(format!(
-                    "{}:\nConfig file is invalid TOML: {}",
-                    path.display(),
-                    err
-                ))
-            })?;
-        /*
-         * Check that a global composing option is set and return a user-friendly
-         * error message because the default serde one is confusing.
-         */
+        let map: toml::value::Table = toml::from_str(&s).map_err(|err| {
+            Error::new(format!(
+                "{}: Config file is invalid TOML; {}",
+                path.display(),
+                err
+            ))
+        })?;
+
+        // Check that a global composing option is set and return a user-friendly
+        // error message because the default serde one is confusing.
         if !map.contains_key("composing") {
             let err_msg = r#"You must set a global `composing` option. If you override `composing` in each account, you can use a dummy global like follows:
 
@@ -469,25 +492,24 @@ This is required so that you don't accidentally start meli and find out later th
                 };
                 if ask.run() {
                     let mut file = OpenOptions::new().append(true).open(&path)?;
-                    file.write_all("[composing]\nsend_mail = 'false'\n".as_bytes())
+                    file.write_all(b"[composing]\nsend_mail = 'false'\n")
                         .map_err(|err| {
                             Error::new(format!("Could not append to {}: {}", path.display(), err))
                         })?;
-                    return FileSettings::validate(path, interactive, clear_extras);
+                    return Self::validate(path, interactive, clear_extras);
                 }
             }
             return Err(Error::new(format!(
                 "{}\n\nEdit the {} and relaunch meli.",
                 if interactive { "" } else { err_msg },
                 path.display()
-            )));
-        }
-        let mut s: FileSettings = toml::from_str(&s).map_err(|err| {
-            Error::new(format!(
-                "{}:\nConfig file contains errors: {}",
-                path.display(),
-                err
             ))
+            .set_kind(ErrorKind::Configuration));
+        }
+        let mut s: Self = toml::from_str(&s).map_err(|err| {
+            Error::new(format!("{}: Config file contains errors", path.display()))
+                .set_source(Some(Arc::new(err)))
+                .set_kind(ErrorKind::Configuration)
         })?;
         let backends = melib::backends::Backends::new();
         let Themes {
@@ -513,10 +535,11 @@ This is required so that you don't accidentally start meli and find out later th
             }
         }
         match s.terminal.theme.as_str() {
-            "dark" | "light" => {}
+            themes::DARK | themes::LIGHT => {}
             t if s.terminal.themes.other_themes.contains_key(t) => {}
             t => {
-                return Err(Error::new(format!("Theme `{}` was not found.", t)));
+                return Err(Error::new(format!("Theme `{}` was not found.", t))
+                    .set_kind(ErrorKind::Configuration));
             }
         }
 
@@ -534,6 +557,7 @@ This is required so that you don't accidentally start meli and find out later th
                 mailboxes,
                 extra,
                 manual_refresh,
+                default_mailbox: _,
                 refresh_command: _,
                 search_backend: _,
                 conf_override: _,
@@ -557,12 +581,14 @@ This is required so that you don't accidentally start meli and find out later th
                     .collect(),
                 extra: extra.into_iter().collect(),
             };
+            s.validate_config()?;
             backends.validate_config(&lowercase_format, &mut s)?;
             if !s.extra.is_empty() {
                 return Err(Error::new(format!(
                     "Unrecognised configuration values: {:?}",
                     s.extra
-                )));
+                ))
+                .set_kind(ErrorKind::Configuration));
             }
             if clear_extras {
                 acc.extra.clear();
@@ -573,7 +599,7 @@ This is required so that you don't accidentally start meli and find out later th
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize)]
+#[derive(Clone, Debug, Default, Serialize)]
 pub struct Settings {
     pub accounts: IndexMap<String, AccountConf>,
     pub pager: PagerSettings,
@@ -586,17 +612,17 @@ pub struct Settings {
     pub terminal: TerminalSettings,
     pub log: LogSettings,
     #[serde(skip)]
-    _logger: StderrLogger,
+    pub _logger: StderrLogger,
 }
 
 impl Settings {
-    pub fn new() -> Result<Settings> {
+    pub fn new() -> Result<Self> {
         let fs = FileSettings::new()?;
         let mut s: IndexMap<String, AccountConf> = IndexMap::new();
 
         for (id, x) in fs.accounts {
             let mut ac = AccountConf::from(x);
-            ac.account.name = id.clone();
+            ac.account.name.clone_from(&id);
 
             s.insert(id, ac);
         }
@@ -607,7 +633,7 @@ impl Settings {
             _logger.change_log_dest(log_path.into());
         }
 
-        Ok(Settings {
+        Ok(Self {
             accounts: s,
             pager: fs.pager,
             listing: fs.listing,
@@ -622,7 +648,7 @@ impl Settings {
         })
     }
 
-    pub fn without_accounts() -> Result<Settings> {
+    pub fn without_accounts() -> Result<Self> {
         let fs = FileSettings::new()?;
         let mut _logger = StderrLogger::new(fs.log.maximum_level);
 
@@ -630,7 +656,7 @@ impl Settings {
             _logger.change_log_dest(log_path.into());
         }
 
-        Ok(Settings {
+        Ok(Self {
             accounts: IndexMap::new(),
             pager: fs.pager,
             listing: fs.listing,
@@ -646,7 +672,7 @@ impl Settings {
     }
 }
 
-#[derive(Copy, Debug, Clone, Hash, PartialEq, Eq, Default)]
+#[derive(Clone, Copy, Debug, Default, Eq, Hash, PartialEq)]
 pub enum IndexStyle {
     Plain,
     Threaded,
@@ -680,16 +706,29 @@ mod default_vals {
         None
     }
 
-    pub(in crate::conf) fn internal_value_false<T: std::convert::From<super::ToggleFlag>>() -> T {
-        super::ToggleFlag::InternalVal(false).into()
+    pub(in crate::conf) fn internal_value_false<T: std::convert::From<melib::conf::ToggleFlag>>(
+    ) -> T {
+        melib::conf::ToggleFlag::InternalVal(false).into()
     }
 
-    pub(in crate::conf) fn internal_value_true<T: std::convert::From<super::ToggleFlag>>() -> T {
-        super::ToggleFlag::InternalVal(true).into()
+    pub(in crate::conf) fn internal_value_true<T: std::convert::From<melib::conf::ToggleFlag>>() -> T
+    {
+        melib::conf::ToggleFlag::InternalVal(true).into()
     }
 
-    pub(in crate::conf) fn ask<T: std::convert::From<super::ToggleFlag>>() -> T {
-        super::ToggleFlag::Ask.into()
+    pub(in crate::conf) fn action_internal_value_false<T: std::convert::From<melib::ActionFlag>>(
+    ) -> T {
+        melib::conf::ActionFlag::InternalVal(false).into()
+    }
+
+    //pub(in crate::conf) fn action_internal_value_true<
+    //    T: std::convert::From<melib::conf::ActionFlag>,
+    //>() -> T {
+    //    melib::conf::ActionFlag::InternalVal(true).into()
+    //}
+
+    pub(in crate::conf) fn ask<T: std::convert::From<melib::conf::ActionFlag>>() -> T {
+        melib::conf::ActionFlag::Ask.into()
     }
 }
 
@@ -763,10 +802,10 @@ impl<'de> Deserialize<'de> for IndexStyle {
     {
         let s = <String>::deserialize(deserializer)?;
         match s.as_str() {
-            "Plain" | "plain" => Ok(IndexStyle::Plain),
-            "Threaded" | "threaded" => Ok(IndexStyle::Threaded),
-            "Compact" | "compact" => Ok(IndexStyle::Compact),
-            "Conversations" | "conversations" => Ok(IndexStyle::Conversations),
+            "Plain" | "plain" => Ok(Self::Plain),
+            "Threaded" | "threaded" => Ok(Self::Threaded),
+            "Compact" | "compact" => Ok(Self::Compact),
+            "Conversations" | "conversations" => Ok(Self::Conversations),
             _ => Err(de::Error::custom("invalid `index_style` value")),
         }
     }
@@ -778,15 +817,15 @@ impl Serialize for IndexStyle {
         S: Serializer,
     {
         match self {
-            IndexStyle::Plain => serializer.serialize_str("plain"),
-            IndexStyle::Threaded => serializer.serialize_str("threaded"),
-            IndexStyle::Compact => serializer.serialize_str("compact"),
-            IndexStyle::Conversations => serializer.serialize_str("conversations"),
+            Self::Plain => serializer.serialize_str("plain"),
+            Self::Threaded => serializer.serialize_str("threaded"),
+            Self::Compact => serializer.serialize_str("compact"),
+            Self::Conversations => serializer.serialize_str("conversations"),
         }
     }
 }
 
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
+#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
 pub enum SearchBackend {
     None,
     #[default]
@@ -807,15 +846,15 @@ impl<'de> Deserialize<'de> for SearchBackend {
                 if sqlite3.eq_ignore_ascii_case("sqlite3")
                     || sqlite3.eq_ignore_ascii_case("sqlite") =>
             {
-                Ok(SearchBackend::Sqlite3)
+                Ok(Self::Sqlite3)
             }
             none if none.eq_ignore_ascii_case("none")
                 || none.eq_ignore_ascii_case("nothing")
                 || none.is_empty() =>
             {
-                Ok(SearchBackend::None)
+                Ok(Self::None)
             }
-            auto if auto.eq_ignore_ascii_case("auto") => Ok(SearchBackend::Auto),
+            auto if auto.eq_ignore_ascii_case("auto") => Ok(Self::Auto),
             _ => Err(de::Error::custom("invalid `search_backend` value")),
         }
     }
@@ -828,9 +867,9 @@ impl Serialize for SearchBackend {
     {
         match self {
             #[cfg(feature = "sqlite3")]
-            SearchBackend::Sqlite3 => serializer.serialize_str("sqlite3"),
-            SearchBackend::None => serializer.serialize_str("none"),
-            SearchBackend::Auto => serializer.serialize_str("auto"),
+            Self::Sqlite3 => serializer.serialize_str("sqlite3"),
+            Self::None => serializer.serialize_str("none"),
+            Self::Auto => serializer.serialize_str("auto"),
         }
     }
 }
@@ -1001,7 +1040,7 @@ mod pp {
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct LogSettings {
     #[serde(default)]
@@ -1042,6 +1081,7 @@ mod dotaddressable {
     impl DotAddressable for melib::LogLevel {}
     impl DotAddressable for PathBuf {}
     impl DotAddressable for ToggleFlag {}
+    impl DotAddressable for ActionFlag {}
     impl DotAddressable for SearchBackend {}
     impl DotAddressable for melib::SpecialUsageMailbox {}
     impl<T: DotAddressable> DotAddressable for Option<T> {}
@@ -1263,7 +1303,7 @@ mod dotaddressable {
 
 #[test]
 fn test_config_parse() {
-    use std::{fmt::Write, fs, io::prelude::*, path::PathBuf};
+    use std::{fmt::Write, fs, path::PathBuf};
 
     struct ConfigFile {
         path: PathBuf,
@@ -1320,11 +1360,8 @@ send_mail = 'false'
 
     impl ConfigFile {
         fn new(content: &str) -> std::result::Result<Self, std::io::Error> {
-            let mut f = fs::File::open("/dev/urandom")?;
-            let mut buf = [0u8; 16];
-            f.read_exact(&mut buf)?;
             let mut filename = String::with_capacity(2 * 16);
-            for byte in &buf {
+            for byte in melib::utils::random::random_u64().to_be_bytes() {
                 write!(&mut filename, "{:02X}", byte).unwrap();
             }
             let mut path = std::env::temp_dir();
@@ -1334,7 +1371,7 @@ send_mail = 'false'
                 .append(true)
                 .open(&path)?;
             file.write_all(content.as_bytes())?;
-            Ok(ConfigFile { path, file })
+            Ok(Self { path, file })
         }
     }
 
@@ -1352,7 +1389,7 @@ send_mail = 'false'
     ));
     new_file
         .file
-        .write_all("[composing]\nsend_mail = 'false'\n".as_bytes())
+        .write_all(b"[composing]\nsend_mail = 'false'\n")
         .unwrap();
     let err = FileSettings::validate(new_file.path.clone(), false, true).unwrap_err();
     assert_eq!(
