@@ -47,11 +47,14 @@ use crate::{
     Context,
 };
 
+pub const LIGHT: &str = "light";
+pub const DARK: &str = "dark";
+
 #[inline(always)]
 pub fn value(context: &Context, key: &'static str) -> ThemeAttribute {
     let theme = match context.settings.terminal.theme.as_str() {
-        "light" => &context.settings.terminal.themes.light,
-        "dark" => &context.settings.terminal.themes.dark,
+        self::LIGHT => &context.settings.terminal.themes.light,
+        self::DARK => &context.settings.terminal.themes.dark,
         t => context
             .settings
             .terminal
@@ -66,8 +69,8 @@ pub fn value(context: &Context, key: &'static str) -> ThemeAttribute {
 #[inline(always)]
 pub fn fg_color(context: &Context, key: &'static str) -> Color {
     let theme = match context.settings.terminal.theme.as_str() {
-        "light" => &context.settings.terminal.themes.light,
-        "dark" => &context.settings.terminal.themes.dark,
+        self::LIGHT => &context.settings.terminal.themes.light,
+        self::DARK => &context.settings.terminal.themes.dark,
         t => context
             .settings
             .terminal
@@ -82,8 +85,8 @@ pub fn fg_color(context: &Context, key: &'static str) -> Color {
 #[inline(always)]
 pub fn bg_color(context: &Context, key: &'static str) -> Color {
     let theme = match context.settings.terminal.theme.as_str() {
-        "light" => &context.settings.terminal.themes.light,
-        "dark" => &context.settings.terminal.themes.dark,
+        self::LIGHT => &context.settings.terminal.themes.light,
+        self::DARK => &context.settings.terminal.themes.dark,
         t => context
             .settings
             .terminal
@@ -98,8 +101,8 @@ pub fn bg_color(context: &Context, key: &'static str) -> Color {
 #[inline(always)]
 pub fn attrs(context: &Context, key: &'static str) -> Attr {
     let theme = match context.settings.terminal.theme.as_str() {
-        "light" => &context.settings.terminal.themes.light,
-        "dark" => &context.settings.terminal.themes.dark,
+        self::LIGHT => &context.settings.terminal.themes.light,
+        self::DARK => &context.settings.terminal.themes.dark,
         t => context
             .settings
             .terminal
@@ -318,12 +321,13 @@ const DEFAULT_KEYS: &[&str] = &[
     "mail.listing.attachment_flag",
     "mail.listing.thread_snooze_flag",
     "mail.listing.tag_default",
+    "mail.listing.highlight_self",
     "pager.highlight_search",
     "pager.highlight_search_current",
 ];
 
 /// `ThemeAttributeInner` but with the links resolved.
-#[derive(Debug, PartialEq, Eq, Clone, Default, Copy, Serialize, Deserialize)]
+#[derive(Clone, Copy, Debug, Default, Deserialize, Eq, PartialEq, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct ThemeAttribute {
     pub fg: Color,
@@ -332,7 +336,7 @@ pub struct ThemeAttribute {
 }
 
 /// Holds {fore,back}ground color and terminal attribute values.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Clone, Debug, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct ThemeAttributeInner {
     #[serde(default)]
@@ -353,7 +357,7 @@ impl Default for ThemeAttributeInner {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub enum ColorField {
     // Like self, i.e. either Fg or Bg
     LikeSelf,
@@ -361,22 +365,22 @@ pub enum ColorField {
     Bg,
 }
 
-/// The field a ThemeValue::Link refers to.
+/// The field a `ThemeValue::Link` refers to.
 trait ThemeLink {
     type LinkType;
 }
 
-/// A color value that's a link can either refer to .fg or .bg field
+/// A color value that's a link can either refer to `.fg` or `.bg` field
 impl ThemeLink for Color {
     type LinkType = ColorField;
 }
 
-/// An attr value that's a link can only refer to an .attr field
+/// An `attr` value that's a link can only refer to an `.attr` field
 impl ThemeLink for Attr {
     type LinkType = ();
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 /// Holds either an actual value or refers to the key name of the attribute that
 /// holds the value.
 enum ThemeValue<T: ThemeLink> {
@@ -387,37 +391,37 @@ enum ThemeValue<T: ThemeLink> {
 
 impl From<&'static str> for ThemeValue<Color> {
     fn from(from: &'static str) -> Self {
-        ThemeValue::Link(from.into(), ColorField::LikeSelf)
+        Self::Link(from.into(), ColorField::LikeSelf)
     }
 }
 
 impl From<&'static str> for ThemeValue<Attr> {
     fn from(from: &'static str) -> Self {
-        ThemeValue::Link(from.into(), ())
+        Self::Link(from.into(), ())
     }
 }
 
 impl From<Color> for ThemeValue<Color> {
     fn from(from: Color) -> Self {
-        ThemeValue::Value(from)
+        Self::Value(from)
     }
 }
 
 impl From<Attr> for ThemeValue<Attr> {
     fn from(from: Attr) -> Self {
-        ThemeValue::Value(from)
+        Self::Value(from)
     }
 }
 
 impl Default for ThemeValue<Color> {
     fn default() -> Self {
-        ThemeValue::Value(Color::Default)
+        Self::Value(Color::Default)
     }
 }
 
 impl Default for ThemeValue<Attr> {
     fn default() -> Self {
-        ThemeValue::Value(Attr::DEFAULT)
+        Self::Value(Attr::DEFAULT)
     }
 }
 
@@ -428,11 +432,11 @@ impl<'de> Deserialize<'de> for ThemeValue<Attr> {
     {
         if let Ok(s) = <String>::deserialize(deserializer) {
             if let Some(stripped) = s.strip_prefix('$') {
-                Ok(ThemeValue::Alias(stripped.to_string().into()))
+                Ok(Self::Alias(stripped.to_string().into()))
             } else if let Ok(c) = Attr::from_string_de::<'de, D, String>(s.clone()) {
-                Ok(ThemeValue::Value(c))
+                Ok(Self::Value(c))
             } else {
-                Ok(ThemeValue::Link(s.into(), ()))
+                Ok(Self::Link(s.into(), ()))
             }
         } else {
             Err(de::Error::custom("invalid theme attribute value"))
@@ -446,9 +450,9 @@ impl Serialize for ThemeValue<Attr> {
         S: Serializer,
     {
         match self {
-            ThemeValue::Value(s) => s.serialize(serializer),
-            ThemeValue::Alias(s) => format!("${}", s).serialize(serializer),
-            ThemeValue::Link(s, ()) => serializer.serialize_str(s.as_ref()),
+            Self::Value(s) => s.serialize(serializer),
+            Self::Alias(s) => format!("${}", s).serialize(serializer),
+            Self::Link(s, ()) => serializer.serialize_str(s.as_ref()),
         }
     }
 }
@@ -459,15 +463,11 @@ impl Serialize for ThemeValue<Color> {
         S: Serializer,
     {
         match self {
-            ThemeValue::Value(s) => s.serialize(serializer),
-            ThemeValue::Alias(s) => format!("${}", s).serialize(serializer),
-            ThemeValue::Link(s, ColorField::LikeSelf) => serializer.serialize_str(s.as_ref()),
-            ThemeValue::Link(s, ColorField::Fg) => {
-                serializer.serialize_str(format!("{}.fg", s).as_ref())
-            }
-            ThemeValue::Link(s, ColorField::Bg) => {
-                serializer.serialize_str(format!("{}.bg", s).as_ref())
-            }
+            Self::Value(s) => s.serialize(serializer),
+            Self::Alias(s) => format!("${}", s).serialize(serializer),
+            Self::Link(s, ColorField::LikeSelf) => serializer.serialize_str(s.as_ref()),
+            Self::Link(s, ColorField::Fg) => serializer.serialize_str(format!("{}.fg", s).as_ref()),
+            Self::Link(s, ColorField::Bg) => serializer.serialize_str(format!("{}.bg", s).as_ref()),
         }
     }
 }
@@ -479,21 +479,21 @@ impl<'de> Deserialize<'de> for ThemeValue<Color> {
     {
         if let Ok(s) = <String>::deserialize(deserializer) {
             if let Some(stripped) = s.strip_prefix('$') {
-                Ok(ThemeValue::Alias(stripped.to_string().into()))
+                Ok(Self::Alias(stripped.to_string().into()))
             } else if let Ok(c) = Color::from_string_de::<'de, D>(s.clone()) {
-                Ok(ThemeValue::Value(c))
+                Ok(Self::Value(c))
             } else if s.ends_with(".fg") {
-                Ok(ThemeValue::Link(
+                Ok(Self::Link(
                     s[..s.len() - 3].to_string().into(),
                     ColorField::Fg,
                 ))
             } else if s.ends_with(".bg") {
-                Ok(ThemeValue::Link(
+                Ok(Self::Link(
                     s[..s.len() - 3].to_string().into(),
                     ColorField::Bg,
                 ))
             } else {
-                Ok(ThemeValue::Link(s.into(), ColorField::LikeSelf))
+                Ok(Self::Link(s.into(), ColorField::LikeSelf))
             }
         } else {
             Err(de::Error::custom("invalid theme color value"))
@@ -501,14 +501,14 @@ impl<'de> Deserialize<'de> for ThemeValue<Color> {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub struct Themes {
     pub light: Theme,
     pub dark: Theme,
     pub other_themes: IndexMap<String, Theme>,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub struct Theme {
     color_aliases: IndexMap<Cow<'static, str>, ThemeValue<Color>>,
     attr_aliases: IndexMap<Cow<'static, str>, ThemeValue<Attr>>,
@@ -533,7 +533,7 @@ mod regexp {
     #[derive(Clone)]
     pub struct RegexpWrapper(pub pcre2::bytes::Regex);
 
-    #[derive(Debug, Clone)]
+    #[derive(Clone, Debug)]
     pub(super) struct TextFormatterSetting {
         pub(super) regexp: RegexpWrapper,
         pub(super) fg: Option<ThemeValue<Color>>,
@@ -542,7 +542,7 @@ mod regexp {
         pub(super) priority: u8,
     }
 
-    #[derive(Debug, Clone)]
+    #[derive(Clone, Debug)]
     pub struct TextFormatter<'r> {
         pub regexp: &'r RegexpWrapper,
         pub tag: FormatTag,
@@ -569,7 +569,7 @@ mod regexp {
     impl Eq for RegexpWrapper {}
 
     impl PartialEq for RegexpWrapper {
-        fn eq(&self, other: &RegexpWrapper) -> bool {
+        fn eq(&self, other: &Self) -> bool {
             self.0.as_str().eq(other.0.as_str())
         }
     }
@@ -652,8 +652,8 @@ mod regexp {
         key: &'static str,
     ) -> SmallVec<[TextFormatter<'ctx>; 64]> {
         let theme = match context.settings.terminal.theme.as_str() {
-            "light" => &context.settings.terminal.themes.light,
-            "dark" => &context.settings.terminal.themes.dark,
+            self::LIGHT => &context.settings.terminal.themes.light,
+            self::DARK => &context.settings.terminal.themes.dark,
             t => context
                 .settings
                 .terminal
@@ -784,7 +784,7 @@ impl<'de> Deserialize<'de> for Themes {
             #[serde(flatten, default)]
             other_themes: IndexMap<String, ThemeOptions>,
         }
-        #[derive(Deserialize, Default)]
+        #[derive(Default, Deserialize)]
         struct ThemeOptions {
             #[serde(default)]
             color_aliases: IndexMap<Cow<'static, str>, ThemeValue<Color>>,
@@ -797,7 +797,7 @@ impl<'de> Deserialize<'de> for Themes {
             keys: IndexMap<Cow<'static, str>, ThemeAttributeInnerOptions>,
         }
         #[cfg(feature = "regexp")]
-        #[derive(Deserialize, Default)]
+        #[derive(Default, Deserialize)]
         struct RegexpOptions {
             #[serde(default = "false_val")]
             caseless: bool,
@@ -816,7 +816,7 @@ impl<'de> Deserialize<'de> for Themes {
             #[serde(flatten)]
             rest: ThemeAttributeInnerOptions,
         }
-        #[derive(Deserialize, Default)]
+        #[derive(Default, Deserialize)]
         #[serde(deny_unknown_fields)]
         struct ThemeAttributeInnerOptions {
             #[serde(default)]
@@ -827,7 +827,7 @@ impl<'de> Deserialize<'de> for Themes {
             attrs: Option<ThemeValue<Attr>>,
         }
 
-        let mut ret = Themes::default();
+        let mut ret = Self::default();
         let mut s = <ThemesOptions>::deserialize(deserializer)?;
         for tk in s.other_themes.keys() {
             ret.other_themes.insert(tk.clone(), ret.dark.clone());
@@ -1037,7 +1037,7 @@ impl Themes {
                 }
                 ThemeValue::Alias(ref ident) => {
                     if !theme.color_aliases.contains_key(ident.as_ref()) {
-                        Some((Some(key), "alias", "nonexistant color alias", ident))
+                        Some((Some(key), "alias", "nonexistent color alias", ident))
                     } else {
                         None
                     }
@@ -1054,7 +1054,7 @@ impl Themes {
                 }
                 ThemeValue::Alias(ref ident) => {
                     if !theme.attr_aliases.contains_key(ident.as_ref()) {
-                        Some((Some(key), "alias", "nonexistant color alias", ident))
+                        Some((Some(key), "alias", "nonexistent color alias", ident))
                     } else {
                         None
                     }
@@ -1070,7 +1070,7 @@ impl Themes {
                     }
                 } else if let ThemeValue::Alias(ref ident) = a.fg {
                     if !theme.color_aliases.contains_key(ident.as_ref()) {
-                        Some((Some(key), "fg alias", "nonexistant color alias", ident))
+                        Some((Some(key), "fg alias", "nonexistent color alias", ident))
                     } else {
                         None
                     }
@@ -1087,7 +1087,7 @@ impl Themes {
                     }
                 } else if let ThemeValue::Alias(ref ident) = a.bg {
                     if !theme.color_aliases.contains_key(ident.as_ref()) {
-                        Some((Some(key), "bg alias", "nonexistant color alias", ident))
+                        Some((Some(key), "bg alias", "nonexistent color alias", ident))
                     } else {
                         None
                     }
@@ -1107,7 +1107,7 @@ impl Themes {
                         Some((
                             Some(key),
                             "attrs alias",
-                            "nonexistant text attribute alias",
+                            "nonexistent text attribute alias",
                             ident,
                         ))
                     } else {
@@ -1145,7 +1145,7 @@ impl Themes {
                                     keys.push((
                                         Some(key),
                                         "fg alias",
-                                        "nonexistant color alias in `text_format_regexps`",
+                                        "nonexistent color alias in `text_format_regexps`",
                                         ident,
                                     ));
                                 }
@@ -1166,7 +1166,7 @@ impl Themes {
                                     keys.push((
                                         Some(key),
                                         "bg alias",
-                                        "nonexistant color alias in `text_format_regexps`",
+                                        "nonexistent color alias in `text_format_regexps`",
                                         ident,
                                     ));
                                 }
@@ -1187,7 +1187,7 @@ impl Themes {
                                     keys.push((
                                         Some(key),
                                         "attrs alias",
-                                        "nonexistant text attribute alias in `text_format_regexps`",
+                                        "nonexistent text attribute alias in `text_format_regexps`",
                                         ident,
                                     ));
                                 }
@@ -1217,10 +1217,10 @@ impl Themes {
     }
     pub fn validate(&self) -> Result<()> {
         let hash_set: HashSet<&'static str> = DEFAULT_KEYS.iter().copied().collect();
-        Themes::validate_keys("light", &self.light, &hash_set)?;
-        Themes::validate_keys("dark", &self.dark, &hash_set)?;
+        Self::validate_keys(self::LIGHT, &self.light, &hash_set)?;
+        Self::validate_keys(self::DARK, &self.dark, &hash_set)?;
         for (name, t) in self.other_themes.iter() {
-            Themes::validate_keys(name, t, &hash_set)?;
+            Self::validate_keys(name, t, &hash_set)?;
         }
         if let Err(err) = is_cyclic(&self.light) {
             return Err(Error::new(format!("light theme contains a cycle: {}", err)));
@@ -1238,8 +1238,8 @@ impl Themes {
 
     pub fn key_to_string(&self, key: &str, unlink: bool) -> String {
         let theme = match key {
-            "light" => &self.light,
-            "dark" => &self.dark,
+            self::LIGHT => &self.light,
+            self::DARK => &self.dark,
             t => self.other_themes.get(t).unwrap_or(&self.dark),
         };
         let mut ret = String::new();
@@ -1274,10 +1274,10 @@ impl Themes {
 impl std::fmt::Display for Themes {
     fn fmt(&self, fmt: &mut std::fmt::Formatter) -> std::fmt::Result {
         let mut ret = String::new();
-        ret.push_str(&self.key_to_string("dark", true));
+        ret.push_str(&self.key_to_string(self::DARK, true));
 
         ret.push_str("\n\n");
-        ret.push_str(&self.key_to_string("light", true));
+        ret.push_str(&self.key_to_string(self::LIGHT, true));
         for name in self.other_themes.keys() {
             ret.push_str("\n\n");
             ret.push_str(&self.key_to_string(name, true));
@@ -1288,7 +1288,7 @@ impl std::fmt::Display for Themes {
 
 impl Default for Themes {
     #[allow(clippy::needless_update)]
-    fn default() -> Themes {
+    fn default() -> Self {
         let mut light = IndexMap::default();
         let mut dark = IndexMap::default();
         let other_themes = IndexMap::default();
@@ -1731,10 +1731,19 @@ impl Default for Themes {
                 attrs: Attr::BOLD
             }
         );
+        add!(
+            "mail.listing.highlight_self",
+            light = {
+                fg: Color::BLUE,
+            },
+            dark = {
+                fg: Color::BLUE,
+            }
+        );
 
         add!("pager.highlight_search", light = { fg: Color::White, bg: Color::Byte(6) /* Teal */, attrs: Attr::BOLD }, dark = { fg: Color::White, bg: Color::Byte(6) /* Teal */, attrs: Attr::BOLD });
         add!("pager.highlight_search_current", light = { fg: Color::White, bg: Color::Byte(17) /* NavyBlue */, attrs: Attr::BOLD }, dark = { fg: Color::White, bg: Color::Byte(17) /* NavyBlue */, attrs: Attr::BOLD });
-        Themes {
+        Self {
             light: Theme {
                 keys: light,
                 attr_aliases: Default::default(),
@@ -1807,15 +1816,15 @@ impl Serialize for Themes {
             other_themes.insert(name.to_string(), new_map);
         }
 
-        other_themes.insert("light".to_string(), light);
-        other_themes.insert("dark".to_string(), dark);
+        other_themes.insert(self::LIGHT.to_string(), light);
+        other_themes.insert(self::DARK.to_string(), dark);
         other_themes.serialize(serializer)
     }
 }
 
 /* Check Theme linked values for cycles */
 fn is_cyclic(theme: &Theme) -> std::result::Result<(), String> {
-    #[derive(Hash, Copy, Clone, PartialEq, Eq)]
+    #[derive(Clone, Copy, Eq, Hash, PartialEq)]
     enum Course {
         Fg,
         Bg,
@@ -2085,7 +2094,7 @@ fn is_cyclic(theme: &Theme) -> std::result::Result<(), String> {
 fn test_theme_parsing() {
     /* MUST SUCCEED: default themes should be valid */
     let def = Themes::default();
-    assert!(def.validate().is_ok());
+    def.validate().unwrap();
     /* MUST SUCCEED: new user theme `hunter2`, theme `dark` has user
      * redefinitions */
     const TEST_STR: &str = r#"[dark]
@@ -2121,27 +2130,27 @@ fn test_theme_parsing() {
         ),
         Color::Byte(15), // White
     );
-    assert!(parsed.validate().is_ok());
+    parsed.validate().unwrap();
     /* MUST FAIL: theme `dark` contains a cycle */
     const HAS_CYCLE: &str = r#"[dark]
 "mail.listing.compact.even" = { fg = "mail.listing.compact.odd" }
 "mail.listing.compact.odd" = { fg = "mail.listing.compact.even" }
 "#;
     let parsed: Themes = toml::from_str(HAS_CYCLE).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
     /* MUST FAIL: theme `dark` contains an invalid key */
     const HAS_INVALID_KEYS: &str = r#"[dark]
 "asdfsafsa" = { fg = "Black" }
 "#;
     let parsed: std::result::Result<Themes, _> = toml::from_str(HAS_INVALID_KEYS);
-    assert!(parsed.is_err());
+    parsed.unwrap_err();
     /* MUST SUCCEED: alias $Jebediah resolves to a valid color */
     const TEST_ALIAS_STR: &str = r##"[dark]
 color_aliases= { "Jebediah" = "#b4da55" }
 "mail.listing.tag_default" = { fg = "$Jebediah" }
 "##;
     let parsed: Themes = toml::from_str(TEST_ALIAS_STR).unwrap();
-    assert!(parsed.validate().is_ok());
+    parsed.validate().unwrap();
     assert_eq!(
         unlink_fg(
             &parsed.dark,
@@ -2150,48 +2159,48 @@ color_aliases= { "Jebediah" = "#b4da55" }
         ),
         Color::Rgb(180, 218, 85)
     );
-    /* MUST FAIL: Mispell color alias $Jebediah as $Jebedia */
+    /* MUST FAIL: Misspell color alias $Jebediah as $Jebedia */
     const TEST_INVALID_ALIAS_STR: &str = r##"[dark]
 color_aliases= { "Jebediah" = "#b4da55" }
 "mail.listing.tag_default" = { fg = "$Jebedia" }
 "##;
     let parsed: Themes = toml::from_str(TEST_INVALID_ALIAS_STR).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
     /* MUST FAIL: Color alias $Jebediah is defined as itself */
     const TEST_CYCLIC_ALIAS_STR: &str = r#"[dark]
 color_aliases= { "Jebediah" = "$Jebediah" }
 "mail.listing.tag_default" = { fg = "$Jebediah" }
 "#;
     let parsed: Themes = toml::from_str(TEST_CYCLIC_ALIAS_STR).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
     /* MUST FAIL: Attr alias $Jebediah is defined as itself */
     const TEST_CYCLIC_ALIAS_ATTR_STR: &str = r#"[dark]
 attr_aliases= { "Jebediah" = "$Jebediah" }
 "mail.listing.tag_default" = { attrs = "$Jebediah" }
 "#;
     let parsed: Themes = toml::from_str(TEST_CYCLIC_ALIAS_ATTR_STR).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
     /* MUST FAIL: alias $Jebediah resolves to a cycle */
     const TEST_CYCLIC_ALIAS_STR_2: &str = r#"[dark]
 color_aliases= { "Jebediah" = "$JebediahJr", "JebediahJr" = "mail.listing.tag_default" }
 "mail.listing.tag_default" = { fg = "$Jebediah" }
 "#;
     let parsed: Themes = toml::from_str(TEST_CYCLIC_ALIAS_STR_2).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
     /* MUST SUCCEED: alias $Jebediah resolves to a key's field */
     const TEST_CYCLIC_ALIAS_STR_3: &str = r#"[dark]
 color_aliases= { "Jebediah" = "$JebediahJr", "JebediahJr" = "mail.listing.tag_default.bg" }
 "mail.listing.tag_default" = { fg = "$Jebediah", bg = "Black" }
 "#;
     let parsed: Themes = toml::from_str(TEST_CYCLIC_ALIAS_STR_3).unwrap();
-    assert!(parsed.validate().is_ok());
+    parsed.validate().unwrap();
     /* MUST FAIL: alias $Jebediah resolves to an invalid key */
     const TEST_INVALID_LINK_KEY_FIELD_STR: &str = r#"[dark]
 color_aliases= { "Jebediah" = "$JebediahJr", "JebediahJr" = "mail.listing.tag_default.attrs" }
 "mail.listing.tag_default" = { fg = "$Jebediah", bg = "Black" }
 "#;
     let parsed: Themes = toml::from_str(TEST_INVALID_LINK_KEY_FIELD_STR).unwrap();
-    assert!(parsed.validate().is_err());
+    parsed.validate().unwrap_err();
 }
 
 #[test]
