@@ -30,8 +30,7 @@ use melib::{
 
 use super::*;
 use crate::{
-    boundaries::*,
-    melib::text_processing::{TextProcessing, Truncate},
+    melib::text::{TextProcessing, Truncate},
     uuid::Uuid,
 };
 
