@@ -27,9 +27,10 @@ use std::{
     process::{Command, Stdio},
 };
 
+use indexmap::IndexSet;
 use melib::{
     email::attachment_types::ContentType, list_management, mailto::Mailto, parser::BytesExt,
-    utils::datetime, Card, Draft, HeaderName, SpecialUsageMailbox,
+    utils::datetime, Card, Draft, FlagOp, HeaderName, SpecialUsageMailbox,
 };
 use smallvec::SmallVec;
 
@@ -39,8 +40,6 @@ use crate::{accounts::JobRequest, jobs::JobId};
 mod utils;
 pub use utils::*;
 
-mod html;
-pub use html::*;
 mod thread;
 pub use thread::*;
 mod types;
@@ -51,6 +50,12 @@ use state::*;
 pub mod envelope;
 pub use envelope::EnvelopeView;
 
+pub mod filters;
+pub use filters::*;
+
+#[cfg(test)]
+mod tests;
+
 /// Contains an Envelope view, with sticky headers, a pager for the body, and
 /// subviews for more menus
 #[derive(Debug)]
@@ -90,7 +95,7 @@ impl MailView {
         initialize_now: bool,
         context: &mut Context,
     ) -> Self {
-        let mut ret = MailView {
+        let mut ret = Self {
             coordinates,
             dirty: true,
             contact_selector: None,
@@ -254,10 +259,28 @@ impl MailView {
                 )));
             return;
         }
+        // First retrieve user's identities, and remove them from the final address
+        // list.
+        let mut seen = {
+            let extra = account.settings.account().extra_identity_addresses();
+            let mut ret = IndexSet::with_capacity(extra.len() + 1);
+            ret.extend(extra);
+            ret.insert(account.settings.account().main_identity_address());
+            ret
+        };
         let envelope: EnvelopeRef = account.collection.get_env(coordinates.2);
 
-        let mut entries = Vec::new();
-        for addr in envelope.from().iter().chain(envelope.to().iter()) {
+        let mut entries: IndexMap<Card, (Card, String)> = IndexMap::default();
+        for addr in envelope
+            .from()
+            .iter()
+            .chain(envelope.to().iter())
+            .chain(envelope.cc().iter())
+        {
+            if seen.contains(addr) {
+                continue;
+            }
+            seen.insert(addr.clone());
             let mut new_card: Card = Card::new();
             new_card
                 .set_email(addr.get_email())
@@ -265,12 +288,12 @@ impl MailView {
             if let Some(display_name) = addr.get_display_name() {
                 new_card.set_name(display_name);
             }
-            entries.push((new_card, format!("{}", addr)));
+            entries.insert(new_card.clone(), (new_card, format!("{}", addr)));
         }
         drop(envelope);
         self.contact_selector = Some(Box::new(Selector::new(
             "select contacts to add",
-            entries,
+            entries.into_iter().map(|(_, v)| v).collect(),
             false,
             Some(Box::new(move |id: ComponentId, results: &[Card]| {
                 Some(UIEvent::FinishedUIDialog(id, Box::new(results.to_vec())))
@@ -310,49 +333,34 @@ impl Component for MailView {
             {
                 let account = &mut context.accounts[&coordinates.0];
                 if !account.collection.get_env(coordinates.2).is_seen() {
-                    let job = account.backend.write().unwrap().set_flags(
+                    if let Err(err) = account.set_flags(
                         coordinates.2.into(),
                         coordinates.1,
-                        smallvec::smallvec![(Ok(Flag::SEEN), true)],
-                    );
-                    match job {
-                        Ok(fut) => {
-                            let handle = account
-                                .main_loop_handler
-                                .job_executor
-                                .spawn_specialized("set_flags".into(), fut);
-                            account.insert_job(
-                                handle.job_id,
-                                JobRequest::SetFlags {
-                                    env_hashes: coordinates.2.into(),
-                                    handle,
-                                },
-                            );
-                        }
-                        Err(err) => {
-                            context.replies.push_back(UIEvent::StatusEvent(
-                                StatusEvent::DisplayMessage(format!(
-                                    "Could not set message as seen: {err}",
-                                )),
-                            ));
-                        }
+                        smallvec::smallvec![FlagOp::Set(Flag::SEEN)],
+                    ) {
+                        context.replies.push_back(UIEvent::StatusEvent(
+                            StatusEvent::DisplayMessage(format!(
+                                "Could not set message as seen: {err}",
+                            )),
+                        ));
                     }
                 }
             }
             env_view.draw(grid, area, context);
         } else if let MailViewState::Error { ref err } = self.state {
-            clear_area(grid, area, self.theme_default);
+            grid.clear_area(area, self.theme_default);
             context.dirty_areas.push_back(area);
-            context.replies.push_back(UIEvent::Notification(
-                Some("Failed to open e-mail".to_string()),
-                err.to_string(),
-                Some(NotificationType::Error(err.kind)),
-            ));
+            context.replies.push_back(UIEvent::Notification {
+                title: Some("Failed to open e-mail".into()),
+                source: None,
+                body: err.to_string().into(),
+                kind: Some(NotificationType::Error(err.kind)),
+            });
             log::error!("Failed to open envelope: {err}");
             self.init_futures(context);
             return;
         } else {
-            clear_area(grid, area, self.theme_default);
+            grid.clear_area(area, self.theme_default);
             context.dirty_areas.push_back(area);
             return;
         };
@@ -367,6 +375,7 @@ impl Component for MailView {
 
     fn process_event(&mut self, mut event: &mut UIEvent, context: &mut Context) -> bool {
         if let Some(ref mut s) = self.contact_selector {
+            // [ref:FIXME]: contact_selector should not forward navigation events and return true
             if s.process_event(event, context) {
                 return true;
             }
@@ -510,7 +519,7 @@ impl Component for MailView {
                                 move |id: ComponentId, result: &[Option<PendingReplyAction>]| {
                                     Some(UIEvent::FinishedUIDialog(
                                         id,
-                                        Box::new(result.get(0).cloned().flatten()),
+                                        Box::new(result.first().cloned().flatten()),
                                     ))
                                 },
                             )),
@@ -591,11 +600,12 @@ impl Component for MailView {
                                                 err
                                             );
                                             log::error!("{err_string}");
-                                            context.replies.push_back(UIEvent::Notification(
-                                                Some("Failed to open e-mail".to_string()),
-                                                err_string,
-                                                Some(NotificationType::Error(err.kind)),
-                                            ));
+                                            context.replies.push_back(UIEvent::Notification {
+                                                title: Some("Failed to open e-mail".into()),
+                                                source: None,
+                                                body: err_string.into(),
+                                                kind: Some(NotificationType::Error(err.kind)),
+                                            });
                                         }
                                     }
                                 }
@@ -675,7 +685,7 @@ impl Component for MailView {
                             /* autosend or open unsubscribe option */
                             let unsubscribe = actions.unsubscribe.as_ref().unwrap();
                             for option in unsubscribe.iter() {
-                                /* [ref:TODO]: Ask for confirmation before proceding with an action */
+                                /* [ref:TODO]: Ask for confirmation before proceeding with an action */
                                 match option {
                                     list_management::ListAction::Email(email) => {
                                         if let Ok(mailto) = Mailto::try_from(*email) {
@@ -685,7 +695,8 @@ impl Component for MailView {
                                                 context.accounts[&coordinates.0]
                                                     .settings
                                                     .account()
-                                                    .make_display_name(),
+                                                    .main_identity_address()
+                                                    .to_string(),
                                             );
                                             /* Manually drop stuff because borrowck doesn't do it
                                              * on its own */
@@ -735,12 +746,16 @@ impl Component for MailView {
                                             .spawn()
                                         {
                                             Ok(child) => {
-                                                context.children.push(child);
+                                                context
+                                                    .children
+                                                    .entry(url_launcher.to_string().into())
+                                                    .or_default()
+                                                    .push(child);
                                             }
                                             Err(err) => {
                                                 context.replies.push_back(UIEvent::StatusEvent(
                                                     StatusEvent::DisplayMessage(format!(
-                                                        "Couldn't launch {:?}: {}",
+                                                        "Couldn't launch {}: {}",
                                                         url_launcher, err
                                                     )),
                                                 ));
@@ -775,11 +790,15 @@ impl Component for MailView {
                                 .stdout(Stdio::piped())
                                 .spawn()
                             {
-                                Ok(child) => context.children.push(child),
+                                Ok(child) => context
+                                    .children
+                                    .entry(url_launcher.to_string().into())
+                                    .or_default()
+                                    .push(child),
                                 Err(err) => {
                                     context.replies.push_back(UIEvent::StatusEvent(
                                         StatusEvent::DisplayMessage(format!(
-                                            "Couldn't launch {:?}: {}",
+                                            "Couldn't launch {}: {}",
                                             url_launcher, err
                                         )),
                                     ));
@@ -799,6 +818,25 @@ impl Component for MailView {
                     .push_back(UIEvent::Action(Tab(New(Some(Box::new(new_tab))))));
                 return true;
             }
+            UIEvent::Input(ref key)
+                if context
+                    .settings
+                    .shortcuts
+                    .envelope_view
+                    .commands
+                    .iter()
+                    .any(|cmd| {
+                        if cmd.shortcut == *key {
+                            for cmd in &cmd.command {
+                                context.replies.push_back(UIEvent::Command(cmd.to_string()));
+                            }
+                            return true;
+                        }
+                        false
+                    }) =>
+            {
+                return true;
+            }
             _ => {}
         }
         false
