@@ -21,7 +21,7 @@
 use std::{collections::BTreeMap, io::Write};
 
 use super::*;
-use crate::melib::text_processing::TextProcessing;
+use crate::melib::text::TextProcessing;
 
 #[derive(Debug)]
 pub struct SVGScreenshotFilter {
@@ -37,7 +37,7 @@ impl std::fmt::Display for SVGScreenshotFilter {
 
 impl SVGScreenshotFilter {
     pub fn new() -> Self {
-        SVGScreenshotFilter {
+        Self {
             save_screenshot: false,
             id: ComponentId::default(),
         }
@@ -51,7 +51,7 @@ impl Default for SVGScreenshotFilter {
 }
 
 impl Component for SVGScreenshotFilter {
-    fn draw(&mut self, _grid: &mut CellBuffer, _area: Area, context: &mut Context) {
+    fn draw(&mut self, _grid: &mut CellBuffer, area: Area, context: &mut Context) {
         if !self.save_screenshot {
             return;
         }
@@ -76,7 +76,7 @@ impl Component for SVGScreenshotFilter {
          *   rectangle element
          * inserted along with the `use` elements
          *
-         * Each row is arbritarily set at 17px high, and each character cell is 8
+         * Each row is arbitrarily set at 17px high, and each character cell is 8
          * pixels wide. Rectangle cells each have one extra pixel (so 18px *
          * 9px) in their dimensions in order to cover the spacing between
          * cells.
@@ -89,7 +89,7 @@ impl Component for SVGScreenshotFilter {
 
         /* keep a map with used colors and write a stylesheet when we're done */
         let mut classes: BTreeMap<(u8, u8, u8), usize> = BTreeMap::new();
-        for (row_idx, row) in grid.bounds_iter(((0, 0), (width, height))).enumerate() {
+        for (row_idx, row) in grid.bounds_iter(area).enumerate() {
             text.clear();
             escaped_text.clear();
             /* Each row is a <g> group element, consisting of text elements */
@@ -98,7 +98,7 @@ impl Component for SVGScreenshotFilter {
              * - Whenever the foreground color changes, emit a text element with the
              *   accumulated
              * text in the specific foreground color.
-             * - Whenever the backgrund color changes, emit a rectangle element filled
+             * - Whenever the background color changes, emit a rectangle element filled
              *   with the
              * specific background color.
              */
@@ -424,26 +424,29 @@ impl Component for SVGScreenshotFilter {
             .unwrap()
             .write_all(&res)
             .unwrap();
-        context.replies.push_back(UIEvent::Notification(
-            Some("Screenshot saved".into()),
-            format!("Screenshot saved to {}", filename),
-            None,
-        ));
+        context.replies.push_back(UIEvent::Notification {
+            title: Some("Screenshot saved".into()),
+            source: None,
+            body: format!("Screenshot saved to {filename}").into(),
+            kind: None,
+        });
     }
+
     fn process_event(&mut self, event: &mut UIEvent, _context: &mut Context) -> bool {
-        if let UIEvent::Input(Key::F(6)) = event {
+        if matches!(
+            event,
+            UIEvent::Input(Key::F(6)) | UIEvent::CmdInput(Key::F(6))
+        ) {
             self.save_screenshot = true;
             true
-        } else if let UIEvent::CmdInput(Key::F(6)) = event {
-            self.save_screenshot = true;
-            true
-        } else if let UIEvent::EmbedInput((Key::F(6), _)) = event {
+        } else if matches!(event, UIEvent::EmbeddedInput((Key::F(6), _))) {
             self.save_screenshot = true;
             false
         } else {
             false
         }
     }
+
     fn set_dirty(&mut self, _value: bool) {}
 
     fn is_dirty(&self) -> bool {
