@@ -19,15 +19,16 @@
  * along with meli. If not, see <http://www.gnu.org/licenses/>.
  */
 
-use melib::text_processing::{LineBreakText, Truncate};
+use melib::text::{LineBreakText, Truncate};
 
 use super::*;
+use crate::{jobs::JoinHandle, terminal::embedded::EmbeddedGrid};
 
 /// A pager for text.
 /// `Pager` holds its own content in its own `CellBuffer` and when `draw` is
 /// called, it draws the current view of the text. It is responsible for
 /// scrolling etc.
-#[derive(Default, Debug, Clone)]
+#[derive(Debug, Default)]
 pub struct Pager {
     text: String,
     cursor: (usize, usize),
@@ -49,13 +50,40 @@ pub struct Pager {
     /// total height? Used to decide whether to accept `scroll_down` key
     /// events.
     rows_lt_height: bool,
-    filtered_content: Option<(String, Result<CellBuffer>)>,
+    filtered_content: Option<(String, Result<EmbeddedGrid>)>,
+    filter_job: Option<(String, JoinHandle<Result<EmbeddedGrid>>)>,
     text_lines: Vec<String>,
     line_breaker: LineBreakText,
     movement: Option<PageMovement>,
     id: ComponentId,
 }
 
+impl Clone for Pager {
+    fn clone(&self) -> Self {
+        Self {
+            filter_job: None,
+            text: self.text.clone(),
+            cursor: self.cursor,
+            reflow: self.reflow,
+            height: self.height,
+            width: self.width,
+            minimum_width: self.minimum_width,
+            search: self.search.clone(),
+            dirty: true,
+            colors: self.colors,
+            initialised: false,
+            show_scrollbar: self.show_scrollbar,
+            cols_lt_width: self.cols_lt_width,
+            rows_lt_height: self.rows_lt_height,
+            filtered_content: self.filtered_content.clone(),
+            text_lines: self.text_lines.clone(),
+            line_breaker: self.line_breaker.clone(),
+            movement: self.movement,
+            id: ComponentId::default(),
+        }
+    }
+}
+
 impl std::fmt::Display for Pager {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "pager")
@@ -123,31 +151,19 @@ impl Pager {
 
     pub fn from_string(
         text: String,
-        context: Option<&Context>,
+        context: &Context,
         cursor_pos: Option<usize>,
         mut width: Option<usize>,
         colors: ThemeAttribute,
     ) -> Self {
-        let pager_filter: Option<&String> = if let Some(context) = context {
-            context.settings.pager.filter.as_ref()
-        } else {
-            None
-        };
+        let pager_filter: Option<&String> = context.settings.pager.filter.as_ref();
 
-        let pager_minimum_width: usize = if let Some(context) = context {
-            context.settings.pager.minimum_width
-        } else {
-            0
-        };
+        let pager_minimum_width: usize = context.settings.pager.minimum_width;
 
-        let reflow: Reflow = if let Some(context) = context {
-            if context.settings.pager.split_long_lines {
-                Reflow::All
-            } else {
-                Reflow::No
-            }
-        } else {
+        let reflow: Reflow = if context.settings.pager.split_long_lines {
             Reflow::All
+        } else {
+            Reflow::No
         };
 
         if let Some(ref mut width) = width.as_mut() {
@@ -173,20 +189,20 @@ impl Pager {
         };
 
         if let Some(bin) = pager_filter {
-            ret.filter(bin);
+            ret.filter(bin, context);
         }
 
         ret
     }
 
-    pub fn filter(&mut self, cmd: &str) {
-        let _f = |bin: &str, text: &str| -> Result<CellBuffer> {
+    pub fn filter(&mut self, cmd: &str, context: &Context) {
+        async fn filter_fut(bin: String, text: String) -> Result<EmbeddedGrid> {
             use std::{
                 io::Write,
                 process::{Command, Stdio},
             };
             let mut filter_child = Command::new("sh")
-                .args(["-c", bin])
+                .args(["-c", &bin])
                 .stdin(Stdio::piped())
                 .stdout(Stdio::piped())
                 .spawn()
@@ -200,20 +216,20 @@ impl Pager {
                 .chain_err_summary(|| "Failed to wait on filter")?
                 .stdout;
             let mut dev_null = std::fs::File::open("/dev/null")?;
-            let mut embedded = crate::terminal::embed::EmbedGrid::new();
+            let mut embedded = EmbeddedGrid::new();
             embedded.set_terminal_size((80, 20));
 
             for b in out {
                 embedded.process_byte(&mut dev_null, b);
             }
-            Ok(std::mem::take(embedded.buffer_mut()))
-        };
-        let buf = _f(cmd, &self.text);
-        if let Some((width, height)) = buf.as_ref().ok().map(CellBuffer::size) {
-            self.width = width;
-            self.height = height;
+            Ok(embedded)
         }
-        self.filtered_content = Some((cmd.to_string(), buf));
+        let fut = Box::pin(filter_fut(cmd.to_string(), self.text.clone()));
+        let handle = context
+            .main_loop_handler
+            .job_executor
+            .spawn_blocking(format!("Running pager filter {cmd}").into(), fut);
+        self.filter_job = Some((cmd.to_string(), handle));
     }
 
     pub fn cursor_pos(&self) -> usize {
@@ -225,7 +241,7 @@ impl Pager {
     }
 
     pub fn initialise(&mut self, grid: &mut CellBuffer, area: Area, context: &mut Context) {
-        let mut width = width!(area);
+        let mut width = area.width();
         if width < self.minimum_width {
             width = self.minimum_width;
         }
@@ -243,7 +259,7 @@ impl Pager {
             self.height = self.text_lines.len();
             self.width = width;
             if let Some(ref mut search) = self.search {
-                use melib::text_processing::search::KMP;
+                use melib::text::search::KMP;
                 search.positions.clear();
                 for (y, l) in self.text_lines.iter().enumerate() {
                     search.positions.extend(
@@ -253,7 +269,7 @@ impl Pager {
                     );
                 }
                 if let Some(pos) = search.positions.get(search.cursor) {
-                    if self.cursor.1 > pos.0 || self.cursor.1 + height!(area) < pos.0 {
+                    if self.cursor.1 > pos.0 || self.cursor.1 + area.height() < pos.0 {
                         self.cursor.1 = pos.0.saturating_sub(3);
                     }
                 }
@@ -262,7 +278,7 @@ impl Pager {
                 grid,
                 area,
                 context,
-                self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * height!(area),
+                self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * area.height(),
             );
         }
         self.draw_page(grid, area, context);
@@ -277,23 +293,23 @@ impl Pager {
         _context: &mut Context,
         up_to: usize,
     ) {
-        if self.line_breaker.is_finished() {
+        if self.line_breaker.is_finished() || self.filtered_content.is_some() {
             return;
         }
         let old_lines_no = self.text_lines.len();
         if up_to == 0 {
             self.text_lines.extend(self.line_breaker.by_ref());
         } else {
-            if old_lines_no >= up_to + height!(area) {
+            if old_lines_no >= up_to + area.height() {
                 return;
             }
-            let new_lines_no = (up_to + height!(area)) - old_lines_no;
+            let new_lines_no = (up_to + area.height()) - old_lines_no;
             self.text_lines
                 .extend(self.line_breaker.by_ref().take(new_lines_no));
         };
         let new_lines_no = self.text_lines.len() - old_lines_no;
         if let Some(ref mut search) = self.search {
-            use melib::text_processing::search::KMP;
+            use melib::text::search::KMP;
             for (y, l) in self.text_lines.iter().enumerate().skip(old_lines_no) {
                 search.positions.extend(
                     l.kmp_search(&search.pattern)
@@ -309,30 +325,21 @@ impl Pager {
         if let Some((ref cmd, ref filtered_content)) = self.filtered_content {
             match filtered_content {
                 Ok(ref content) => {
-                    copy_area(
-                        grid,
-                        content,
+                    grid.copy_area(
+                        content.buffer(),
                         area,
-                        (
-                            (
-                                std::cmp::min(
-                                    self.cursor.0,
-                                    content.size().0.saturating_sub(width!(area)),
-                                ),
-                                std::cmp::min(
-                                    self.cursor.1,
-                                    content.size().1.saturating_sub(height!(area)),
-                                ),
-                            ),
-                            pos_dec(content.size(), (1, 1)),
-                        ),
+                        content
+                            .area()
+                            .skip_cols(self.cursor.0)
+                            .skip_rows(self.cursor.1)
+                            .take_cols(content.terminal_size().0.min(area.width()))
+                            .take_rows(content.terminal_size().1.min(area.height())),
                     );
                     context
                         .replies
                         .push_back(UIEvent::StatusEvent(StatusEvent::UpdateSubStatus(
                             cmd.to_string(),
                         )));
-                    return;
                 }
                 Err(ref err) => {
                     let mut cmd = cmd.as_str();
@@ -345,90 +352,99 @@ impl Pager {
                         ))));
                 }
             }
+            return;
         }
 
-        let (mut upper_left, bottom_right) = area;
-        for l in self
-            .text_lines
-            .iter()
-            .skip(self.cursor.1)
-            .take(height!(area))
         {
-            write_string_to_grid(
-                l,
-                grid,
-                self.colors.fg,
-                self.colors.bg,
-                Attr::DEFAULT,
-                (upper_left, bottom_right),
-                None,
-            );
-            if l.starts_with('⤷') {
-                grid[upper_left]
-                    .set_fg(crate::conf::value(context, "highlight").fg)
-                    .set_attrs(crate::conf::value(context, "highlight").attrs);
+            let mut area2 = area;
+            for l in self
+                .text_lines
+                .iter()
+                .skip(self.cursor.1)
+                .take(area2.height())
+            {
+                if area2.is_empty() {
+                    break;
+                }
+                grid.write_string(
+                    l,
+                    self.colors.fg,
+                    self.colors.bg,
+                    Attr::DEFAULT,
+                    area2,
+                    None,
+                    None,
+                );
+                if l.starts_with('⤷') {
+                    grid[area2.upper_left()]
+                        .set_fg(crate::conf::value(context, "highlight").fg)
+                        .set_attrs(crate::conf::value(context, "highlight").attrs);
+                }
+                area2 = area2.skip_rows(1);
+            }
+
+            if area2.height() <= 1 {
+                grid.clear_area(area2, crate::conf::value(context, "theme_default"));
             }
-            upper_left = pos_inc(upper_left, (0, 1));
         }
 
-        if get_y(upper_left) <= get_y(bottom_right) {
-            clear_area(
-                grid,
-                (upper_left, bottom_right),
-                crate::conf::value(context, "theme_default"),
-            );
-        }
-
-        let (upper_left, _bottom_right) = area;
-        #[cfg(feature = "regexp")]
         {
-            for text_formatter in crate::conf::text_format_regexps(context, "pager.envelope.body") {
-                let t = grid.insert_tag(text_formatter.tag);
-                for (i, l) in self
-                    .text_lines
-                    .iter()
-                    .skip(self.cursor.1)
-                    .enumerate()
-                    .take(height!(area) + 1)
+            #[cfg(feature = "regexp")]
+            {
+                let area3 = area;
+                for text_formatter in
+                    crate::conf::text_format_regexps(context, "pager.envelope.body")
                 {
-                    let i = i + get_y(upper_left);
-                    for (start, end) in text_formatter.regexp.find_iter(l) {
-                        let start = start + get_x(upper_left);
-                        let end = end + get_x(upper_left);
-                        grid.set_tag(t, (start, i), (end, i));
+                    let t = grid.insert_tag(text_formatter.tag);
+                    for (i, l) in self
+                        .text_lines
+                        .iter()
+                        .skip(self.cursor.1)
+                        .enumerate()
+                        .take(area3.height() + 1)
+                    {
+                        let i = i + area3.upper_left().1;
+                        for (start, end) in text_formatter.regexp.find_iter(l) {
+                            let start = start + area3.upper_left().0;
+                            let end = end + area3.upper_left().0;
+                            grid.set_tag(t, (start, i), (end, i));
+                        }
                     }
                 }
             }
-        }
-        let cursor_line = self.cursor.1;
-        if let Some(ref mut search) = self.search {
-            let results_attr = crate::conf::value(context, "pager.highlight_search");
-            let results_current_attr =
-                crate::conf::value(context, "pager.highlight_search_current");
-            search.cursor = std::cmp::min(search.positions.len().saturating_sub(1), search.cursor);
-            for (i, (y, x)) in search
-                .positions
-                .iter()
-                .enumerate()
-                .filter(|(_, (y, _))| *y >= cursor_line)
-                .take(height!(area) + 1)
-            {
-                let x = *x + get_x(upper_left);
-                let y = *y - cursor_line;
-                for c in grid.row_iter(
-                    x..x + search.pattern.grapheme_width(),
-                    y + get_y(upper_left),
-                ) {
-                    if i == search.cursor {
-                        grid[c]
-                            .set_fg(results_current_attr.fg)
-                            .set_bg(results_current_attr.bg)
-                            .set_attrs(results_current_attr.attrs);
+            if let Some(ref mut search) = self.search {
+                // Last row will be reserved for the "Results for ..." line.
+                let area3 = area.skip_rows_from_end(1);
+                let cursor_line = self.cursor.1;
+                let results_attr = crate::conf::value(context, "pager.highlight_search");
+                let results_current_attr =
+                    crate::conf::value(context, "pager.highlight_search_current");
+                search.cursor =
+                    std::cmp::min(search.positions.len().saturating_sub(1), search.cursor);
+                for (i, (y, offset)) in search
+                    .positions
+                    .iter()
+                    .enumerate()
+                    .filter(|(_, &(y, _))| y >= cursor_line && y < cursor_line + area3.height())
+                {
+                    let attr = if i == search.cursor {
+                        results_current_attr
                     } else {
+                        results_attr
+                    };
+
+                    let (y, x) = (*y, *offset);
+                    let row_iter = grid.row_iter(
+                        area3.nth_row(y - cursor_line),
+                        x..x + search.pattern.grapheme_width(),
+                        0,
+                    );
+                    debug_assert_eq!(row_iter.area().width(), search.pattern.grapheme_width());
+                    for c in row_iter {
                         grid[c]
-                            .set_fg(results_attr.fg)
-                            .set_bg(results_attr.bg)
-                            .set_attrs(results_attr.attrs);
+                            .set_fg(attr.fg)
+                            .set_bg(attr.bg)
+                            .set_attrs(attr.attrs);
                     }
                 }
             }
@@ -438,9 +454,6 @@ impl Pager {
 
 impl Component for Pager {
     fn draw(&mut self, grid: &mut CellBuffer, area: Area, context: &mut Context) {
-        if !is_valid_area!(area) {
-            return;
-        }
         if !self.is_dirty() {
             return;
         }
@@ -451,14 +464,42 @@ impl Component for Pager {
 
         self.dirty = false;
 
-        let height = height!(area);
+        if self.height == 0 || self.width == 0 {
+            grid.clear_area(area, crate::conf::value(context, "theme_default"));
+            return;
+        }
+
+        let (mut cols, mut rows) = (area.width(), area.height());
+        let (has_more_lines, (width, height)) = if self.filtered_content.is_some() {
+            (false, (self.width, self.height))
+        } else {
+            (
+                !self.line_breaker.is_finished(),
+                (self.line_breaker.width().unwrap_or(cols), self.height),
+            )
+        };
+        if cols < 2 || rows < 2 {
+            return;
+        }
+
+        if self.show_scrollbar && rows < height {
+            cols -= 1;
+            rows -= 1;
+        } else if self.search.is_some() {
+            rows -= 1;
+        }
+
+        if self.show_scrollbar && cols < width {
+            rows -= 1;
+        }
+
         if let Some(mvm) = self.movement.take() {
             match mvm {
                 PageMovement::Up(amount) => {
                     self.cursor.1 = self.cursor.1.saturating_sub(amount);
                 }
                 PageMovement::PageUp(multiplier) => {
-                    self.cursor.1 = self.cursor.1.saturating_sub(height * multiplier);
+                    self.cursor.1 = self.cursor.1.saturating_sub(rows * multiplier);
                 }
                 PageMovement::Down(amount) => {
                     if self.cursor.1 + amount + 1 < self.height {
@@ -470,22 +511,22 @@ impl Component for Pager {
                         grid,
                         area,
                         context,
-                        self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * height,
+                        self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * rows,
                     );
                 }
                 PageMovement::PageDown(multiplier) => {
-                    if self.cursor.1 + height * multiplier + 1 < self.height {
-                        self.cursor.1 += height * multiplier;
-                    } else if self.cursor.1 + height * multiplier > self.height {
+                    if self.cursor.1 + rows * multiplier + 1 < self.height {
+                        self.cursor.1 += rows * multiplier;
+                    } else if self.cursor.1 + rows * multiplier > self.height {
                         self.cursor.1 = self.height.saturating_sub(1);
                     } else {
-                        self.cursor.1 = (self.height / height) * height;
+                        self.cursor.1 = (self.height / rows) * rows;
                     }
                     self.draw_lines_up_to(
                         grid,
                         area,
                         context,
-                        self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * height,
+                        self.cursor.1 + Self::PAGES_AHEAD_TO_RENDER_NO * rows,
                     );
                 }
                 PageMovement::Right(amount) => {
@@ -508,73 +549,42 @@ impl Component for Pager {
             }
         }
 
-        if self.height == 0 || self.width == 0 {
-            return;
-        }
         if let Some(ref mut search) = self.search {
             if !search.positions.is_empty() {
                 if let Some(mvm) = search.movement.take() {
                     match mvm {
-                        PageMovement::Up(_) => {
+                        SearchMovement::First | SearchMovement::Last => {
+                            self.cursor.1 = search.positions[search.cursor].0;
+                        }
+                        SearchMovement::Previous => {
                             if self.cursor.1 > search.positions[search.cursor].0 {
                                 self.cursor.1 = search.positions[search.cursor].0;
                             }
                         }
-                        PageMovement::Down(_) => {
-                            if self.cursor.1 + height < search.positions[search.cursor].0 {
+                        SearchMovement::Next => {
+                            if self.cursor.1 + rows < search.positions[search.cursor].0 {
                                 self.cursor.1 = search.positions[search.cursor].0;
                             }
                         }
-                        _ => {}
                     }
                 }
             }
         }
 
-        clear_area(grid, area, crate::conf::value(context, "theme_default"));
-        let (mut cols, mut rows) = (width!(area), height!(area));
-        let (has_more_lines, (width, height)) = if self.filtered_content.is_some() {
-            (false, (self.width, self.height))
-        } else {
-            (
-                !self.line_breaker.is_finished(),
-                (self.line_breaker.width().unwrap_or(cols), self.height),
-            )
-        };
+        grid.clear_area(area, crate::conf::value(context, "theme_default"));
 
         self.cols_lt_width = cols + self.cursor.0 < width;
         self.rows_lt_height = rows + self.cursor.1 < height;
 
-        if cols < 2 || rows < 2 {
-            return;
-        }
-
-        if self.show_scrollbar && rows < height {
-            cols -= 1;
-            rows -= 1;
-        } else if self.search.is_some() {
-            rows -= 1;
-        }
-
-        if self.show_scrollbar && cols < width {
-            rows -= 1;
-        }
         self.cursor = (
             std::cmp::min(width.saturating_sub(cols), self.cursor.0),
             std::cmp::min(height.saturating_sub(rows), self.cursor.1),
         );
-        self.draw_page(
-            grid,
-            (upper_left!(area), pos_inc(upper_left!(area), (cols, rows))),
-            context,
-        );
+        self.draw_page(grid, area.take_cols(cols).take_rows(rows), context);
         if self.show_scrollbar && rows < height {
             ScrollBar::default().set_show_arrows(true).draw(
                 grid,
-                (
-                    set_x(upper_left!(area), get_x(bottom_right!(area))),
-                    bottom_right!(area),
-                ),
+                area.nth_col(area.width()),
                 context,
                 /* position */
                 self.cursor.1,
@@ -587,10 +597,7 @@ impl Component for Pager {
         if self.show_scrollbar && cols < width {
             ScrollBar::default().set_show_arrows(true).draw_horizontal(
                 grid,
-                (
-                    set_y(upper_left!(area), get_y(bottom_right!(area))),
-                    bottom_right!(area),
-                ),
+                area.nth_row(area.height()),
                 context,
                 self.cursor.0,
                 cols,
@@ -638,24 +645,27 @@ impl Component for Pager {
                 if !context.settings.terminal.use_color() {
                     attribute.attrs |= Attr::REVERSE;
                 }
-                let (_, y) = write_string_to_grid(
+                grid.write_string(
                     &status_message,
-                    grid,
                     attribute.fg,
                     attribute.bg,
                     attribute.attrs,
-                    (
-                        set_y(upper_left!(area), get_y(bottom_right!(area))),
-                        bottom_right!(area),
-                    ),
+                    area.nth_row(area.height().saturating_sub(1)),
+                    None,
                     None,
                 );
                 /* set search pattern to italics */
-                let start_x = get_x(upper_left!(area)) + RESULTS_STR.len();
-                for c in grid.row_iter(start_x..(start_x + search.pattern.grapheme_width()), y) {
+                let start_x = RESULTS_STR.len();
+                let row_iter = grid.row_iter(
+                    area.nth_row(area.height().saturating_sub(1)),
+                    start_x..(start_x + search.pattern.grapheme_width()),
+                    0,
+                );
+                debug_assert_eq!(row_iter.area().width(), search.pattern.grapheme_width());
+                for c in row_iter {
                     grid[c].set_attrs(attribute.attrs | Attr::ITALICS);
                 }
-            };
+            }
         }
         context.dirty_areas.push_back(area);
     }
@@ -766,34 +776,64 @@ impl Component for Pager {
                 return true;
             }
             UIEvent::Action(View(Filter(ref cmd))) => {
-                self.filter(cmd);
+                self.filter(cmd, context);
                 self.initialised = false;
                 self.dirty = true;
                 return true;
             }
+            UIEvent::StatusEvent(StatusEvent::JobFinished(ref job_id))
+                if self
+                    .filter_job
+                    .as_ref()
+                    .map(|(_, h)| h == job_id)
+                    .unwrap_or(false) =>
+            {
+                let (cmd, mut handle) = self.filter_job.take().unwrap();
+                match handle.chan.try_recv() {
+                    Err(_) => { /* search was canceled */ }
+                    Ok(None) => { /* something happened, perhaps a worker thread panicked */ }
+                    Ok(Some(buf)) => {
+                        if let Some((width, height)) =
+                            buf.as_ref().ok().map(EmbeddedGrid::terminal_size)
+                        {
+                            self.width = width;
+                            self.height = height;
+                        }
+                        self.filtered_content = Some((cmd, buf));
+                    }
+                }
+                self.initialised = false;
+                self.set_dirty(true);
+            }
             UIEvent::Action(Action::Listing(ListingAction::Search(pattern))) => {
                 self.search = Some(SearchPattern {
                     pattern: pattern.to_string(),
                     positions: vec![],
                     cursor: 0,
-                    movement: Some(PageMovement::Home),
+                    movement: Some(SearchMovement::First),
                 });
                 self.initialised = false;
                 self.dirty = true;
                 return true;
             }
-            UIEvent::Input(Key::Char('n')) if self.search.is_some() => {
+            UIEvent::Input(ref key)
+                if shortcut!(key == shortcuts[Shortcuts::GENERAL]["next_search_result"])
+                    && self.search.is_some() =>
+            {
                 if let Some(ref mut search) = self.search {
-                    search.movement = Some(PageMovement::Down(1));
+                    search.movement = Some(SearchMovement::Next);
                     search.cursor += 1;
                     self.initialised = false;
                     self.dirty = true;
                     return true;
                 }
             }
-            UIEvent::Input(Key::Char('N')) if self.search.is_some() => {
+            UIEvent::Input(ref key)
+                if shortcut!(key == shortcuts[Shortcuts::GENERAL]["previous_search_result"])
+                    && self.search.is_some() =>
+            {
                 if let Some(ref mut search) = self.search {
-                    search.movement = Some(PageMovement::Up(1));
+                    search.movement = Some(SearchMovement::Previous);
                     search.cursor = search.cursor.saturating_sub(1);
                     self.initialised = false;
                     self.dirty = true;
@@ -836,13 +876,26 @@ impl Component for Pager {
                         String::new(),
                     )));
             }
+            UIEvent::Input(ref key)
+                if context.settings.shortcuts.pager.commands.iter().any(|cmd| {
+                    if cmd.shortcut == *key {
+                        for cmd in &cmd.command {
+                            context.replies.push_back(UIEvent::Command(cmd.to_string()));
+                        }
+                        return true;
+                    }
+                    false
+                }) =>
+            {
+                return true;
+            }
             _ => {}
         }
         false
     }
 
     fn is_dirty(&self) -> bool {
-        self.dirty
+        self.dirty || !self.initialised
     }
 
     fn set_dirty(&mut self, value: bool) {
@@ -865,4 +918,12 @@ impl Component for Pager {
     fn id(&self) -> ComponentId {
         self.id
     }
+
+    fn kill(&mut self, uuid: ComponentId, context: &mut Context) {
+        if self.id != uuid {
+            return;
+        }
+
+        context.replies.push_back(UIEvent::Action(Tab(Kill(uuid))));
+    }
 }
