@@ -22,9 +22,9 @@
 use std::{borrow::Cow, collections::HashMap, time::Duration};
 
 use super::*;
-use crate::melib::text_processing::TextProcessing;
+use crate::melib::text::TextProcessing;
 
-#[derive(Debug, PartialEq, Eq, Default)]
+#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
 enum FormFocus {
     #[default]
     Fields,
@@ -49,8 +49,8 @@ impl std::fmt::Debug for Field {
 }
 
 impl Default for Field {
-    fn default() -> Field {
-        Field::Text(TextField::default())
+    fn default() -> Self {
+        Self::Text(TextField::default())
     }
 }
 
@@ -118,14 +118,14 @@ impl Component for Field {
                 text_field.draw(grid, area, context);
             }
             Self::Choice(_, _, _) => {
-                write_string_to_grid(
+                grid.write_string(
                     str,
-                    grid,
                     theme_attr.fg,
                     theme_attr.bg,
                     theme_attr.attrs,
                     area,
                     None,
+                    None,
                 );
             }
         }
@@ -195,7 +195,7 @@ impl std::fmt::Display for Field {
     }
 }
 
-#[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
+#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
 pub enum FormButtonActions {
     Accept,
     Reset,
@@ -254,8 +254,8 @@ impl<T: 'static + std::fmt::Debug + Copy + Default + Send + Sync> FormWidget<T>
         action: (Cow<'static, str>, T),
         cursor_up_shortcut: Key,
         cursor_down_shortcut: Key,
-    ) -> FormWidget<T> {
-        FormWidget {
+    ) -> Self {
+        Self {
             buttons: ButtonWidget::new(action),
             focus: FormFocus::Fields,
             hide_buttons: false,
@@ -348,108 +348,62 @@ impl<T: 'static + std::fmt::Debug + Copy + Default + Send + Sync> FormWidget<T>
 
 impl<T: 'static + std::fmt::Debug + Copy + Default + Send + Sync> Component for FormWidget<T> {
     fn draw(&mut self, grid: &mut CellBuffer, area: Area, context: &mut Context) {
-        let upper_left = upper_left!(area);
-        let bottom_right = bottom_right!(area);
-
         if self.is_dirty() {
             let theme_default = crate::conf::value(context, "theme_default");
-            clear_area(
-                grid,
-                (
-                    upper_left,
-                    set_y(bottom_right, get_y(upper_left) + self.layout.len()),
-                ),
-                theme_default,
-            );
+
+            grid.clear_area(area, theme_default);
             let label_attrs = crate::conf::value(context, "widgets.form.label");
+            let mut highlighted = crate::conf::value(context, "highlight");
+            if !context.settings.terminal.use_color() {
+                highlighted.attrs |= Attr::REVERSE;
+            }
 
             for (i, k) in self.layout.iter().enumerate().rev() {
+                let theme_attr = if i == self.cursor && self.focus == FormFocus::Fields {
+                    grid.change_theme(area.nth_row(i), highlighted);
+                    highlighted
+                } else {
+                    label_attrs
+                };
                 let v = self.fields.get_mut(k).unwrap();
                 /* Write field label */
-                write_string_to_grid(
+                grid.write_string(
                     k.as_ref(),
-                    grid,
-                    label_attrs.fg,
-                    label_attrs.bg,
-                    label_attrs.attrs,
-                    (
-                        pos_inc(upper_left, (1, i)),
-                        set_y(bottom_right, i + get_y(upper_left)),
-                    ),
+                    theme_attr.fg,
+                    theme_attr.bg,
+                    theme_attr.attrs,
+                    area.nth_row(i).skip_cols(1),
+                    None,
                     None,
                 );
                 /* draw field */
                 v.draw(
                     grid,
-                    (
-                        pos_inc(upper_left, (self.field_name_max_length + 3, i)),
-                        set_y(bottom_right, i + get_y(upper_left)),
-                    ),
+                    area.nth_row(i).skip_cols(self.field_name_max_length + 3),
                     context,
                 );
+                grid.change_theme(area.nth_row(i), theme_attr);
 
                 /* Highlight if necessary */
-                if i == self.cursor {
-                    if self.focus == FormFocus::Fields {
-                        let mut field_attrs =
-                            crate::conf::value(context, "widgets.form.highlighted");
-                        if !context.settings.terminal.use_color() {
-                            field_attrs.attrs |= Attr::REVERSE;
-                        }
-                        for row in grid.bounds_iter((
-                            pos_inc(upper_left, (0, i)),
-                            (get_x(bottom_right).saturating_sub(1), i + get_y(upper_left)),
-                        )) {
-                            for c in row {
-                                grid[c]
-                                    .set_fg(field_attrs.fg)
-                                    .set_bg(field_attrs.bg)
-                                    .set_attrs(field_attrs.attrs);
-                            }
-                        }
-                    }
-                    if self.focus == FormFocus::TextInput {
-                        v.draw_cursor(
-                            grid,
-                            (
-                                pos_inc(upper_left, (self.field_name_max_length + 3, i)),
-                                (get_x(bottom_right), i + get_y(upper_left)),
-                            ),
-                            (
-                                pos_inc(upper_left, (self.field_name_max_length + 3, i + 1)),
-                                bottom_right,
-                            ),
-                            context,
-                        );
-                    }
+                if i == self.cursor && self.focus == FormFocus::TextInput {
+                    v.draw_cursor(
+                        grid,
+                        area.nth_row(i).skip_cols(self.field_name_max_length + 3),
+                        area.nth_row(i + 1)
+                            .skip_cols(self.field_name_max_length + 3),
+                        context,
+                    );
                 }
             }
 
             let length = self.layout.len();
-            clear_area(
-                grid,
-                (
-                    pos_inc(upper_left, (0, length)),
-                    set_y(bottom_right, length + 2 + get_y(upper_left)),
-                ),
-                theme_default,
-            );
+
             if !self.hide_buttons {
-                self.buttons.draw(
-                    grid,
-                    (
-                        pos_inc(upper_left, (1, length + 3)),
-                        set_y(bottom_right, length + 3 + get_y(upper_left)),
-                    ),
-                    context,
-                );
+                self.buttons
+                    .draw(grid, area.skip_rows(length + 3).skip_cols(1), context);
             }
-            if length + 4 < height!(area) {
-                clear_area(
-                    grid,
-                    (pos_inc(upper_left, (0, length + 4)), bottom_right),
-                    theme_default,
-                );
+            if length + 4 < area.height() {
+                grid.clear_area(area.skip_rows(length + 3 + 1), theme_default);
             }
             self.set_dirty(false);
             context.dirty_areas.push_back(area);
@@ -604,8 +558,8 @@ impl<T> ButtonWidget<T>
 where
     T: 'static + std::fmt::Debug + Copy + Default + Send + Sync,
 {
-    pub fn new(init_val: (Cow<'static, str>, T)) -> ButtonWidget<T> {
-        ButtonWidget {
+    pub fn new(init_val: (Cow<'static, str>, T)) -> Self {
+        Self {
             layout: vec![init_val.0.clone()],
             buttons: vec![init_val].into_iter().collect(),
             result: None,
@@ -648,15 +602,13 @@ where
     fn draw(&mut self, grid: &mut CellBuffer, area: Area, context: &mut Context) {
         if self.dirty {
             let theme_default = crate::conf::value(context, "theme_default");
-            clear_area(grid, area, theme_default);
-            let upper_left = upper_left!(area);
+            grid.clear_area(area, theme_default);
 
             let mut len = 0;
             for (i, k) in self.layout.iter().enumerate() {
                 let cur_len = k.len();
-                write_string_to_grid(
+                grid.write_string(
                     k.as_ref(),
-                    grid,
                     theme_default.fg,
                     if i == self.cursor && self.focus {
                         crate::conf::value(context, "highlight").bg
@@ -664,10 +616,8 @@ where
                         theme_default.bg
                     },
                     Attr::BOLD,
-                    (
-                        pos_inc(upper_left, (len, 0)),
-                        pos_inc(upper_left, (cur_len + len, 0)),
-                    ),
+                    area.skip_cols(len),
+                    None,
                     None,
                 );
                 len += cur_len + 3;
@@ -715,7 +665,7 @@ where
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct AutoCompleteEntry {
     pub entry: String,
     pub description: String,
@@ -729,7 +679,7 @@ impl AutoCompleteEntry {
 
 impl From<String> for AutoCompleteEntry {
     fn from(val: String) -> Self {
-        AutoCompleteEntry {
+        Self {
             entry: val,
             description: String::new(),
         }
@@ -739,7 +689,7 @@ impl From<String> for AutoCompleteEntry {
 impl From<&(&str, &str, TokenStream)> for AutoCompleteEntry {
     fn from(val: &(&str, &str, TokenStream)) -> Self {
         let (a, b, _) = val;
-        AutoCompleteEntry {
+        Self {
             entry: a.to_string(),
             description: b.to_string(),
         }
@@ -749,17 +699,16 @@ impl From<&(&str, &str, TokenStream)> for AutoCompleteEntry {
 impl From<(String, String)> for AutoCompleteEntry {
     fn from(val: (String, String)) -> Self {
         let (a, b) = val;
-        AutoCompleteEntry {
+        Self {
             entry: a,
             description: b,
         }
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct AutoComplete {
     entries: Vec<AutoCompleteEntry>,
-    content: CellBuffer,
     cursor: usize,
 
     dirty: bool,
@@ -779,52 +728,73 @@ impl Component for AutoComplete {
         };
         self.dirty = false;
 
-        let (upper_left, bottom_right) = area;
-        let rows = get_y(bottom_right) - get_y(upper_left);
+        let rows = area.height();
         if rows == 0 {
             return;
         }
         let page_no = (self.cursor.saturating_sub(1)).wrapping_div(rows);
         let top_idx = page_no * rows;
-        let x_offset = if rows < self.entries.len() { 1 } else { 0 };
+        let x_offset = usize::from(rows < self.entries.len());
+
+        grid.clear_area(area, crate::conf::value(context, "theme_default"));
+        let width = self
+            .entries
+            .iter()
+            .map(|a| a.entry.grapheme_len() + a.description.grapheme_len() + 2)
+            .max()
+            .unwrap_or(0)
+            + 1;
+        // [ref:hardcoded_color_value]
+        let theme_attr = ThemeAttribute {
+            fg: Color::Byte(23),
+            bg: Color::Byte(7),
+            attrs: Attr::DEFAULT,
+        };
+        grid.change_theme(area, theme_attr);
+        for (i, e) in self.entries.iter().skip(top_idx).enumerate() {
+            let (x, _) = grid.write_string(
+                &e.entry,
+                Color::Byte(23),
+                Color::Byte(7),
+                Attr::DEFAULT,
+                area.nth_row(i).take_cols(width),
+                None,
+                None,
+            );
+            grid.write_string(
+                &e.description,
+                Color::Byte(23),
+                Color::Byte(7),
+                Attr::ITALICS,
+                area.nth_row(i).skip_cols(x + 2).take_cols(width),
+                None,
+                None,
+            );
+            grid.write_string(
+                "▒",
+                Color::Byte(23),
+                Color::Byte(7),
+                Attr::DEFAULT,
+                area.nth_row(i).skip_cols(width - 1),
+                None,
+                None,
+            );
+        }
 
-        let (width, height) = self.content.size();
-        clear_area(grid, area, crate::conf::value(context, "theme_default"));
-        copy_area(
-            grid,
-            &self.content,
-            (upper_left, pos_dec(bottom_right, (x_offset, 0))),
-            (
-                (0, top_idx),
-                (width.saturating_sub(1), height.saturating_sub(1)),
-            ),
-        );
         /* Highlight cursor */
         if self.cursor > 0 {
             let highlight = crate::conf::value(context, "highlight");
-            change_theme(
-                grid,
-                (
-                    pos_inc(upper_left, (0, (self.cursor - 1) % rows)),
-                    (
-                        std::cmp::min(
-                            get_x(upper_left) + width.saturating_sub(1),
-                            get_x(bottom_right),
-                        )
-                        .saturating_sub(x_offset),
-                        get_y(pos_inc(upper_left, (0, (self.cursor - 1) % rows))),
-                    ),
-                ),
+
+            grid.change_theme(
+                area.nth_row((self.cursor - 1) % rows)
+                    .skip_cols(width.saturating_sub(1 + x_offset)),
                 highlight,
             );
         }
         if rows < self.entries.len() {
             ScrollBar { show_arrows: false }.draw(
                 grid,
-                (
-                    set_y(pos_dec(bottom_right, (x_offset, 0)), get_y(upper_left)),
-                    pos_dec(bottom_right, (x_offset, 0)),
-                ),
+                area.take_rows(x_offset),
                 context,
                 self.cursor.saturating_sub(1),
                 rows,
@@ -833,12 +803,15 @@ impl Component for AutoComplete {
         }
         context.dirty_areas.push_back(area);
     }
+
     fn process_event(&mut self, _event: &mut UIEvent, _context: &mut Context) -> bool {
         false
     }
+
     fn is_dirty(&self) -> bool {
         self.dirty
     }
+
     fn set_dirty(&mut self, value: bool) {
         self.dirty = value;
     }
@@ -850,9 +823,8 @@ impl Component for AutoComplete {
 
 impl AutoComplete {
     pub fn new(entries: Vec<AutoCompleteEntry>) -> Box<Self> {
-        let mut ret = AutoComplete {
+        let mut ret = Self {
             entries: Vec::new(),
-            content: CellBuffer::default(),
             cursor: 0,
             dirty: true,
             id: ComponentId::default(),
@@ -866,48 +838,6 @@ impl AutoComplete {
             return false;
         }
 
-        // [ref:hardcoded_color_value]
-        let mut content = CellBuffer::new(
-            entries
-                .iter()
-                .map(|a| a.entry.grapheme_len() + a.description.grapheme_len() + 2)
-                .max()
-                .unwrap_or(0)
-                + 1,
-            entries.len(),
-            Cell::with_style(Color::Byte(23), Color::Byte(7), Attr::DEFAULT),
-        );
-        let width = content.cols();
-        for (i, e) in entries.iter().enumerate() {
-            let (x, _) = write_string_to_grid(
-                &e.entry,
-                &mut content,
-                Color::Byte(23),
-                Color::Byte(7),
-                Attr::DEFAULT,
-                ((0, i), (width - 1, i)),
-                None,
-            );
-            write_string_to_grid(
-                &e.description,
-                &mut content,
-                Color::Byte(23),
-                Color::Byte(7),
-                Attr::ITALICS,
-                ((x + 2, i), (width - 1, i)),
-                None,
-            );
-            write_string_to_grid(
-                "▒",
-                &mut content,
-                Color::Byte(23),
-                Color::Byte(7),
-                Attr::DEFAULT,
-                ((width - 1, i), (width - 1, i)),
-                None,
-            );
-        }
-        self.content = content;
         self.entries = entries;
         self.cursor = 0;
         true
@@ -940,7 +870,6 @@ impl AutoComplete {
         let ret = self.entries.remove(self.cursor - 1);
         self.entries.clear();
         self.cursor = 0;
-        self.content.empty();
         Some(ret.entry)
     }
 
@@ -949,7 +878,7 @@ impl AutoComplete {
     }
 }
 
-#[derive(Default, Copy, Clone)]
+#[derive(Clone, Copy, Default)]
 pub struct ScrollBar {
     pub show_arrows: bool,
 }
@@ -972,33 +901,33 @@ impl ScrollBar {
         if length == 0 {
             return;
         }
-        let height = height!(area);
+        let height = area.height();
         if height < 3 {
             return;
         }
         let theme_default = crate::conf::value(context, "theme_default");
-        clear_area(grid, area, theme_default);
+        grid.clear_area(area, theme_default);
 
         let visible_rows = std::cmp::min(visible_rows, length);
         let ascii_drawing = grid.ascii_drawing;
         let ratio: f64 = (height as f64) / (length as f64);
         let scrollbar_height = std::cmp::max((ratio * (visible_rows as f64)) as usize, 1);
         let scrollbar_offset = (ratio * (pos as f64)) as usize;
-        let (mut upper_left, bottom_right) = area;
+        let mut area2 = area;
 
         if self.show_arrows {
-            grid[upper_left]
+            grid[area2.upper_left()]
                 .set_ch(if ascii_drawing { '^' } else { '▀' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg);
-            upper_left = pos_inc(upper_left, (0, 1));
+            area2 = area2.skip_rows(1);
         }
 
-        upper_left = pos_inc(upper_left, (0, scrollbar_offset));
+        area2 = area2.skip_rows(scrollbar_offset);
         for _ in 0..scrollbar_height {
-            if get_y(upper_left) >= get_y(bottom_right) {
+            if area2.is_empty() {
                 break;
             }
-            grid[upper_left]
+            grid[area2.upper_left()]
                 .set_ch(if ascii_drawing { '#' } else { '█' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg)
                 .set_attrs(if !context.settings.terminal.use_color() {
@@ -1006,10 +935,10 @@ impl ScrollBar {
                 } else {
                     theme_default.attrs
                 });
-            upper_left = pos_inc(upper_left, (0, 1));
+            area2 = area2.skip_rows(1);
         }
         if self.show_arrows {
-            grid[bottom_right]
+            grid[area2.bottom_right()]
                 .set_ch(if ascii_drawing { 'v' } else { '▄' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg)
                 .set_bg(crate::conf::value(context, "theme_default").bg);
@@ -1028,33 +957,33 @@ impl ScrollBar {
         if length == 0 {
             return;
         }
-        let width = width!(area);
+        let width = area.width();
         if width < 3 {
             return;
         }
         let theme_default = crate::conf::value(context, "theme_default");
-        clear_area(grid, area, theme_default);
+        grid.clear_area(area, theme_default);
 
         let visible_cols = std::cmp::min(visible_cols, length);
         let ascii_drawing = grid.ascii_drawing;
         let ratio: f64 = (width as f64) / (length as f64);
         let scrollbar_width = std::cmp::min((ratio * (visible_cols as f64)) as usize, 1);
         let scrollbar_offset = (ratio * (pos as f64)) as usize;
-        let (mut upper_left, bottom_right) = area;
+        let mut area2 = area;
 
         if self.show_arrows {
-            grid[upper_left]
+            grid[area2.upper_left()]
                 .set_ch(if ascii_drawing { '<' } else { '▐' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg);
-            upper_left = pos_inc(upper_left, (1, 0));
+            area2 = area2.skip_cols(1);
         }
 
-        upper_left = pos_inc(upper_left, (scrollbar_offset, 0));
+        area2 = area2.skip_cols(scrollbar_offset);
         for _ in 0..scrollbar_width {
-            if get_x(upper_left) >= get_x(bottom_right) {
+            if area2.is_empty() {
                 break;
             }
-            grid[upper_left]
+            grid[area2.upper_left()]
                 .set_ch(if ascii_drawing { '#' } else { '█' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg)
                 .set_attrs(if !context.settings.terminal.use_color() {
@@ -1062,10 +991,10 @@ impl ScrollBar {
                 } else {
                     theme_default.attrs
                 });
-            upper_left = pos_inc(upper_left, (1, 0));
+            area2 = area2.skip_cols(1);
         }
         if self.show_arrows {
-            grid[bottom_right]
+            grid[area2.bottom_right()]
                 .set_ch(if ascii_drawing { '>' } else { '▌' })
                 .set_fg(crate::conf::value(context, "widgets.options.highlighted").bg)
                 .set_bg(crate::conf::value(context, "theme_default").bg);
@@ -1179,7 +1108,7 @@ impl ProgressSpinner {
             theme_attr.attrs |= Attr::REVERSE;
         }
         theme_attr.attrs |= Attr::BOLD;
-        ProgressSpinner {
+        Self {
             timer,
             stage: 0,
             kind: Ok(kind),
@@ -1244,19 +1173,19 @@ impl std::fmt::Display for ProgressSpinner {
 impl Component for ProgressSpinner {
     fn draw(&mut self, grid: &mut CellBuffer, area: Area, context: &mut Context) {
         if self.dirty {
-            clear_area(grid, area, self.theme_attr);
+            grid.clear_area(area, self.theme_attr);
             if self.active {
-                write_string_to_grid(
+                grid.write_string(
                     match self.kind.as_ref() {
                         Ok(kind) => (Self::KINDS[*kind].1)[self.stage],
                         Err(custom) => custom[self.stage].as_ref(),
                     },
-                    grid,
                     self.theme_attr.fg,
                     self.theme_attr.bg,
                     self.theme_attr.attrs,
                     area,
                     None,
+                    None,
                 );
             }
             context.dirty_areas.push_back(area);
