@@ -246,7 +246,7 @@ impl Backends {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub enum BackendEvent {
     Notice {
         description: String,
@@ -270,10 +270,10 @@ impl From<Error> for BackendEvent {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub enum RefreshEventKind {
     Update(EnvelopeHash, Box<Envelope>),
-    /// Rename(old_hash, new_hash)
+    /// `Rename(old_hash, new_hash)`
     Rename(EnvelopeHash, EnvelopeHash),
     Create(Box<Envelope>),
     Remove(EnvelopeHash),
@@ -290,7 +290,7 @@ pub enum RefreshEventKind {
     MailboxUnsubscribe(MailboxHash),
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub struct RefreshEvent {
     pub mailbox_hash: MailboxHash,
     pub account_hash: AccountHash,
@@ -320,7 +320,38 @@ impl Deref for BackendEventConsumer {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+pub enum FlagOp {
+    Set(Flag),
+    SetTag(String),
+    UnSet(Flag),
+    UnSetTag(String),
+}
+
+impl From<&FlagOp> for bool {
+    fn from(val: &FlagOp) -> Self {
+        matches!(val, FlagOp::Set(_) | FlagOp::SetTag(_))
+    }
+}
+
+impl FlagOp {
+    #[inline]
+    pub fn is_flag(&self) -> bool {
+        matches!(self, Self::Set(_) | Self::UnSet(_))
+    }
+
+    #[inline]
+    pub fn is_tag(&self) -> bool {
+        matches!(self, Self::SetTag(_) | Self::UnSetTag(_))
+    }
+
+    #[inline]
+    pub fn as_bool(&self) -> bool {
+        self.into()
+    }
+}
+
+#[derive(Clone, Debug)]
 pub struct MailBackendCapabilities {
     pub is_async: bool,
     pub is_remote: bool,
@@ -331,7 +362,7 @@ pub struct MailBackendCapabilities {
     pub extra_submission_headers: &'static [HeaderName],
 }
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Clone, Copy, Debug)]
 pub enum MailBackendExtensionStatus {
     Unsupported { comment: Option<&'static str> },
     Supported { comment: Option<&'static str> },
@@ -376,7 +407,7 @@ pub trait MailBackend: ::std::fmt::Debug + Send + Sync {
         &mut self,
         env_hashes: EnvelopeHashBatch,
         mailbox_hash: MailboxHash,
-        flags: SmallVec<[(std::result::Result<Flag, String>, bool); 8]>,
+        flags: SmallVec<[FlagOp; 8]>,
     ) -> ResultFuture<()>;
 
     fn delete_messages(
@@ -471,7 +502,7 @@ pub trait BackendOp: ::std::fmt::Debug + ::std::marker::Send {
     fn as_bytes(&mut self) -> ResultFuture<Vec<u8>>;
 }
 
-/// Wrapper for BackendOps that are to be set read-only.
+/// Wrapper for [`BackendOp`] that are to be set read-only.
 ///
 /// Warning: Backend implementations may still cause side-effects (for example
 /// IMAP can set the Seen flag when fetching an envelope)
@@ -493,7 +524,7 @@ impl BackendOp for ReadOnlyOp {
     }
 }
 
-#[derive(Default, Debug, Copy, Hash, Eq, Clone, Serialize, Deserialize, PartialEq)]
+#[derive(Clone, Copy, Debug, Default, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub enum SpecialUsageMailbox {
     #[default]
     Normal,
@@ -575,7 +606,7 @@ impl Clone for Mailbox {
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 pub struct MailboxPermissions {
     pub create_messages: bool,
     pub remove_messages: bool,
@@ -608,7 +639,7 @@ impl std::fmt::Display for MailboxPermissions {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct EnvelopeHashBatch {
     pub first: EnvelopeHash,
     pub rest: SmallVec<[EnvelopeHash; 64]>,
@@ -661,7 +692,7 @@ impl EnvelopeHashBatch {
     }
 }
 
-#[derive(Default, Clone)]
+#[derive(Clone, Default)]
 pub struct LazyCountSet {
     pub not_yet_seen: usize,
     pub set: BTreeSet<EnvelopeHash>,
@@ -730,6 +761,11 @@ impl LazyCountSet {
     pub fn remove(&mut self, env_hash: EnvelopeHash) -> bool {
         self.set.remove(&env_hash)
     }
+
+    #[inline(always)]
+    pub fn contains(&self, value: &EnvelopeHash) -> bool {
+        self.set.contains(value)
+    }
 }
 
 pub struct IsSubscribedFn(pub Box<dyn Fn(&str) -> bool + Send + Sync>);
