@@ -23,9 +23,9 @@
 //!
 //! # Parsing bytes into an `Envelope`
 //!
-//! An [`Envelope`](Envelope) represents the information you can get from an
-//! email's headers and body structure. Addresses in `To`, `From` fields etc
-//! are parsed into [`Address`](crate::email::Address) types.
+//! An [`Envelope`] represents the information you can get from an email's
+//! headers and body structure. Addresses in `To`, `From` fields etc are parsed
+//! into [`Address`] types.
 //!
 //! ```
 //! use melib::{Attachment, Envelope};
@@ -116,6 +116,7 @@ use imap_codec::imap_types::{
     fetch::{MacroOrMessageDataItemNames, MessageDataItemName, Section},
     flag::Flag as ImapCodecFlag,
 };
+use indexmap::IndexSet;
 use smallvec::SmallVec;
 
 use crate::{
@@ -222,7 +223,7 @@ impl Flag {
 /// the raw bytes that describe the email in `bytes`. Its body as an
 /// `melib::email::Attachment` can be parsed on demand
 /// with the `melib::email::Mail::body` method.
-#[derive(Debug, Clone, Default)]
+#[derive(Clone, Debug, Default)]
 pub struct Mail {
     pub envelope: Envelope,
     pub bytes: Vec<u8>,
@@ -266,7 +267,7 @@ crate::declare_u64_hash!(EnvelopeHash);
 ///
 ///To access the email attachments, you need to parse them from the raw email
 /// bytes into an `Attachment` object.
-#[derive(Clone, Serialize, Deserialize)]
+#[derive(Clone, Deserialize, Serialize)]
 pub struct Envelope {
     // ----- IMAP4rev1 -----
     pub date: String,
@@ -287,7 +288,7 @@ pub struct Envelope {
     pub thread: ThreadNodeHash,
     pub flags: Flag,
     pub has_attachments: bool,
-    pub tags: SmallVec<[TagHash; 8]>,
+    pub tags: IndexSet<TagHash>,
 }
 
 impl std::fmt::Debug for Envelope {
@@ -330,7 +331,7 @@ impl Envelope {
             thread: ThreadNodeHash::null(),
             has_attachments: false,
             flags: Flag::default(),
-            tags: SmallVec::new(),
+            tags: IndexSet::new(),
         }
     }
 
@@ -851,13 +852,23 @@ impl Envelope {
         self.has_attachments
     }
 
-    pub fn tags(&self) -> &SmallVec<[TagHash; 8]> {
+    pub fn tags(&self) -> &IndexSet<TagHash> {
         &self.tags
     }
 
-    pub fn tags_mut(&mut self) -> &mut SmallVec<[TagHash; 8]> {
+    pub fn tags_mut(&mut self) -> &mut IndexSet<TagHash> {
         &mut self.tags
     }
+
+    /// Returns `true` if `is_recipient` address is included in To:, Cc: or Bcc:
+    /// headers.
+    pub fn recipient_any(&self, is_recipient: &Address) -> bool {
+        self.to()
+            .iter()
+            .chain(self.cc().iter())
+            .chain(self.bcc().iter())
+            .any(|a| a == is_recipient)
+    }
 }
 
 impl Eq for Envelope {}
