@@ -20,7 +20,7 @@
  */
 
 //! Email addresses. Parsing functions are in
-//! [melib::email::parser::address](../parser/address/index.html).
+//! [`crate::email::parser::address`].
 use std::{
     collections::HashSet,
     convert::TryFrom,
@@ -28,15 +28,17 @@ use std::{
 };
 
 use super::*;
+#[cfg(test)]
+mod tests;
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Deserialize, Serialize)]
 pub struct GroupAddress {
     pub raw: Vec<u8>,
     pub display_name: StrBuilder,
     pub mailbox_list: Vec<Address>,
 }
 
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Deserialize, Serialize)]
 /**
  * Container for an address.
  *
@@ -100,7 +102,7 @@ impl PartialEq for MailboxAddress {
 /// assert_eq!(addr.get_display_name(), Some("Jörg Doe".to_string()));
 /// assert_eq!(addr.get_email(), "joerg@example.com".to_string());
 /// ```
-#[derive(Clone, Serialize, Deserialize)]
+#[derive(Clone, Deserialize, Serialize)]
 pub enum Address {
     Mailbox(MailboxAddress),
     Group(GroupAddress),
@@ -330,6 +332,56 @@ impl Address {
     pub fn display(&self) -> UIAddress {
         UIAddress(self)
     }
+
+    /// Returns a type that prints the names of addresses (or the e-mail part,
+    /// if the name is missing) suitably for UI display, e.g. without
+    /// quotes.
+    ///
+    /// ## Example
+    ///
+    /// ```rust
+    /// # use melib::email::Address;
+    /// let addr = Address::new(
+    ///     Some("Jörg T. Doe".to_string()),
+    ///     "joerg@example.com".to_string(),
+    /// );
+    /// assert_eq!(
+    ///     addr.to_string().as_str(),
+    ///     r#""Jörg T. Doe" <joerg@example.com>"#
+    /// );
+    /// assert_eq!(addr.display_name().to_string().as_str(), "Jörg T. Doe");
+    /// ```
+    pub fn display_name(&self) -> UINameAddress {
+        UINameAddress(self)
+    }
+
+    /// Formats a slice of `Address`es with their `Address::display` method,
+    /// separated by comma or `separator` if passed.
+    pub fn display_slice(slice: &[Self]) -> String {
+        match slice.first() {
+            None => String::new(),
+            Some(f) if slice.len() == 1 => f.display().to_string(),
+            Some(_) => slice
+                .iter()
+                .map(|a| a.display().to_string())
+                .collect::<Vec<String>>()
+                .join(", "),
+        }
+    }
+
+    /// Formats a slice of `Address`es with their `Address::display_name`
+    /// method, separated by comma or `separator` if passed.
+    pub fn display_name_slice(slice: &[Self]) -> String {
+        match slice.first() {
+            None => String::new(),
+            Some(f) if slice.len() == 1 => f.display_name().to_string(),
+            Some(_) => slice
+                .iter()
+                .map(|a| a.display_name().to_string())
+                .collect::<Vec<String>>()
+                .join(", "),
+        }
+    }
 }
 
 impl Eq for Address {}
@@ -417,7 +469,7 @@ impl TryFrom<&str> for Address {
     }
 }
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Clone, Copy, Debug)]
 #[repr(transparent)]
 pub struct UIAddress<'a>(&'a Address);
 
@@ -433,7 +485,32 @@ impl std::fmt::Display for UIAddress<'_> {
             Address::Mailbox(m) => write!(f, "{}", m.address_spec.display(&m.raw)),
             Address::Group(g) => {
                 let attachment_strings: Vec<String> =
-                    g.mailbox_list.iter().map(|a| format!("{}", a)).collect();
+                    g.mailbox_list.iter().map(|a| Self(a).to_string()).collect();
+                write!(
+                    f,
+                    "{}: {}",
+                    g.display_name.display(&g.raw),
+                    attachment_strings.join(", ")
+                )
+            }
+        }
+    }
+}
+
+#[derive(Clone, Copy, Debug)]
+#[repr(transparent)]
+pub struct UINameAddress<'a>(&'a Address);
+
+impl std::fmt::Display for UINameAddress<'_> {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        match self.0 {
+            Address::Mailbox(m) if m.display_name.length > 0 => {
+                write!(f, "{}", m.display_name.display(&m.raw),)
+            }
+            Address::Mailbox(m) => write!(f, "{}", m.address_spec.display(&m.raw)),
+            Address::Group(g) => {
+                let attachment_strings: Vec<String> =
+                    g.mailbox_list.iter().map(|a| Self(a).to_string()).collect();
                 write!(
                     f,
                     "{}: {}",
@@ -446,7 +523,7 @@ impl std::fmt::Display for UIAddress<'_> {
 }
 
 /// Helper struct to return slices from a struct field on demand.
-#[derive(Clone, Debug, Serialize, Deserialize, Default, PartialEq, Eq, Copy, Hash)]
+#[derive(Clone, Copy, Debug, Default, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub struct StrBuilder {
     pub offset: usize,
     pub length: usize,
@@ -475,7 +552,7 @@ impl StrBuilder {
 }
 
 /// `MessageID` is accessed through the `StrBuild` trait.
-#[derive(Clone, Serialize, Deserialize, Default)]
+#[derive(Clone, Default, Deserialize, Serialize)]
 pub struct MessageID(pub Vec<u8>, pub StrBuilder);
 
 impl StrBuild for MessageID {
@@ -542,7 +619,7 @@ impl std::fmt::Debug for MessageID {
     }
 }
 
-#[derive(Clone, Serialize, Deserialize)]
+#[derive(Clone, Deserialize, Serialize)]
 pub struct References {
     pub raw: Vec<u8>,
     pub refs: Vec<MessageID>,
@@ -584,32 +661,3 @@ macro_rules! make_address {
         })
     };
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    const M_ID: &str = "<20170825132332.6734-1@mail.ntua.gr>";
-    const M_LEN: usize = M_ID.len();
-
-    #[test]
-    fn test_message_id_strbuilder() {
-        let (_, val) = parser::address::msg_id(M_ID.as_bytes()).unwrap();
-        assert_eq!(
-            val,
-            MessageID(
-                M_ID.as_bytes().to_vec(),
-                StrBuilder {
-                    offset: 1,
-                    length: 35,
-                }
-            )
-        );
-    }
-
-    #[test]
-    fn test_message_id_comparisons() {
-        let (_, val) = parser::address::msg_id(M_ID.as_bytes()).unwrap();
-        assert_eq!(val, M_ID);
-        assert_eq!(val, M_ID[1..][..M_LEN - 2]);
-    }
-}
