@@ -59,7 +59,7 @@ impl<'att> From<Option<Charset>> for DecodeOptions<'att> {
     }
 }
 
-#[derive(Default, Debug, Clone, Serialize, Deserialize, PartialEq, Eq)]
+#[derive(Clone, Debug, Default, Deserialize, Eq, PartialEq, Serialize)]
 /// A struct analogous to [`Attachment`] which can have incomplete and partial
 /// content before being turned into an [`Attachment`] with
 /// [`AttachmentBuilder::build`].
@@ -119,6 +119,13 @@ impl AttachmentBuilder {
         builder
     }
 
+    pub fn new_plaintext(content: Vec<u8>) -> Self {
+        let mut ret = Self::new(&[]);
+        ret.set_raw(content).set_body_to_raw();
+
+        ret
+    }
+
     pub fn raw(&self) -> &[u8] {
         &self.raw
     }
@@ -134,7 +141,7 @@ impl AttachmentBuilder {
 
     /// Set body to the entire raw contents, use this if raw contains only data
     /// and no headers If raw contains data and headers pass it through
-    /// AttachmentBuilder::new().
+    /// [`AttachmentBuilder::new`].
     pub fn set_body_to_raw(&mut self) -> &mut Self {
         self.body = StrBuilder {
             offset: 0,
@@ -387,7 +394,7 @@ impl From<AttachmentBuilder> for Attachment {
 }
 
 /// Immutable attachment type.
-#[derive(Clone, Serialize, Deserialize, PartialEq, Hash, Eq)]
+#[derive(Clone, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub struct Attachment {
     pub content_type: ContentType,
     pub content_transfer_encoding: ContentTransferEncoding,
@@ -711,16 +718,15 @@ impl Attachment {
             ContentType::Text {
                 kind: Text::Plain, ..
             } => false,
+            ContentType::Multipart {
+                kind: MultipartType::Digest,
+                ..
+            } => false,
             ContentType::Multipart {
                 kind: MultipartType::Alternative,
                 ref parts,
                 ..
             } => parts.iter().all(Self::is_html),
-            ContentType::Multipart {
-                kind: MultipartType::Related,
-                ..
-            } => false,
-            ContentType::Multipart { ref parts, .. } => parts.iter().any(Self::is_html),
             _ => false,
         }
     }
