@@ -25,7 +25,7 @@ use std::{borrow::Cow, io, result, str, string, sync::Arc};
 
 pub type Result<T> = result::Result<T, Error>;
 
-#[derive(Debug, Copy, PartialEq, Eq, Clone)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum NetworkErrorKind {
     /// Unspecified
     None,
@@ -55,7 +55,7 @@ pub enum NetworkErrorKind {
     RequestBodyNotRewindable,
     /// Connection (not request) timeout.
     Timeout,
-    /// TooManyRedirects
+    /// Too many redirects
     TooManyRedirects,
     /// Invalid TLS connection
     InvalidTLSConnection,
@@ -316,11 +316,12 @@ impl From<isahc::http::StatusCode> for NetworkErrorKind {
     }
 }
 
-#[derive(Debug, Copy, PartialEq, Eq, Clone)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum ErrorKind {
     None,
     External,
+    LinkedLibrary(&'static str),
     Authentication,
     Configuration,
     /// Protocol error.
@@ -334,6 +335,7 @@ pub enum ErrorKind {
     Network(NetworkErrorKind),
     TimedOut,
     OSError,
+    Platform,
     NotImplemented,
     NotSupported,
     ValueError,
@@ -341,26 +343,26 @@ pub enum ErrorKind {
 
 impl std::fmt::Display for ErrorKind {
     fn fmt(&self, fmt: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(
-            fmt,
-            "{}",
-            match self {
-                Self::None => "None",
-                Self::External => "External",
-                Self::Authentication => "Authentication",
-                Self::Bug => "Bug, please report this!",
-                Self::Network(ref inner) => inner.as_str(),
-                Self::ProtocolError => "Protocol error",
-                Self::ProtocolNotSupported =>
-                    "Protocol is not supported. It could be the wrong type or version.",
-                Self::TimedOut => "Timed Out",
-                Self::OSError => "OS Error",
-                Self::Configuration => "Configuration",
-                Self::NotImplemented => "Not implemented",
-                Self::NotSupported => "Not supported",
-                Self::ValueError => "Invalid value",
-            }
-        )
+        match self {
+            Self::None => write!(fmt, "None"),
+            Self::External => write!(fmt, "External"),
+            Self::LinkedLibrary(ref name) => write!(fmt, "Linked library error: {name}"),
+            Self::Authentication => write!(fmt, "Authentication"),
+            Self::Bug => write!(fmt, "Bug, please report this!"),
+            Self::Network(ref inner) => write!(fmt, "{}", inner.as_str()),
+            Self::ProtocolError => write!(fmt, "Protocol error"),
+            Self::ProtocolNotSupported => write!(
+                fmt,
+                "Protocol is not supported. It could be the wrong type or version."
+            ),
+            Self::Platform => write!(fmt, "Platform/Runtime environment; OS or hardware"),
+            Self::TimedOut => write!(fmt, "Timed Out"),
+            Self::OSError => write!(fmt, "OS Error"),
+            Self::Configuration => write!(fmt, "Configuration"),
+            Self::NotImplemented => write!(fmt, "Not implemented"),
+            Self::NotSupported => write!(fmt, "Not supported"),
+            Self::ValueError => write!(fmt, "Invalid value"),
+        }
     }
 }
 
@@ -378,6 +380,7 @@ impl ErrorKind {
     is_variant! { is_bug, Bug }
     is_variant! { is_configuration, Configuration }
     is_variant! { is_external, External }
+    is_variant! { is_from_linked_library, LinkedLibrary(_) }
     is_variant! { is_network, Network(_) }
     is_variant! { is_network_down, Network(ref k) if k.is_network_down() }
     is_variant! { is_not_implemented, NotImplemented }
@@ -389,7 +392,14 @@ impl ErrorKind {
     is_variant! { is_value_error, ValueError }
 }
 
-#[derive(Debug, Clone)]
+#[macro_export]
+macro_rules! src_err_arc_wrap {
+    ($err:expr) => {{
+        (Box::new($err) as Box<dyn std::error::Error + Send + Sync + 'static>).into()
+    }};
+}
+
+#[derive(Clone, Debug)]
 pub struct Error {
     pub summary: Cow<'static, str>,
     pub details: Option<Cow<'static, str>>,
@@ -413,7 +423,10 @@ pub trait ResultIntoError<T> {
     where
         F: Fn() -> M,
         M: Into<Cow<'static, str>>;
-
+    fn chain_err_details<M, F>(self, msg_fn: F) -> Result<T>
+    where
+        F: Fn() -> M,
+        M: Into<Cow<'static, str>>;
     fn chain_err_kind(self, kind: ErrorKind) -> Result<T>;
 }
 
@@ -453,6 +466,15 @@ impl<T, I: Into<Error>> ResultIntoError<T> for std::result::Result<T, I> {
         self.map_err(|err| err.set_err_summary(msg_fn()))
     }
 
+    #[inline]
+    fn chain_err_details<M, F>(self, msg_fn: F) -> Result<T>
+    where
+        F: Fn() -> M,
+        M: Into<Cow<'static, str>>,
+    {
+        self.map_err(|err| err.set_err_details(msg_fn()))
+    }
+
     #[inline]
     fn chain_err_kind(self, kind: ErrorKind) -> Result<T> {
         self.map_err(|err| err.set_err_kind(kind))
@@ -514,7 +536,9 @@ impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "{}", self.summary)?;
         if let Some(details) = self.details.as_ref() {
-            write!(f, "\n{}", details)?;
+            if !details.trim().is_empty() {
+                write!(f, "\n{}", details)?;
+            }
         }
         if let Some(source) = self.source.as_ref() {
             write!(f, "\nCaused by: {}", source)?;
@@ -620,6 +644,13 @@ impl From<std::num::ParseIntError> for Error {
     }
 }
 
+impl From<std::fmt::Error> for Error {
+    #[inline]
+    fn from(kind: std::fmt::Error) -> Self {
+        Self::new(kind.to_string()).set_source(Some(Arc::new(kind)))
+    }
+}
+
 #[cfg(feature = "http")]
 impl From<&isahc::error::ErrorKind> for NetworkErrorKind {
     #[inline]
@@ -700,6 +731,24 @@ impl From<rusqlite::Error> for Error {
     }
 }
 
+#[cfg(any(feature = "maildir", feature = "mbox", feature = "notmuch"))]
+impl From<notify::Error> for Error {
+    #[inline]
+    fn from(err: notify::Error) -> Self {
+        let kind = match err.kind {
+            notify::ErrorKind::MaxFilesWatch
+            | notify::ErrorKind::WatchNotFound
+            | notify::ErrorKind::Generic(_) => ErrorKind::External,
+            notify::ErrorKind::Io(_) => ErrorKind::OSError,
+            notify::ErrorKind::PathNotFound => ErrorKind::Configuration,
+            notify::ErrorKind::InvalidConfig(_) => ErrorKind::Bug,
+        };
+        Self::new(err.to_string())
+            .set_source(Some(Arc::new(err)))
+            .set_kind(kind)
+    }
+}
+
 impl From<libloading::Error> for Error {
     #[inline]
     fn from(kind: libloading::Error) -> Self {
