@@ -49,7 +49,7 @@ use crate::{
         pgp::{DecryptionMetadata, Recipient},
         Address,
     },
-    error::{Error, ErrorKind, IntoError, Result, ResultIntoError},
+    error::{Error, ErrorKind, Result, ResultIntoError},
 };
 
 macro_rules! call {
@@ -76,7 +76,7 @@ pub mod bindings;
 use bindings::*;
 pub mod io;
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum GpgmeFlag {
     ///"auto-key-retrieve"
     AutoKeyRetrieve,
@@ -557,7 +557,7 @@ impl Context {
                     return Err(Error::new(
                         "Unspecified libgpgme error: gpgme_op_verify_result returned NULL.",
                     )
-                    .set_err_kind(ErrorKind::External));
+                    .set_kind(ErrorKind::External));
                 }
             }
             let io_state_lck = io_state.lock().unwrap();
@@ -914,7 +914,7 @@ impl Context {
                 return Err(Error::new(
                     "Unspecified libgpgme error: gpgme_op_decrypt_result returned NULL.",
                 )
-                .set_err_kind(ErrorKind::External));
+                .set_kind(ErrorKind::External));
             }
             let mut recipients = vec![];
             let is_mime;
@@ -1123,7 +1123,7 @@ impl Context {
                 return Err(Error::new(
                     "Unspecified libgpgme error: gpgme_op_encrypt_result returned NULL.",
                 )
-                .set_err_kind(ErrorKind::External));
+                .set_kind(ErrorKind::External));
             }
             /* Rewind cursor */
             cipher
