@@ -25,6 +25,7 @@ use imap_codec::imap_types::{
     sequence::SequenceSet,
     status::StatusDataItemName,
 };
+use indexmap::IndexSet;
 
 use super::*;
 
@@ -36,22 +37,22 @@ impl ImapConnection {
             return Ok(None);
         }
 
-        #[cfg(not(feature = "sqlite3"))]
-        let mut cache_handle = DefaultCache::get(self.uid_store.clone())?;
-        #[cfg(feature = "sqlite3")]
-        let mut cache_handle = Sqlite3Cache::get(self.uid_store.clone())?;
-        if cache_handle.mailbox_state(mailbox_hash)?.is_none() {
-            return Ok(None);
-        }
-
-        match self.sync_policy {
-            SyncPolicy::None => Ok(None),
-            SyncPolicy::Basic => self.resync_basic(cache_handle, mailbox_hash).await,
-            SyncPolicy::Condstore => self.resync_condstore(cache_handle, mailbox_hash).await,
-            SyncPolicy::CondstoreQresync => {
-                self.resync_condstoreqresync(cache_handle, mailbox_hash)
-                    .await
+        if let Some(mut cache_handle) = self.uid_store.cache_handle()? {
+            if cache_handle.mailbox_state(mailbox_hash)?.is_none() {
+                return Ok(None);
             }
+
+            match self.sync_policy {
+                SyncPolicy::None => Ok(None),
+                SyncPolicy::Basic => self.resync_basic(cache_handle, mailbox_hash).await,
+                SyncPolicy::Condstore => self.resync_condstore(cache_handle, mailbox_hash).await,
+                SyncPolicy::CondstoreQresync => {
+                    self.resync_condstoreqresync(cache_handle, mailbox_hash)
+                        .await
+                }
+            }
+        } else {
+            Ok(None)
         }
     }
 
@@ -60,14 +61,8 @@ impl ImapConnection {
         mailbox_hash: MailboxHash,
     ) -> Option<Result<Vec<EnvelopeHash>>> {
         debug!("load_cache {}", mailbox_hash);
-        #[cfg(not(feature = "sqlite3"))]
-        let mut cache_handle = match DefaultCache::get(self.uid_store.clone()) {
-            Ok(v) => v,
-            Err(err) => return Some(Err(err)),
-        };
-        #[cfg(feature = "sqlite3")]
-        let mut cache_handle = match Sqlite3Cache::get(self.uid_store.clone()) {
-            Ok(v) => v,
+        let mut cache_handle = match self.uid_store.cache_handle() {
+            Ok(v) => v?,
             Err(err) => return Some(Err(err)),
         };
         match cache_handle.mailbox_state(mailbox_hash) {
@@ -84,7 +79,7 @@ impl ImapConnection {
         }
     }
 
-    //rfc4549_Synchronization_Operations_for_Disconnected_IMAP4_Clients
+    /// RFC4549 Synchronization Operations for Disconnected IMAP4 Clients
     pub async fn resync_basic(
         &mut self,
         mut cache_handle: Box<dyn ImapCache>,
@@ -173,7 +168,7 @@ impl ImapConnection {
                 for f in keywords {
                     let hash = TagHash::from_bytes(f.as_bytes());
                     tag_lck.entry(hash).or_insert_with(|| f.to_string());
-                    env.tags_mut().push(hash);
+                    env.tags_mut().insert(hash);
                 }
             }
         }
@@ -266,7 +261,7 @@ impl ImapConnection {
                     != &tags
                         .iter()
                         .map(|t| TagHash::from_bytes(t.as_bytes()))
-                        .collect::<SmallVec<[TagHash; 8]>>()
+                        .collect::<IndexSet<TagHash>>()
             {
                 env_lck.entry(env_hash).and_modify(|entry| {
                     entry.inner.set_flags(flags);
@@ -316,8 +311,8 @@ impl ImapConnection {
         Ok(Some(payload.into_iter().map(|(_, env)| env).collect()))
     }
 
-    //rfc4549_Synchronization_Operations_for_Disconnected_IMAP4_Clients
-    //Section 6.1
+    /// RFC4549 Synchronization Operations for Disconnected IMAP4 Clients
+    /// > Section 6.1
     pub async fn resync_condstore(
         &mut self,
         mut cache_handle: Box<dyn ImapCache>,
@@ -466,7 +461,7 @@ impl ImapConnection {
                     for f in keywords {
                         let hash = TagHash::from_bytes(f.as_bytes());
                         tag_lck.entry(hash).or_insert_with(|| f.to_string());
-                        env.tags_mut().push(hash);
+                        env.tags_mut().insert(hash);
                     }
                 }
             }
@@ -556,7 +551,7 @@ impl ImapConnection {
                         != &tags
                             .iter()
                             .map(|t| TagHash::from_bytes(t.as_bytes()))
-                            .collect::<SmallVec<[TagHash; 8]>>()
+                            .collect::<IndexSet<TagHash>>()
                 {
                     env_lck.entry(env_hash).and_modify(|entry| {
                         entry.inner.set_flags(flags);
@@ -626,8 +621,8 @@ impl ImapConnection {
         Ok(Some(payload.into_iter().map(|(_, env)| env).collect()))
     }
 
-    //rfc7162_Quick Flag Changes Resynchronization (CONDSTORE)_and Quick Mailbox
-    // Resynchronization (QRESYNC)
+    /// RFC7162 Quick Flag Changes Resynchronization (CONDSTORE) and Quick
+    /// Mailbox Resynchronization (QRESYNC)
     pub async fn resync_condstoreqresync(
         &mut self,
         _cache_handle: Box<dyn ImapCache>,
