@@ -19,11 +19,15 @@
  * along with meli. If not, see <http://www.gnu.org/licenses/>.
  */
 
-use super::protocol_parser::{ImapLineSplit, ImapResponse, RequiredResponses, SelectResponse};
 use crate::{
     backends::{BackendEvent, MailboxHash, RefreshEvent},
     email::parser::BytesExt,
     error::*,
+    imap::{
+        protocol_parser::{self, ImapLineSplit, ImapResponse, RequiredResponses, SelectResponse},
+        Capabilities, ImapServerConf, UIDStore,
+    },
+    text::Truncate,
     utils::{
         connections::{std_net::connect as tcp_stream_connect, Connection},
         futures::timeout,
@@ -31,9 +35,8 @@ use crate::{
     LogLevel,
 };
 extern crate native_tls;
-#[cfg(debug_assertions)]
-use std::borrow::Cow;
 use std::{
+    borrow::Cow,
     collections::HashSet,
     convert::TryFrom,
     future::Future,
@@ -64,20 +67,16 @@ pub use smol::Async as AsyncWrapper;
 
 const IMAP_PROTOCOL_TIMEOUT: Duration = Duration::from_secs(60 * 28);
 
-macro_rules! imap_trace {
-    ($conn:expr, $fmt:literal, $($t:tt)*) => {
-        #[cfg(debug_assertions)]
-        log::trace!(std::concat!("{} ", $fmt), $conn.id, $($t)*);
+macro_rules! imap_log {
+    ($fn:ident, $conn:expr, $fmt:literal, $($t:tt)*) => {
+        log::$fn!(std::concat!("{} ", $fmt), $conn.id, $($t)*);
     };
-    ($conn:expr, $fmt:literal) => {
-        #[cfg(debug_assertions)]
-        log::trace!(std::concat!("{} ", $fmt), $conn.id);
+    ($fn:ident, $conn:expr, $fmt:literal) => {
+        log::$fn!(std::concat!("{} ", $fmt), $conn.id);
     };
 }
 
-use super::{protocol_parser, Capabilities, ImapServerConf, UIDStore};
-
-#[derive(Debug, Clone, Copy)]
+#[derive(Clone, Copy, Debug)]
 pub enum SyncPolicy {
     None,
     /// RFC4549 `Synch Ops for Disconnected IMAP4 Clients` <https://tools.ietf.org/html/rfc4549>
@@ -88,13 +87,13 @@ pub enum SyncPolicy {
     CondstoreQresync,
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Clone, Copy, Debug)]
 pub enum ImapProtocol {
     IMAP { extension_use: ImapExtensionUse },
     ManageSieve,
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Clone, Copy, Debug)]
 pub struct ImapExtensionUse {
     pub condstore: bool,
     pub idle: bool,
@@ -116,7 +115,6 @@ impl Default for ImapExtensionUse {
 #[derive(Debug)]
 pub struct ImapStream {
     pub cmd_id: usize,
-    #[cfg(debug_assertions)]
     pub id: Cow<'static, str>,
     pub stream: AsyncWrapper<Connection>,
     pub protocol: ImapProtocol,
@@ -124,7 +122,7 @@ pub struct ImapStream {
     pub timeout: Option<Duration>,
 }
 
-#[derive(Debug, Copy, Clone, Eq, PartialEq)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum MailboxSelection {
     None,
     Select(MailboxHash),
@@ -143,7 +141,6 @@ async fn try_await(cl: impl Future<Output = Result<()>> + Send) -> Result<()> {
 
 #[derive(Debug)]
 pub struct ImapConnection {
-    #[cfg(debug_assertions)]
     pub id: Cow<'static, str>,
     pub stream: Result<ImapStream>,
     pub server_conf: ImapServerConf,
@@ -154,7 +151,7 @@ pub struct ImapConnection {
 impl ImapStream {
     pub async fn new_connection(
         server_conf: &ImapServerConf,
-        #[cfg(debug_assertions)] id: Cow<'static, str>,
+        id: Cow<'static, str>,
         uid_store: &UIDStore,
     ) -> Result<(Capabilities, Self)> {
         let path = &server_conf.server_hostname;
@@ -175,10 +172,12 @@ impl ImapStream {
                 .build()
                 .chain_err_kind(ErrorKind::Network(NetworkErrorKind::InvalidTLSConnection))?;
 
-            let addr = (path.as_str(), server_conf.server_port);
-
             let mut socket = AsyncWrapper::new({
-                let conn = Connection::new_tcp(tcp_stream_connect(addr, server_conf.timeout)?);
+                let addr = (path.clone(), server_conf.server_port);
+                let timeout = server_conf.timeout;
+                let conn = Connection::new_tcp(
+                    smol::unblock(move || tcp_stream_connect(addr, timeout)).await?,
+                );
                 #[cfg(feature = "imap-trace")]
                 {
                     conn.trace(true).with_id("imap")
@@ -249,39 +248,50 @@ impl ImapStream {
             }
 
             {
-                // [ref:FIXME]: This is blocking
-                let socket = socket.into_inner()?;
-                let mut conn_result = connector.connect(path, socket);
-                if let Err(native_tls::HandshakeError::WouldBlock(midhandshake_stream)) =
-                    conn_result
-                {
-                    let mut midhandshake_stream = Some(midhandshake_stream);
-                    loop {
-                        match midhandshake_stream.take().unwrap().handshake() {
-                            Ok(r) => {
-                                conn_result = Ok(r);
-                                break;
-                            }
-                            Err(native_tls::HandshakeError::WouldBlock(stream)) => {
-                                midhandshake_stream = Some(stream);
-                            }
-                            p => {
-                                p.chain_err_kind(ErrorKind::Network(
-                                    NetworkErrorKind::InvalidTLSConnection,
-                                ))?;
+                let path = Arc::new(path.to_string());
+                let conn = smol::unblock({
+                    let socket = socket.into_inner()?;
+                    let path = Arc::clone(&path);
+                    move || {
+                        let conn_result = connector.connect(&path, socket);
+                        if let Err(native_tls::HandshakeError::WouldBlock(midhandshake_stream)) =
+                            conn_result
+                        {
+                            let mut midhandshake_stream = Some(midhandshake_stream);
+                            loop {
+                                match midhandshake_stream.take().unwrap().handshake() {
+                                    Ok(r) => {
+                                        return Ok(r);
+                                    }
+                                    Err(native_tls::HandshakeError::WouldBlock(stream)) => {
+                                        midhandshake_stream = Some(stream);
+                                    }
+                                    Err(err) => {
+                                        return Err(Error::from(err).set_kind(ErrorKind::Network(
+                                            NetworkErrorKind::InvalidTLSConnection,
+                                        )));
+                                    }
+                                }
                             }
                         }
+                        conn_result.chain_err_kind(ErrorKind::Network(
+                            NetworkErrorKind::InvalidTLSConnection,
+                        ))
                     }
-                }
-                AsyncWrapper::new(Connection::new_tls(conn_result.chain_err_summary(
-                    || format!("Could not initiate TLS negotiation to {}.", path),
-                )?))
-                .chain_err_summary(|| format!("Could not initiate TLS negotiation to {}.", path))?
+                })
+                .await
+                .chain_err_summary(|| format!("Could not initiate TLS negotiation to {}.", path))?;
+                AsyncWrapper::new(Connection::new_tls(conn))
+                    .chain_err_summary(|| format!("{} connection failed.", path))
+                    .chain_err_kind(ErrorKind::OSError)?
             }
         } else {
-            let addr = (path.as_str(), server_conf.server_port);
             AsyncWrapper::new({
-                let conn = Connection::new_tcp(tcp_stream_connect(addr, server_conf.timeout)?);
+                let addr = (path.clone(), server_conf.server_port);
+                let timeout = server_conf.timeout;
+                let conn = Connection::new_tcp(
+                    smol::unblock(move || tcp_stream_connect(addr, timeout)).await?,
+                );
                 #[cfg(feature = "imap-trace")]
                 {
                     conn.trace(true).with_id("imap")
@@ -301,7 +311,6 @@ impl ImapStream {
         let mut res = Vec::with_capacity(8 * 1024);
         let mut ret = Self {
             cmd_id,
-            #[cfg(debug_assertions)]
             id,
             stream,
             protocol: server_conf.protocol,
@@ -341,16 +350,24 @@ impl ImapStream {
                     .map(|(_, v)| v)
             });
 
-        if capabilities.is_err() {
-            return Err(Error::new(format!(
-                "Could not connect to {}: expected CAPABILITY response but got:{}",
-                &server_conf.server_hostname,
-                String::from_utf8_lossy(&res)
-            ))
-            .set_kind(ErrorKind::ProtocolError));
-        }
+        let capabilities = match capabilities {
+            Err(_err) => {
+                log::debug!(
+                    "Could not connect to {}: expected CAPABILITY response but got: {} `{}`",
+                    &server_conf.server_hostname,
+                    _err,
+                    String::from_utf8_lossy(&res)
+                );
+                return Err(Error::new(format!(
+                    "Could not connect to {}: expected CAPABILITY response but got: `{}`",
+                    &server_conf.server_hostname,
+                    String::from_utf8_lossy(&res).as_ref().trim_at_boundary(40)
+                ))
+                .set_kind(ErrorKind::ProtocolError));
+            }
+            Ok(v) => v,
+        };
 
-        let capabilities = capabilities.unwrap();
         if !capabilities
             .iter()
             .any(|cap| cap.eq_ignore_ascii_case(b"IMAP4rev1"))
@@ -368,7 +385,7 @@ impl ImapStream {
                 "Could not connect to {}: server does not accept logins [LOGINDISABLED]",
                 &server_conf.server_hostname
             ))
-            .set_err_kind(ErrorKind::Authentication));
+            .set_kind(ErrorKind::Authentication));
         }
 
         (uid_store.event_consumer)(
@@ -395,7 +412,7 @@ impl ImapStream {
                             .collect::<Vec<String>>()
                             .join(" ")
                     ))
-                    .set_err_kind(ErrorKind::Authentication));
+                    .set_kind(ErrorKind::Authentication));
                 }
                 let xoauth2 = base64::decode(&server_conf.server_password)
                     .chain_err_summary(|| {
@@ -442,7 +459,7 @@ impl ImapStream {
                             "Could not connect. Server replied with '{}'",
                             String::from_utf8_lossy(l[tag_start.len()..].trim())
                         ))
-                        .set_err_kind(ErrorKind::Authentication));
+                        .set_kind(ErrorKind::Authentication));
                     }
                     should_break = true;
                 }
@@ -518,7 +535,7 @@ impl ImapStream {
                 }
             }
         }
-        //imap_trace!(self, "returning IMAP response:\n{:?}", &ret);
+        //imap_log!(trace, self, "returning IMAP response:\n{:?}", &ret);
         Ok(())
     }
 
@@ -539,9 +556,9 @@ impl ImapStream {
             match self.protocol {
                 ImapProtocol::IMAP { .. } => {
                     if matches!(command.body, CommandBody::Login { .. }) {
-                        imap_trace!(self, "sent: M{} LOGIN ..", self.cmd_id - 1);
+                        imap_log!(trace, self, "sent: M{} LOGIN ..", self.cmd_id);
                     } else {
-                        imap_trace!(self, "sent: M{} {:?}", self.cmd_id - 1, command.body);
+                        imap_log!(trace, self, "sent: M{} {:?}", self.cmd_id, command.body);
                     }
                 }
                 ImapProtocol::ManageSieve => {}
@@ -595,11 +612,11 @@ impl ImapStream {
                 match self.protocol {
                     ImapProtocol::IMAP { .. } => {
                         if !command.starts_with(b"LOGIN") {
-                            imap_trace!(self, "sent: M{} {}", self.cmd_id - 1, unsafe {
+                            imap_log!(trace, self, "sent: M{} {}", self.cmd_id - 1, unsafe {
                                 std::str::from_utf8_unchecked(command)
                             });
                         } else {
-                            imap_trace!(self, "sent: M{} LOGIN ..", self.cmd_id - 1);
+                            imap_log!(trace, self, "sent: M{} LOGIN ..", self.cmd_id - 1);
                         }
                     }
                     ImapProtocol::ManageSieve => {}
@@ -629,12 +646,11 @@ impl ImapStream {
 impl ImapConnection {
     pub fn new_connection(
         server_conf: &ImapServerConf,
-        #[cfg(debug_assertions)] id: Cow<'static, str>,
+        id: Cow<'static, str>,
         uid_store: Arc<UIDStore>,
     ) -> Self {
         Self {
             stream: Err(Error::new("Offline".to_string())),
-            #[cfg(debug_assertions)]
             id,
             server_conf: server_conf.clone(),
             sync_policy: if *uid_store.keep_offline_cache.lock().unwrap() {
@@ -670,9 +686,15 @@ impl ImapConnection {
                 })
                 .await
                 {
-                    imap_trace!(self, "connect(): connection is probably dead: {:?}", &_err);
+                    imap_log!(
+                        trace,
+                        self,
+                        "connect(): connection is probably dead: {:?}",
+                        &_err
+                    );
                 } else {
-                    imap_trace!(
+                    imap_log!(
+                        trace,
                         self,
                         "connect(): connection is probably alive, NOOP returned {:?}",
                         &String::from_utf8_lossy(&ret)
@@ -680,13 +702,9 @@ impl ImapConnection {
                     return Ok(());
                 }
             }
-            let new_stream = ImapStream::new_connection(
-                &self.server_conf,
-                #[cfg(debug_assertions)]
-                self.id.clone(),
-                &self.uid_store,
-            )
-            .await;
+            let new_stream =
+                ImapStream::new_connection(&self.server_conf, self.id.clone(), &self.uid_store)
+                    .await;
             if let Err(err) = new_stream.as_ref() {
                 self.uid_store.is_online.lock().unwrap().1 = Err(err.clone());
             } else {
@@ -760,7 +778,6 @@ impl ImapConnection {
                             ImapResponse::Ok(_) => {
                                 let ImapStream {
                                     cmd_id,
-                                    #[cfg(debug_assertions)]
                                     id,
                                     stream,
                                     protocol,
@@ -770,7 +787,6 @@ impl ImapConnection {
                                 let stream = stream.into_inner()?;
                                 self.stream = Ok(ImapStream {
                                     cmd_id,
-                                    #[cfg(debug_assertions)]
                                     id,
                                     stream: AsyncWrapper::new(stream.deflate())?,
                                     protocol,
@@ -814,7 +830,8 @@ impl ImapConnection {
                         ImapResponse::No(ref _response_code)
                             if required_responses.intersects(RequiredResponses::NO_REQUIRED) =>
                         {
-                            imap_trace!(
+                            imap_log!(
+                                trace,
                                 self,
                                 "Received expected NO response: {:?} {:?}",
                                 _response_code,
@@ -822,7 +839,8 @@ impl ImapConnection {
                             );
                         }
                         ImapResponse::No(ref response_code) => {
-                            imap_trace!(
+                            imap_log!(
+                                trace,
                                 self,
                                 "Received NO response: {:?} {:?}",
                                 response_code,
@@ -840,7 +858,8 @@ impl ImapConnection {
                             return r.into();
                         }
                         ImapResponse::Bad(ref response_code) => {
-                            imap_trace!(
+                            imap_log!(
+                                trace,
                                 self,
                                 "Received BAD response: {:?} {:?}",
                                 response_code,
@@ -859,12 +878,12 @@ impl ImapConnection {
                         }
                         _ => {}
                     }
-                    /* imap_trace!(self,
+                    /* imap_log!(trace, self,
                         "check every line for required_responses: {:#?}",
                         &required_responses
                     );*/
                     for l in response.split_rn() {
-                        /* imap_trace!(self, "check line: {}", &l); */
+                        /* imap_log!(trace, self, "check line: {}", &l); */
                         if required_responses.check(l) || !self.process_untagged(l).await? {
                             ret.extend_from_slice(l);
                         }
@@ -983,7 +1002,8 @@ impl ImapConnection {
             .await?;
         self.read_response(ret, RequiredResponses::SELECT_REQUIRED)
             .await?;
-        imap_trace!(
+        imap_log!(
+            trace,
             self,
             "{} SELECT response {}",
             imap_path,
@@ -993,11 +1013,7 @@ impl ImapConnection {
             format!("Could not parse select response for mailbox {}", imap_path)
         })?;
         {
-            if *self.uid_store.keep_offline_cache.lock().unwrap() {
-                #[cfg(not(feature = "sqlite3"))]
-                let mut cache_handle = super::cache::DefaultCache::get(self.uid_store.clone())?;
-                #[cfg(feature = "sqlite3")]
-                let mut cache_handle = super::cache::Sqlite3Cache::get(self.uid_store.clone())?;
+            if let Some(mut cache_handle) = self.uid_store.cache_handle()? {
                 if let Err(err) = cache_handle.mailbox_state(mailbox_hash).and_then(|r| {
                     if r.is_none() {
                         cache_handle.clear(mailbox_hash, &select_response)
@@ -1071,7 +1087,12 @@ impl ImapConnection {
         self.read_response(ret, RequiredResponses::EXAMINE_REQUIRED)
             .await?;
 
-        imap_trace!(self, "EXAMINE response {}", String::from_utf8_lossy(ret));
+        imap_log!(
+            trace,
+            self,
+            "EXAMINE response {}",
+            String::from_utf8_lossy(ret)
+        );
         let select_response = protocol_parser::select_response(ret).chain_err_summary(|| {
             format!("Could not parse select response for mailbox {}", imap_path)
         })?;
