@@ -19,6 +19,9 @@
  * along with meli. If not, see <http://www.gnu.org/licenses/>.
  */
 
+// In case we forget to wait some future.
+#![deny(unused_must_use)]
+
 use smallvec::SmallVec;
 #[macro_use]
 mod protocol_parser;
@@ -49,9 +52,8 @@ use std::{
     time::{Duration, SystemTime},
 };
 
-#[cfg(feature = "sqlite3")]
-use cache::ImapCacheReset;
-use cache::ModSequence;
+pub extern crate imap_codec;
+pub use cache::ModSequence;
 use futures::{lock::Mutex as FutureMutex, stream::Stream};
 use imap_codec::imap_types::{
     command::CommandBody,
@@ -100,7 +102,7 @@ pub struct EnvelopeCache {
     flags: Option<Flag>,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Clone, Debug)]
 pub struct ImapServerConf {
     pub server_hostname: String,
     pub server_username: String,
@@ -146,31 +148,31 @@ macro_rules! get_conf_val {
 
 #[derive(Debug)]
 pub struct UIDStore {
-    account_hash: AccountHash,
-    account_name: Arc<str>,
-    keep_offline_cache: Arc<Mutex<bool>>,
-    capabilities: Arc<Mutex<Capabilities>>,
-    hash_index: Arc<Mutex<HashMap<EnvelopeHash, (UID, MailboxHash)>>>,
-    uid_index: Arc<Mutex<HashMap<(MailboxHash, UID), EnvelopeHash>>>,
-    msn_index: Arc<Mutex<HashMap<MailboxHash, Vec<UID>>>>,
+    pub account_hash: AccountHash,
+    pub account_name: Arc<str>,
+    pub keep_offline_cache: Arc<Mutex<bool>>,
+    pub capabilities: Arc<Mutex<Capabilities>>,
+    pub hash_index: Arc<Mutex<HashMap<EnvelopeHash, (UID, MailboxHash)>>>,
+    pub uid_index: Arc<Mutex<HashMap<(MailboxHash, UID), EnvelopeHash>>>,
+    pub msn_index: Arc<Mutex<HashMap<MailboxHash, Vec<UID>>>>,
 
-    byte_cache: Arc<Mutex<HashMap<UID, EnvelopeCache>>>,
-    collection: Collection,
+    pub byte_cache: Arc<Mutex<HashMap<UID, EnvelopeCache>>>,
+    pub collection: Collection,
 
-    /* Offline caching */
-    uidvalidity: Arc<Mutex<HashMap<MailboxHash, UID>>>,
-    envelopes: Arc<Mutex<HashMap<EnvelopeHash, cache::CachedEnvelope>>>,
-    max_uids: Arc<Mutex<HashMap<MailboxHash, UID>>>,
-    modseq: Arc<Mutex<HashMap<EnvelopeHash, ModSequence>>>,
-    highestmodseqs: Arc<Mutex<HashMap<MailboxHash, std::result::Result<ModSequence, ()>>>>,
-    mailboxes: Arc<FutureMutex<HashMap<MailboxHash, ImapMailbox>>>,
-    is_online: Arc<Mutex<(SystemTime, Result<()>)>>,
-    event_consumer: BackendEventConsumer,
-    timeout: Option<Duration>,
+    // Offline caching
+    pub uidvalidity: Arc<Mutex<HashMap<MailboxHash, UID>>>,
+    pub envelopes: Arc<Mutex<HashMap<EnvelopeHash, cache::CachedEnvelope>>>,
+    pub max_uids: Arc<Mutex<HashMap<MailboxHash, UID>>>,
+    pub modseq: Arc<Mutex<HashMap<EnvelopeHash, ModSequence>>>,
+    pub highestmodseqs: Arc<Mutex<HashMap<MailboxHash, std::result::Result<ModSequence, ()>>>>,
+    pub mailboxes: Arc<FutureMutex<HashMap<MailboxHash, ImapMailbox>>>,
+    pub is_online: Arc<Mutex<(SystemTime, Result<()>)>>,
+    pub event_consumer: BackendEventConsumer,
+    pub timeout: Option<Duration>,
 }
 
 impl UIDStore {
-    fn new(
+    pub fn new(
         account_hash: AccountHash,
         account_name: Arc<str>,
         event_consumer: BackendEventConsumer,
@@ -200,14 +202,38 @@ impl UIDStore {
             timeout,
         }
     }
+
+    pub fn cache_handle(self: &Arc<Self>) -> Result<Option<Box<dyn cache::ImapCache>>> {
+        if !*self.keep_offline_cache.lock().unwrap() {
+            return Ok(None);
+        }
+        #[cfg(not(feature = "sqlite3"))]
+        return Ok(None);
+        #[cfg(feature = "sqlite3")]
+        return Ok(Some(cache::sqlite3_cache::Sqlite3Cache::get(Arc::clone(
+            self,
+        ))?));
+    }
+
+    pub fn reset_db(self: &Arc<Self>) -> Result<()> {
+        if !*self.keep_offline_cache.lock().unwrap() {
+            return Ok(());
+        }
+        #[cfg(not(feature = "sqlite3"))]
+        return Ok(());
+        #[cfg(feature = "sqlite3")]
+        use crate::imap::cache::ImapCacheReset;
+        #[cfg(feature = "sqlite3")]
+        return cache::sqlite3_cache::Sqlite3Cache::reset_db(self);
+    }
 }
 
 #[derive(Debug)]
 pub struct ImapType {
-    _is_subscribed: Arc<IsSubscribedFn>,
-    connection: Arc<FutureMutex<ImapConnection>>,
-    server_conf: ImapServerConf,
-    uid_store: Arc<UIDStore>,
+    pub _is_subscribed: Arc<IsSubscribedFn>,
+    pub connection: Arc<FutureMutex<ImapConnection>>,
+    pub server_conf: ImapServerConf,
+    pub uid_store: Arc<UIDStore>,
 }
 
 impl MailBackend for ImapType {
@@ -301,39 +327,32 @@ impl MailBackend for ImapType {
         mailbox_hash: MailboxHash,
     ) -> Result<Pin<Box<dyn Stream<Item = Result<Vec<Envelope>>> + Send + 'static>>> {
         let cache_handle = {
-            #[cfg(feature = "sqlite3")]
-            if *self.uid_store.keep_offline_cache.lock().unwrap() {
-                match cache::Sqlite3Cache::get(self.uid_store.clone()).chain_err_summary(|| {
-                    format!(
-                        "Could not initialize cache for IMAP account {}. Resetting database.",
-                        self.uid_store.account_name
-                    )
-                }) {
-                    Ok(v) => Some(v),
-                    Err(err) => {
-                        (self.uid_store.event_consumer)(self.uid_store.account_hash, err.into());
-                        match cache::Sqlite3Cache::reset_db(&self.uid_store)
-                            .and_then(|()| cache::Sqlite3Cache::get(self.uid_store.clone()))
-                            .chain_err_summary(|| "Could not reset IMAP cache database.")
-                        {
-                            Ok(v) => Some(v),
-                            Err(err) => {
-                                *self.uid_store.keep_offline_cache.lock().unwrap() = false;
-                                log::trace!(
-                                    "{}: sqlite3 cache error: {}",
-                                    self.uid_store.account_name,
-                                    err
-                                );
-                                None
-                            }
+            match self.uid_store.cache_handle().chain_err_summary(|| {
+                format!(
+                    "Could not initialize cache for IMAP account {}. Resetting database.",
+                    self.uid_store.account_name
+                )
+            }) {
+                Ok(Some(v)) => Some(v),
+                Ok(None) => None,
+                Err(err) => {
+                    (self.uid_store.event_consumer)(self.uid_store.account_hash, err.into());
+                    match self
+                        .uid_store
+                        .reset_db()
+                        .and_then(|()| self.uid_store.cache_handle())
+                        .chain_err_summary(|| "Could not reset IMAP cache database.")
+                    {
+                        Ok(Some(v)) => Some(v),
+                        Ok(None) => None,
+                        Err(err) => {
+                            *self.uid_store.keep_offline_cache.lock().unwrap() = false;
+                            log::trace!("{}: cache error: {}", self.uid_store.account_name, err);
+                            None
                         }
                     }
                 }
-            } else {
-                None
             }
-            #[cfg(not(feature = "sqlite3"))]
-            None
         };
         let mut state = FetchState {
             stage: if *self.uid_store.keep_offline_cache.lock().unwrap() && cache_handle.is_some() {
@@ -363,7 +382,6 @@ impl MailBackend for ImapType {
             }
         };
         Ok(Box::pin(async_stream::try_stream! {
-            #[cfg(debug_assertions)]
             let id = state.connection.lock().await.id.clone();
             {
                 let f = &state.uid_store.mailboxes.lock().await[&mailbox_hash];
@@ -375,8 +393,7 @@ impl MailBackend for ImapType {
             };
             loop {
                 let res = fetch_hlpr(&mut state).await.map_err(|err| {
-                    #[cfg(debug_assertions)]
-                    log::trace!("{} fetch_hlpr err {:?}", id,  &err);
+                    log::trace!("{} fetch_hlpr at stage {:?} err {:?}", id,  state.stage, &err);
                     err
                 })?;
                 yield res;
@@ -395,7 +412,7 @@ impl MailBackend for ImapType {
             let inbox = timeout(uid_store.timeout, uid_store.mailboxes.lock())
                 .await?
                 .get(&mailbox_hash)
-                .map(std::clone::Clone::clone)
+                .cloned()
                 .unwrap();
             let mut conn = timeout(uid_store.timeout, main_conn.lock()).await?;
             watch::examine_updates(inbox, &mut conn, &uid_store).await?;
@@ -458,7 +475,7 @@ impl MailBackend for ImapType {
         Ok(Box::pin(async move {
             match timeout(timeout_dur, connection.lock()).await {
                 Ok(mut conn) => {
-                    imap_trace!(conn, "is_online");
+                    imap_log!(trace, conn, "is_online");
                     match timeout(timeout_dur, conn.connect()).await {
                         Ok(Ok(())) => Ok(()),
                         Err(err) | Ok(Err(err)) => {
@@ -494,7 +511,6 @@ impl MailBackend for ImapType {
                 idle(ImapWatchKit {
                     conn: ImapConnection::new_connection(
                         &server_conf,
-                        #[cfg(debug_assertions)]
                         "watch()::idle".into(),
                         uid_store.clone(),
                     ),
@@ -506,7 +522,6 @@ impl MailBackend for ImapType {
                 poll_with_examine(ImapWatchKit {
                     conn: ImapConnection::new_connection(
                         &server_conf,
-                        #[cfg(debug_assertions)]
                         "watch()::poll_with_examine".into(),
                         uid_store.clone(),
                     ),
@@ -539,7 +554,7 @@ impl MailBackend for ImapType {
                     }
                     Ok(()) => {
                         log::trace!(
-                            "{} Watch reconnect attempt succesful",
+                            "{} Watch reconnect attempt successful",
                             uid_store.account_name
                         );
                         continue;
@@ -701,7 +716,7 @@ impl MailBackend for ImapType {
         &mut self,
         env_hashes: EnvelopeHashBatch,
         mailbox_hash: MailboxHash,
-        flags: SmallVec<[(std::result::Result<Flag, String>, bool); 8]>,
+        flags: SmallVec<[FlagOp; 8]>,
     ) -> ResultFuture<()> {
         let connection = self.connection.clone();
         let uid_store = self.uid_store.clone();
@@ -724,7 +739,7 @@ impl MailBackend for ImapType {
             let mut conn = connection.lock().await;
             conn.select_mailbox(mailbox_hash, &mut response, false)
                 .await?;
-            if flags.iter().any(|(_, b)| *b) {
+            if flags.iter().any(<bool>::from) {
                 /* Set flags/tags to true */
                 let mut set_seen = false;
                 let command = {
@@ -734,28 +749,25 @@ impl MailBackend for ImapType {
                         cmd = format!("{},{}", cmd, uid);
                     }
                     cmd = format!("{} +FLAGS (", cmd);
-                    for (f, v) in flags.iter() {
-                        if !*v {
-                            continue;
-                        }
-                        match f {
-                            Ok(flag) if *flag == Flag::REPLIED => {
+                    for op in flags.iter().filter(|op| <bool>::from(*op)) {
+                        match op {
+                            FlagOp::Set(flag) if *flag == Flag::REPLIED => {
                                 cmd.push_str("\\Answered ");
                             }
-                            Ok(flag) if *flag == Flag::FLAGGED => {
+                            FlagOp::Set(flag) if *flag == Flag::FLAGGED => {
                                 cmd.push_str("\\Flagged ");
                             }
-                            Ok(flag) if *flag == Flag::TRASHED => {
+                            FlagOp::Set(flag) if *flag == Flag::TRASHED => {
                                 cmd.push_str("\\Deleted ");
                             }
-                            Ok(flag) if *flag == Flag::SEEN => {
+                            FlagOp::Set(flag) if *flag == Flag::SEEN => {
                                 cmd.push_str("\\Seen ");
                                 set_seen = true;
                             }
-                            Ok(flag) if *flag == Flag::DRAFT => {
+                            FlagOp::Set(flag) if *flag == Flag::DRAFT => {
                                 cmd.push_str("\\Draft ");
                             }
-                            Ok(_) => {
+                            FlagOp::Set(_) => {
                                 log::error!(
                                     "Application error: more than one flag bit set in set_flags: \
                                      {:?}",
@@ -768,12 +780,13 @@ impl MailBackend for ImapType {
                                 ))
                                 .set_kind(crate::ErrorKind::Bug));
                             }
-                            Err(tag) => {
+                            FlagOp::SetTag(tag) => {
                                 let hash = TagHash::from_bytes(tag.as_bytes());
                                 tag_lck.entry(hash).or_insert_with(|| tag.to_string());
                                 cmd.push_str(tag);
                                 cmd.push(' ');
                             }
+                            _ => {}
                         }
                     }
                     // pop last space
@@ -794,7 +807,7 @@ impl MailBackend for ImapType {
                     }
                 }
             }
-            if flags.iter().any(|(_, b)| !*b) {
+            if flags.iter().any(|b| !<bool>::from(b)) {
                 let mut set_unseen = false;
                 /* Set flags/tags to false */
                 let command = {
@@ -803,28 +816,25 @@ impl MailBackend for ImapType {
                         cmd = format!("{},{}", cmd, uid);
                     }
                     cmd = format!("{} -FLAGS (", cmd);
-                    for (f, v) in flags.iter() {
-                        if *v {
-                            continue;
-                        }
-                        match f {
-                            Ok(flag) if *flag == Flag::REPLIED => {
+                    for op in flags.iter().filter(|op| !<bool>::from(*op)) {
+                        match op {
+                            FlagOp::UnSet(flag) if *flag == Flag::REPLIED => {
                                 cmd.push_str("\\Answered ");
                             }
-                            Ok(flag) if *flag == Flag::FLAGGED => {
+                            FlagOp::UnSet(flag) if *flag == Flag::FLAGGED => {
                                 cmd.push_str("\\Flagged ");
                             }
-                            Ok(flag) if *flag == Flag::TRASHED => {
+                            FlagOp::UnSet(flag) if *flag == Flag::TRASHED => {
                                 cmd.push_str("\\Deleted ");
                             }
-                            Ok(flag) if *flag == Flag::SEEN => {
+                            FlagOp::UnSet(flag) if *flag == Flag::SEEN => {
                                 cmd.push_str("\\Seen ");
                                 set_unseen = true;
                             }
-                            Ok(flag) if *flag == Flag::DRAFT => {
+                            FlagOp::UnSet(flag) if *flag == Flag::DRAFT => {
                                 cmd.push_str("\\Draft ");
                             }
-                            Ok(_) => {
+                            FlagOp::UnSet(_) => {
                                 log::error!(
                                     "Application error: more than one flag bit set in set_flags: \
                                      {:?}",
@@ -836,12 +846,14 @@ impl MailBackend for ImapType {
                                     flags
                                 )));
                             }
-                            Err(tag) => {
+                            FlagOp::UnSetTag(tag) => {
                                 cmd.push_str(tag);
                                 cmd.push(' ');
                             }
+                            _ => {}
                         }
                     }
+                    // [ref:TODO] there should be a check that cmd is not empty here.
                     // pop last space
                     cmd.pop();
                     cmd.push(')');
@@ -852,14 +864,18 @@ impl MailBackend for ImapType {
                     .await?;
                 if set_unseen {
                     for f in uid_store.mailboxes.lock().await.values() {
-                        if let Ok(mut unseen) = f.unseen.lock() {
-                            for env_hash in env_hashes.iter() {
+                        if let (Ok(mut unseen), Ok(exists)) = (f.unseen.lock(), f.exists.lock()) {
+                            for env_hash in env_hashes.iter().filter(|h| exists.contains(h)) {
                                 unseen.insert_new(env_hash);
                             }
                         };
                     }
                 }
             }
+            if let Some(mut cache_handle) = uid_store.cache_handle()? {
+                let res = cache_handle.update_flags(env_hashes, mailbox_hash, flags);
+                log::trace!("update_flags in cache: {:?}", res);
+            }
             Ok(())
         }))
     }
@@ -872,7 +888,7 @@ impl MailBackend for ImapType {
         let flag_future = self.set_flags(
             env_hashes,
             mailbox_hash,
-            smallvec::smallvec![(Ok(Flag::TRASHED), true)],
+            smallvec::smallvec![FlagOp::Set(Flag::TRASHED)],
         )?;
         let connection = self.connection.clone();
         Ok(Box::pin(async move {
@@ -882,7 +898,8 @@ impl MailBackend for ImapType {
             conn.send_command(CommandBody::Expunge).await?;
             conn.read_response(&mut response, RequiredResponses::empty())
                 .await?;
-            imap_trace!(
+            imap_log!(
+                trace,
                 conn,
                 "EXPUNGE response: {}",
                 &String::from_utf8_lossy(&response)
@@ -913,11 +930,11 @@ impl MailBackend for ImapType {
         Ok(Box::pin(async move {
             /* Must transform path to something the IMAP server will accept
              *
-             * Each root mailbox has a hierarchy delimeter reported by the LIST entry.
-             * All paths must use this delimeter to indicate children of this
+             * Each root mailbox has a hierarchy delimiter reported by the LIST entry.
+             * All paths must use this delimiter to indicate children of this
              * mailbox.
              *
-             * A new root mailbox should have the default delimeter, which can be found
+             * A new root mailbox should have the default delimiter, which can be found
              * out by issuing an empty LIST command as described in RFC3501:
              * C: A101 LIST "" ""
              * S: * LIST (\Noselect) "/" ""
@@ -981,7 +998,7 @@ impl MailBackend for ImapType {
                 new_hash,
                 new_mailbox_fut?.await.map_err(|err| {
                     Error::new(format!(
-                        "Mailbox create was succesful (returned `{}`) but listing mailboxes \
+                        "Mailbox create was successful (returned `{}`) but listing mailboxes \
                          afterwards returned `{}`",
                         String::from_utf8_lossy(&response),
                         err
@@ -1042,7 +1059,7 @@ impl MailBackend for ImapType {
             uid_store.mailboxes.lock().await.clear();
             new_mailbox_fut?.await.map_err(|err| {
                 format!(
-                    "Mailbox delete was succesful (returned `{}`) but listing mailboxes \
+                    "Mailbox delete was successful (returned `{}`) but listing mailboxes \
                      afterwards returned `{}`",
                     String::from_utf8_lossy(&response),
                     err
@@ -1147,7 +1164,7 @@ impl MailBackend for ImapType {
             uid_store.mailboxes.lock().await.clear();
             new_mailbox_fut?.await.map_err(|err| {
                 format!(
-                    "Mailbox rename was succesful (returned `{}`) but listing mailboxes \
+                    "Mailbox rename was successful (returned `{}`) but listing mailboxes \
                      afterwards returned `{}`",
                     String::from_utf8_lossy(&response),
                     err
@@ -1208,7 +1225,8 @@ impl MailBackend for ImapType {
             .await?;
             conn.read_response(&mut response, RequiredResponses::SEARCH)
                 .await?;
-            imap_trace!(
+            imap_log!(
+                trace,
                 conn,
                 "searching for {} returned: {}",
                 query_str,
@@ -1304,12 +1322,8 @@ impl ImapType {
                 server_conf.timeout,
             )
         });
-        let connection = ImapConnection::new_connection(
-            &server_conf,
-            #[cfg(debug_assertions)]
-            "ImapType::new".into(),
-            uid_store.clone(),
-        );
+        let connection =
+            ImapConnection::new_connection(&server_conf, "ImapType::new".into(), uid_store.clone());
 
         Ok(Box::new(Self {
             server_conf,
@@ -1322,7 +1336,6 @@ impl ImapType {
     pub fn shell(&mut self) {
         let mut conn = ImapConnection::new_connection(
             &self.server_conf,
-            #[cfg(debug_assertions)]
             "ImapType::shell".into(),
             self.uid_store.clone(),
         );
@@ -1370,7 +1383,7 @@ impl ImapType {
                     if input.trim() == "IDLE" {
                         let mut iter = ImapBlockingConnection::from(conn);
                         while let Some(line) = iter.next() {
-                            imap_trace!("out: {}", unsafe { std::str::from_utf8_unchecked(&line) });
+                            imap_log!(trace, "out: {}", unsafe { std::str::from_utf8_unchecked(&line) });
                         }
                         conn = iter.into_conn();
                     }
@@ -1409,7 +1422,7 @@ impl ImapType {
             conn.read_response(&mut res, RequiredResponses::LIST_REQUIRED)
                 .await?;
         }
-        imap_trace!(conn, "LIST reply: {}", String::from_utf8_lossy(&res));
+        imap_log!(trace, conn, "LIST reply: {}", String::from_utf8_lossy(&res));
         for l in res.split_rn() {
             if !l.starts_with(b"*") {
                 continue;
@@ -1451,14 +1464,14 @@ impl ImapType {
                     }
                 }
             } else {
-                imap_trace!(conn, "parse error for {:?}", l);
+                imap_log!(trace, conn, "parse error for {:?}", l);
             }
         }
         mailboxes.retain(|_, v| !v.hash.is_null());
         conn.send_command(CommandBody::lsub("", "*")?).await?;
         conn.read_response(&mut res, RequiredResponses::LSUB_REQUIRED)
             .await?;
-        imap_trace!(conn, "LSUB reply: {}", String::from_utf8_lossy(&res));
+        imap_log!(trace, conn, "LSUB reply: {}", String::from_utf8_lossy(&res));
         for l in res.split_rn() {
             if !l.starts_with(b"*") {
                 continue;
@@ -1473,7 +1486,7 @@ impl ImapType {
                     f.is_subscribed = true;
                 }
             } else {
-                imap_trace!(conn, "parse error for {:?}", l);
+                imap_log!(trace, conn, "parse error for {:?}", l);
             }
         }
         Ok(mailboxes)
@@ -1586,7 +1599,7 @@ impl ImapType {
     }
 }
 
-#[derive(Debug, PartialEq, Copy, Clone)]
+#[derive(Clone, Copy, Debug, PartialEq)]
 enum FetchStage {
     InitialFresh,
     InitialCache,
@@ -1605,7 +1618,8 @@ struct FetchState {
 }
 
 async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
-    imap_trace!(
+    imap_log!(
+        trace,
         state.connection.lock().await,
         "fetch_hlpr mailbox: {:?} stage: {:?}",
         state.mailbox_hash,
@@ -1667,7 +1681,8 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                     }
                     Ok(Some(cached_payload)) => {
                         state.stage = FetchStage::ResyncCache;
-                        imap_trace!(
+                        imap_log!(
+                            trace,
                             state.connection.lock().await,
                             "fetch_hlpr fetch_cached_envs payload {} len for mailbox_hash {}",
                             cached_payload.len(),
@@ -1739,7 +1754,13 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                 conn.examine_mailbox(mailbox_hash, &mut response, false)
                     .await?;
                 if max_uid_left > 0 {
-                    imap_trace!(conn, "{} max_uid_left= {}", mailbox_hash, max_uid_left);
+                    imap_log!(
+                        trace,
+                        conn,
+                        "{} max_uid_left= {}",
+                        mailbox_hash,
+                        max_uid_left
+                    );
                     let sequence_set = if max_uid_left == 1 {
                         SequenceSet::from(ONE)
                     } else {
@@ -1763,7 +1784,8 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                             )
                         })?;
                     let (_, mut v, _) = protocol_parser::fetch_responses(&response)?;
-                    imap_trace!(
+                    imap_log!(
+                        trace,
                         conn,
                         "fetch response is {} bytes and {} lines and has {} parsed Envelopes",
                         response.len(),
@@ -1780,7 +1802,8 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                     } in v.iter_mut()
                     {
                         if uid.is_none() || envelope.is_none() || flags.is_none() {
-                            imap_trace!(
+                            imap_log!(
+                                trace,
                                 conn,
                                 "BUG? something in fetch is none. UID: {:?}, envelope: {:?} \
                                  flags: {:?}",
@@ -1788,7 +1811,8 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                                 envelope,
                                 flags
                             );
-                            imap_trace!(
+                            imap_log!(
+                                trace,
                                 conn,
                                 "response was: {}",
                                 String::from_utf8_lossy(&response)
@@ -1811,7 +1835,7 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                             for f in keywords {
                                 let hash = TagHash::from_bytes(f.as_bytes());
                                 tag_lck.entry(hash).or_insert_with(|| f.to_string());
-                                env.tags_mut().push(hash);
+                                env.tags_mut().insert(hash);
                             }
                         }
                     }
@@ -1842,7 +1866,7 @@ async fn fetch_hlpr(state: &mut FetchState) -> Result<Vec<Envelope>> {
                         let uid = uid.unwrap();
                         let env = envelope.unwrap();
                         /*
-                        imap_trace!(
+                        imap_log!(trace,
                             "env hash {} {} UID = {} MSN = {}",
                             env.hash(),
                             env.subject(),
