@@ -23,7 +23,6 @@ use std::convert::{TryFrom, TryInto};
 
 use imap_codec::imap_types::{command::CommandBody, search::SearchKey, sequence::SequenceSet};
 
-use super::{ImapConnection, MailboxSelection, UID};
 use crate::{
     backends::{
         BackendMailbox, RefreshEvent,
@@ -32,8 +31,12 @@ use crate::{
     },
     email::common_attributes,
     error::*,
-    imap::protocol_parser::{
-        generate_envelope_hash, FetchResponse, ImapLineSplit, RequiredResponses, UntaggedResponse,
+    imap::{
+        protocol_parser::{
+            generate_envelope_hash, FetchResponse, ImapLineSplit, RequiredResponses,
+            UntaggedResponse,
+        },
+        ImapConnection, MailboxSelection, UID,
     },
 };
 
@@ -43,7 +46,8 @@ impl ImapConnection {
             ($mailbox_hash: expr, $($result:expr $(,)*)+) => {
                 $(if let Err(err) = $result {
                     self.uid_store.is_online.lock().unwrap().1 = Err(err.clone());
-                    debug!("failure: {}", err.to_string());
+                    imap_log!(trace, self, "failure: {}", err.to_string());
+                    log::debug!("failure: {}", err.to_string());
                     self.add_refresh_event(RefreshEvent {
                         account_hash: self.uid_store.account_hash,
                         mailbox_hash: $mailbox_hash,
@@ -60,10 +64,7 @@ impl ImapConnection {
         let mailbox =
             std::clone::Clone::clone(&self.uid_store.mailboxes.lock().await[&mailbox_hash]);
 
-        #[cfg(not(feature = "sqlite3"))]
-        let mut cache_handle = super::cache::DefaultCache::get(self.uid_store.clone())?;
-        #[cfg(feature = "sqlite3")]
-        let mut cache_handle = super::cache::Sqlite3Cache::get(self.uid_store.clone())?;
+        let mut cache_handle = self.uid_store.cache_handle();
         let mut response = Vec::with_capacity(8 * 1024);
         let untagged_response =
             match super::protocol_parser::untagged_responses(line).map(|(_, v, _)| v) {
@@ -120,35 +121,46 @@ impl ImapConnection {
                         .lock()
                         .unwrap()
                         .iter()
-                        .filter(|((mailbox_hash_, u), _)| {
-                            *mailbox_hash_ == mailbox_hash && !results.contains(u)
-                        })
+                        .filter(|((_, u), _)| !results.contains(u))
                         .map(|((_, uid), hash)| (*uid, *hash))
                         .collect::<Vec<(UID, crate::email::EnvelopeHash)>>();
+                    let mut mboxes_to_update = vec![];
                     for (deleted_uid, deleted_hash) in deleteds {
-                        mailbox.exists.lock().unwrap().remove(deleted_hash);
-                        mailbox.unseen.lock().unwrap().remove(deleted_hash);
-                        self.uid_store
-                            .uid_index
-                            .lock()
-                            .unwrap()
-                            .remove(&(mailbox_hash, deleted_uid));
-                        self.uid_store
-                            .hash_index
-                            .lock()
-                            .unwrap()
-                            .remove(&deleted_hash);
-                        events.push((
-                            deleted_uid,
-                            RefreshEvent {
-                                account_hash: self.uid_store.account_hash,
-                                mailbox_hash,
-                                kind: Remove(deleted_hash),
-                            },
-                        ));
+                        for (mailbox_hash, mbx) in self.uid_store.mailboxes.lock().await.iter_mut()
+                        {
+                            mbx.exists.lock().unwrap().remove(deleted_hash);
+                            mbx.unseen.lock().unwrap().remove(deleted_hash);
+                            let mut existed = self
+                                .uid_store
+                                .uid_index
+                                .lock()
+                                .unwrap()
+                                .remove(&(*mailbox_hash, deleted_uid))
+                                .is_some();
+                            existed |= self
+                                .uid_store
+                                .hash_index
+                                .lock()
+                                .unwrap()
+                                .remove(&deleted_hash)
+                                .is_some();
+                            if existed {
+                                mboxes_to_update.push(*mailbox_hash);
+                                events.push((
+                                    deleted_uid,
+                                    RefreshEvent {
+                                        account_hash: self.uid_store.account_hash,
+                                        mailbox_hash: *mailbox_hash,
+                                        kind: Remove(deleted_hash),
+                                    },
+                                ));
+                            }
+                        }
                     }
-                    if *self.uid_store.keep_offline_cache.lock().unwrap() {
-                        cache_handle.update(mailbox_hash, &events)?;
+                    if let Ok(Some(ref mut cache_handle)) = cache_handle {
+                        for mailbox_hash in mboxes_to_update {
+                            cache_handle.update(mailbox_hash, &events)?;
+                        }
                     }
                     for (_, event) in events {
                         self.add_refresh_event(event);
@@ -164,7 +176,7 @@ impl ImapConnection {
                     .entry(mailbox_hash)
                     .or_default()
                     .remove(TryInto::<usize>::try_into(n).unwrap().saturating_sub(1));
-                debug!("expunge {}, UID = {}", n, deleted_uid);
+                imap_log!(trace, self, "expunge {}, UID = {}", n, deleted_uid);
                 let deleted_hash: crate::email::EnvelopeHash = match self
                     .uid_store
                     .uid_index
@@ -175,35 +187,44 @@ impl ImapConnection {
                     Some(v) => v,
                     None => return Ok(true),
                 };
-                mailbox.exists.lock().unwrap().remove(deleted_hash);
-                mailbox.unseen.lock().unwrap().remove(deleted_hash);
+                let mut mboxes_to_update = vec![];
+                for (mailbox_hash, mbx) in self.uid_store.mailboxes.lock().await.iter_mut() {
+                    if mbx.exists.lock().unwrap().remove(deleted_hash) {
+                        mboxes_to_update.push(*mailbox_hash);
+                    }
+                    mbx.unseen.lock().unwrap().remove(deleted_hash);
+                }
                 self.uid_store
                     .hash_index
                     .lock()
                     .unwrap()
                     .remove(&deleted_hash);
-                let mut event: [(UID, RefreshEvent); 1] = [(
-                    deleted_uid,
-                    RefreshEvent {
-                        account_hash: self.uid_store.account_hash,
-                        mailbox_hash,
-                        kind: Remove(deleted_hash),
-                    },
-                )];
-                if *self.uid_store.keep_offline_cache.lock().unwrap() {
-                    cache_handle.update(mailbox_hash, &event)?;
+                let events = mboxes_to_update
+                    .into_iter()
+                    .map(|mailbox_hash| {
+                        (
+                            mailbox_hash,
+                            [(
+                                deleted_uid,
+                                RefreshEvent {
+                                    account_hash: self.uid_store.account_hash,
+                                    mailbox_hash,
+                                    kind: Remove(deleted_hash),
+                                },
+                            )],
+                        )
+                    })
+                    .collect::<Vec<(_, [(UID, RefreshEvent); 1])>>();
+                for (mailbox_hash, pair) in events {
+                    if let Ok(Some(ref mut cache_handle)) = cache_handle {
+                        cache_handle.update(mailbox_hash, &pair)?;
+                    }
+                    let [(_, event)] = pair;
+                    self.add_refresh_event(event);
                 }
-                self.add_refresh_event(std::mem::replace(
-                    &mut event[0].1,
-                    RefreshEvent {
-                        account_hash: self.uid_store.account_hash,
-                        mailbox_hash,
-                        kind: Rescan,
-                    },
-                ));
             }
             UntaggedResponse::Exists(n) => {
-                debug!("exists {}", n);
+                imap_log!(trace, self, "exists {}", n);
                 try_fail!(
                     mailbox_hash,
                     self.send_command(CommandBody::fetch(n, common_attributes(), false)?).await
@@ -212,14 +233,16 @@ impl ImapConnection {
                 let mut v = match super::protocol_parser::fetch_responses(&response) {
                     Ok((_, v, _)) => v,
                     Err(err) => {
-                        debug!(
+                        imap_log!(
+                            trace,
+                            self,
                             "Error when parsing FETCH response after untagged exists {:?}",
                             err
                         );
                         return Ok(true);
                     }
                 };
-                debug!("responses len is {}", v.len());
+                imap_log!(trace, self, "responses len is {}", v.len());
                 for FetchResponse {
                     ref uid,
                     ref mut envelope,
@@ -246,7 +269,7 @@ impl ImapConnection {
                         for f in keywords {
                             let hash = TagHash::from_bytes(f.as_bytes());
                             tag_lck.entry(hash).or_insert_with(|| f.to_string());
-                            env.tags_mut().push(hash);
+                            env.tags_mut().insert(hash);
                         }
                     }
                     mailbox.exists.lock().unwrap().insert_new(env.hash());
@@ -275,14 +298,16 @@ impl ImapConnection {
                         .lock()
                         .unwrap()
                         .insert((mailbox_hash, uid), env.hash());
-                    debug!(
+                    imap_log!(
+                        trace,
+                        self,
                         "Create event {} {} {}",
                         env.hash(),
                         env.subject(),
                         mailbox.path(),
                     );
                 }
-                if *self.uid_store.keep_offline_cache.lock().unwrap() {
+                if let Ok(Some(ref mut cache_handle)) = cache_handle {
                     if let Err(err) = cache_handle
                         .insert_envelopes(mailbox_hash, &v)
                         .chain_err_summary(|| {
@@ -292,7 +317,7 @@ impl ImapConnection {
                             )
                         })
                     {
-                        log::info!("{err}");
+                        imap_log!(info, self, "{}", err);
                     }
                 }
                 for response in v {
@@ -320,7 +345,7 @@ impl ImapConnection {
                     .map_err(Error::from)
                 {
                     Ok(&[]) => {
-                        debug!("UID SEARCH RECENT returned no results");
+                        imap_log!(trace, self, "UID SEARCH RECENT returned no results");
                     }
                     Ok(v) => {
                         let command = {
@@ -345,14 +370,16 @@ impl ImapConnection {
                         let mut v = match super::protocol_parser::fetch_responses(&response) {
                             Ok((_, v, _)) => v,
                             Err(err) => {
-                                debug!(
+                                imap_log!(
+                                    debug,
+                                    self,
                                     "Error when parsing FETCH response after untagged recent {:?}",
                                     err
                                 );
                                 return Ok(true);
                             }
                         };
-                        debug!("responses len is {}", v.len());
+                        imap_log!(trace, self, "responses len is {}", v.len());
                         for FetchResponse {
                             ref uid,
                             ref mut envelope,
@@ -379,12 +406,12 @@ impl ImapConnection {
                                 for f in keywords {
                                     let hash = TagHash::from_bytes(f.as_bytes());
                                     tag_lck.entry(hash).or_insert_with(|| f.to_string());
-                                    env.tags_mut().push(hash);
+                                    env.tags_mut().insert(hash);
                                 }
                             }
                             mailbox.exists.lock().unwrap().insert_new(env.hash());
                         }
-                        if *self.uid_store.keep_offline_cache.lock().unwrap() {
+                        if let Ok(Some(ref mut cache_handle)) = cache_handle {
                             if let Err(err) = cache_handle
                                 .insert_envelopes(mailbox_hash, &v)
                                 .chain_err_summary(|| {
@@ -505,9 +532,15 @@ impl ImapConnection {
                         temp
                     } {
                         if !flags.0.intersects(crate::email::Flag::SEEN) {
-                            mailbox.unseen.lock().unwrap().insert_new(env_hash);
+                            for mbx in self.uid_store.mailboxes.lock().await.values_mut() {
+                                if mbx.exists.lock().unwrap().contains(&env_hash) {
+                                    mbx.unseen.lock().unwrap().insert_new(env_hash);
+                                }
+                            }
                         } else {
-                            mailbox.unseen.lock().unwrap().remove(env_hash);
+                            for mbx in self.uid_store.mailboxes.lock().await.values_mut() {
+                                mbx.unseen.lock().unwrap().remove(env_hash);
+                            }
                         }
                         mailbox.exists.lock().unwrap().insert_new(env_hash);
                         if let Some(modseq) = modseq {
@@ -525,7 +558,7 @@ impl ImapConnection {
                                 kind: NewFlags(env_hash, flags),
                             },
                         )];
-                        if *self.uid_store.keep_offline_cache.lock().unwrap() {
+                        if let Ok(Some(ref mut cache_handle)) = cache_handle {
                             cache_handle.update(mailbox_hash, &event)?;
                         }
                         self.add_refresh_event(std::mem::replace(
