@@ -20,7 +20,8 @@
  */
 
 use std::{
-    collections::{hash_map::HashMap, BTreeMap},
+    borrow::Cow,
+    collections::{hash_map::HashMap, BTreeMap, BTreeSet},
     ffi::{CStr, CString, OsStr},
     io::Read,
     os::unix::ffi::OsStrExt,
@@ -31,20 +32,27 @@ use std::{
 };
 
 use futures::Stream;
+use notify::{RecommendedWatcher, RecursiveMode, Watcher};
 use smallvec::SmallVec;
 
 use crate::{
     backends::*,
     conf::AccountSettings,
     email::{Envelope, EnvelopeHash, Flag},
-    error::{Error, Result},
+    error::{Error, ErrorKind, IntoError, Result},
     utils::shellexpand::ShellExpandTrait,
-    Collection, ErrorKind,
+    Collection,
 };
 
 macro_rules! call {
     ($lib:expr, $func:ty) => {{
-        let func: libloading::Symbol<$func> = $lib.get(stringify!($func).as_bytes()).unwrap();
+        #[cfg(debug_assertions)]
+        debug_assert!(
+            !stringify!($func).starts_with("ffi::"),
+            "{} must be a valid FFI symbol.",
+            stringify!($func)
+        );
+        let func: libloading::Symbol<$func> = $lib.inner.get(stringify!($func).as_bytes()).unwrap();
         func
     }};
 }
@@ -52,19 +60,28 @@ macro_rules! call {
 macro_rules! try_call {
     ($lib:expr, $call:expr) => {{
         let status = $call;
-        if status == _notmuch_status_NOTMUCH_STATUS_SUCCESS {
+        if status == $crate::notmuch::ffi::NOTMUCH_STATUS_SUCCESS {
             Ok(())
         } else {
             let c_str = call!($lib, notmuch_status_to_string)(status);
-            Err(NotmuchError(
-                CStr::from_ptr(c_str).to_string_lossy().into_owned(),
+            Err($crate::notmuch::NotmuchError(
+                std::ffi::CStr::from_ptr(c_str)
+                    .to_string_lossy()
+                    .into_owned(),
             ))
         }
     }};
 }
 
-pub mod bindings;
-use bindings::*;
+pub mod query;
+use query::{MelibQueryToNotmuchQuery, Query};
+pub mod mailbox;
+use mailbox::NotmuchMailbox;
+pub mod ffi;
+use ffi::{
+    notmuch_database_close, notmuch_database_destroy, notmuch_database_get_revision,
+    notmuch_database_open, notmuch_status_to_string,
+};
 mod message;
 pub use message::*;
 mod tags;
@@ -74,22 +91,27 @@ pub use thread::*;
 
 #[derive(Debug)]
 #[repr(transparent)]
-struct DbPointer(NonNull<notmuch_database_t>);
+struct DbPointer(NonNull<ffi::notmuch_database_t>);
 
 unsafe impl Send for DbPointer {}
 unsafe impl Sync for DbPointer {}
 
 impl DbPointer {
     #[inline]
-    pub(self) fn as_mut(&mut self) -> *mut notmuch_database_t {
+    pub(self) fn as_mut(&mut self) -> *mut ffi::notmuch_database_t {
         unsafe { self.0.as_mut() }
     }
 }
 
+#[derive(Debug)]
+pub struct NotmuchLibrary {
+    pub inner: libloading::Library,
+    pub dlpath: Cow<'static, str>,
+}
+
 #[derive(Debug)]
 pub struct DbConnection {
-    #[allow(dead_code)]
-    pub lib: Arc<libloading::Library>,
+    pub lib: Arc<NotmuchLibrary>,
     inner: Arc<Mutex<DbPointer>>,
     pub revision_uuid: Arc<RwLock<u64>>,
 }
@@ -187,10 +209,9 @@ impl DbConnection {
                         );
                     }
                 }
-                false
-            } else {
-                true
+                return false;
             }
+            true
         });
         Ok(())
     }
@@ -219,14 +240,14 @@ impl Drop for DbConnection {
                 self.lib,
                 call!(self.lib, notmuch_database_close)(inner.as_mut())
             ) {
-                debug!(err);
+                log::error!("Could not call C notmuch_database_close: {}", err);
                 return;
             }
             if let Err(err) = try_call!(
                 self.lib,
                 call!(self.lib, notmuch_database_destroy)(inner.as_mut())
             ) {
-                debug!(err);
+                log::error!("Could not call C notmuch_database_destroy: {}", err);
             }
         }
     }
@@ -235,7 +256,7 @@ impl Drop for DbConnection {
 #[derive(Debug)]
 pub struct NotmuchDb {
     #[allow(dead_code)]
-    lib: Arc<libloading::Library>,
+    lib: Arc<NotmuchLibrary>,
     revision_uuid: Arc<RwLock<u64>>,
     mailboxes: Arc<RwLock<HashMap<MailboxHash, NotmuchMailbox>>>,
     index: Arc<RwLock<HashMap<EnvelopeHash, CString>>>,
@@ -248,77 +269,6 @@ pub struct NotmuchDb {
     save_messages_to: Option<PathBuf>,
 }
 
-unsafe impl Send for NotmuchDb {}
-unsafe impl Sync for NotmuchDb {}
-
-#[derive(Debug, Clone, Default)]
-struct NotmuchMailbox {
-    hash: MailboxHash,
-    children: Vec<MailboxHash>,
-    parent: Option<MailboxHash>,
-    name: String,
-    path: String,
-    query_str: String,
-    usage: Arc<RwLock<SpecialUsageMailbox>>,
-
-    total: Arc<Mutex<usize>>,
-    unseen: Arc<Mutex<usize>>,
-}
-
-impl BackendMailbox for NotmuchMailbox {
-    fn hash(&self) -> MailboxHash {
-        self.hash
-    }
-
-    fn name(&self) -> &str {
-        self.name.as_str()
-    }
-
-    fn path(&self) -> &str {
-        self.path.as_str()
-    }
-
-    fn clone(&self) -> Mailbox {
-        Box::new(std::clone::Clone::clone(self))
-    }
-
-    fn children(&self) -> &[MailboxHash] {
-        &self.children
-    }
-
-    fn parent(&self) -> Option<MailboxHash> {
-        self.parent
-    }
-
-    fn special_usage(&self) -> SpecialUsageMailbox {
-        *self.usage.read().unwrap()
-    }
-
-    fn permissions(&self) -> MailboxPermissions {
-        MailboxPermissions::default()
-    }
-
-    fn is_subscribed(&self) -> bool {
-        true
-    }
-
-    fn set_is_subscribed(&mut self, _new_val: bool) -> Result<()> {
-        Ok(())
-    }
-
-    fn set_special_usage(&mut self, new_val: SpecialUsageMailbox) -> Result<()> {
-        *self.usage.write()? = new_val;
-        Ok(())
-    }
-
-    fn count(&self) -> Result<(usize, usize)> {
-        Ok((*self.unseen.lock()?, *self.total.lock()?))
-    }
-}
-
-unsafe impl Send for NotmuchMailbox {}
-unsafe impl Sync for NotmuchMailbox {}
-
 impl NotmuchDb {
     #[allow(clippy::new_ret_no_self)]
     pub fn new(
@@ -327,38 +277,49 @@ impl NotmuchDb {
         event_consumer: BackendEventConsumer,
     ) -> Result<Box<dyn MailBackend>> {
         #[cfg(target_os = "linux")]
-        let mut dlpath = "libnotmuch.so.5";
+        let mut dlpath = Cow::Borrowed("libnotmuch.so.5");
         #[cfg(target_os = "macos")]
-        let mut dlpath = "libnotmuch.5.dylib";
+        let mut dlpath = Cow::Borrowed("libnotmuch.5.dylib");
         #[cfg(not(any(target_os = "linux", target_os = "macos")))]
-        let mut dlpath = "libnotmuch.so";
+        let mut dlpath = Cow::Borrowed("libnotmuch.so");
         let mut custom_dlpath = false;
         if let Some(lib_path) = s.extra.get("library_file_path") {
-            dlpath = lib_path.as_str();
+            let expanded_path = Path::new(lib_path).expand();
+            let expanded_path_string = expanded_path.display().to_string();
+            dlpath = if &expanded_path_string != lib_path
+                && expanded_path.try_exists().unwrap_or(false)
+            {
+                Cow::Owned(expanded_path_string)
+            } else {
+                Cow::Owned(lib_path.to_string())
+            };
             custom_dlpath = true;
         }
-        let lib = Arc::new(unsafe {
-            match libloading::Library::new(dlpath) {
-                Ok(l) => l,
-                Err(err) => {
-                    if custom_dlpath {
-                        return Err(Error::new(format!(
-                            "Notmuch `library_file_path` setting value `{}` for account {} does \
-                             not exist or is a directory or not a valid library file.",
-                            dlpath, s.name
-                        ))
-                        .set_kind(ErrorKind::Configuration)
-                        .set_source(Some(Arc::new(err))));
-                    } else {
-                        return Err(Error::new("Could not load libnotmuch!")
-                            .set_details(super::NOTMUCH_ERROR_DETAILS)
+        let lib = Arc::new(NotmuchLibrary {
+            inner: unsafe {
+                match libloading::Library::new(dlpath.as_ref()) {
+                    Ok(l) => l,
+                    Err(err) => {
+                        if custom_dlpath {
+                            return Err(Error::new(format!(
+                                "Notmuch `library_file_path` setting value `{}` for account {} \
+                                 does not exist or is a directory or not a valid library file.",
+                                dlpath, s.name
+                            ))
+                            .set_kind(ErrorKind::Configuration)
                             .set_source(Some(Arc::new(err))));
+                        } else {
+                            return Err(Error::new("Could not load libnotmuch!")
+                                .set_details(super::NOTMUCH_ERROR_DETAILS)
+                                .set_source(Some(Arc::new(err))));
+                        }
                     }
                 }
-            }
+            },
+            dlpath,
         });
         let mut path = Path::new(s.root_mailbox.as_str()).expand();
-        if !path.exists() {
+        if !path.try_exists().unwrap_or(false) {
             return Err(Error::new(format!(
                 "Notmuch `root_mailbox` {} for account {} does not exist.",
                 s.root_mailbox.as_str(),
@@ -375,7 +336,7 @@ impl NotmuchDb {
             .set_kind(ErrorKind::Configuration));
         }
         path.push(".notmuch");
-        if !path.exists() || !path.is_dir() {
+        if !path.try_exists().unwrap_or(false) || !path.is_dir() {
             return Err(Error::new(format!(
                 "Notmuch `root_mailbox` {} for account {} does not contain a `.notmuch` \
                  subdirectory.",
@@ -459,7 +420,7 @@ impl NotmuchDb {
 
     pub fn validate_config(s: &mut AccountSettings) -> Result<()> {
         let mut path = Path::new(s.root_mailbox.as_str()).expand();
-        if !path.exists() {
+        if !path.try_exists().unwrap_or(false) {
             return Err(Error::new(format!(
                 "Notmuch `root_mailbox` {} for account {} does not exist.",
                 s.root_mailbox.as_str(),
@@ -476,7 +437,7 @@ impl NotmuchDb {
             .set_kind(ErrorKind::Configuration));
         }
         path.push(".notmuch");
-        if !path.exists() || !path.is_dir() {
+        if !path.try_exists().unwrap_or(false) || !path.is_dir() {
             return Err(Error::new(format!(
                 "Notmuch `root_mailbox` {} for account {} does not contain a `.notmuch` \
                  subdirectory.",
@@ -489,7 +450,12 @@ impl NotmuchDb {
 
         let account_name = s.name.to_string();
         if let Some(lib_path) = s.extra.remove("library_file_path") {
-            if !Path::new(&lib_path).exists() || Path::new(&lib_path).is_dir() {
+            let expanded_path = Path::new(&lib_path).expand();
+            if (!Path::new(&lib_path).try_exists().unwrap_or(false)
+                || Path::new(&lib_path).is_dir())
+                && !Path::new(&expanded_path).try_exists().unwrap_or(false)
+                || Path::new(&expanded_path).is_dir()
+            {
                 return Err(Error::new(format!(
                     "Notmuch `library_file_path` setting value `{}` for account {} does not exist \
                      or is a directory.",
@@ -543,19 +509,19 @@ impl NotmuchDb {
     fn new_connection(
         path: &Path,
         revision_uuid: Arc<RwLock<u64>>,
-        lib: Arc<libloading::Library>,
+        lib: Arc<NotmuchLibrary>,
         write: bool,
     ) -> Result<DbConnection> {
-        let path_c = std::ffi::CString::new(path.to_str().unwrap()).unwrap();
+        let path_c = CString::new(path.to_str().unwrap()).unwrap();
         let path_ptr = path_c.as_ptr();
-        let mut database: *mut notmuch_database_t = std::ptr::null_mut();
+        let mut database: *mut ffi::notmuch_database_t = std::ptr::null_mut();
         let status = unsafe {
             call!(lib, notmuch_database_open)(
                 path_ptr,
                 if write {
-                    notmuch_database_mode_t_NOTMUCH_DATABASE_MODE_READ_WRITE
+                    ffi::NOTMUCH_DATABASE_MODE_READ_WRITE
                 } else {
-                    notmuch_database_mode_t_NOTMUCH_DATABASE_MODE_READ_ONLY
+                    ffi::NOTMUCH_DATABASE_MODE_READ_ONLY
                 },
                 std::ptr::addr_of_mut!(database),
             )
@@ -708,7 +674,7 @@ impl MailBackend for NotmuchDb {
             iter: v.into_iter(),
         };
         Ok(Box::pin(async_stream::try_stream! {
-            while let Some(res) = state.fetch().await.map_err(|err| { debug!("fetch err {:?}", &err); err})? {
+            while let Some(res) = state.fetch().await.map_err(|err| { log::debug!("fetch err {:?}", &err); err})? {
                 yield res;
             }
         }))
@@ -746,9 +712,6 @@ impl MailBackend for NotmuchDb {
     }
 
     fn watch(&self) -> ResultFuture<()> {
-        extern crate notify;
-        use notify::{watcher, RecursiveMode, Watcher};
-
         let account_hash = self.account_hash;
         let tag_index = self.collection.tag_index.clone();
         let lib = self.lib.clone();
@@ -760,8 +723,15 @@ impl MailBackend for NotmuchDb {
         let event_consumer = self.event_consumer.clone();
 
         let (tx, rx) = std::sync::mpsc::channel();
-        let mut watcher = watcher(tx, std::time::Duration::from_secs(2)).unwrap();
-        watcher.watch(&self.path, RecursiveMode::Recursive).unwrap();
+        let watcher = RecommendedWatcher::new(
+            tx,
+            notify::Config::default().with_poll_interval(std::time::Duration::from_secs(2)),
+        )
+        .and_then(|mut watcher| {
+            watcher.watch(&self.path, RecursiveMode::Recursive)?;
+            Ok(watcher)
+        })
+        .map_err(|err| err.set_err_details("Failed to create file change monitor."))?;
         Ok(Box::pin(async move {
             let _watcher = watcher;
             let rx = rx;
@@ -849,8 +819,8 @@ impl MailBackend for NotmuchDb {
     fn set_flags(
         &mut self,
         env_hashes: EnvelopeHashBatch,
-        _mailbox_hash: MailboxHash,
-        flags: SmallVec<[(std::result::Result<Flag, String>, bool); 8]>,
+        mailbox_hash: MailboxHash,
+        flags: SmallVec<[FlagOp; 8]>,
     ) -> ResultFuture<()> {
         let database = Self::new_connection(
             self.path.as_path(),
@@ -859,22 +829,24 @@ impl MailBackend for NotmuchDb {
             true,
         )?;
         let tag_index = self.collection.clone().tag_index;
+        let mailboxes = self.mailboxes.clone();
+        let mailbox_index = self.mailbox_index.clone();
         let index = self.index.clone();
 
         Ok(Box::pin(async move {
             let mut index_lck = index.write().unwrap();
+            let mut has_seen_changes_mailboxes_set = BTreeSet::new();
             for env_hash in env_hashes.iter() {
-                debug!(&env_hash);
                 let message = match Message::find_message(&database, &index_lck[&env_hash]) {
                     Ok(v) => v,
                     Err(err) => {
-                        debug!("not found {}", err);
+                        log::debug!("not found {}", err);
                         continue;
                     }
                 };
+                message.freeze();
 
-                let tags = debug!(message.tags().collect::<Vec<&CStr>>());
-                //flags.set(f, value);
+                let tags = message.tags().collect::<Vec<&CStr>>();
 
                 macro_rules! cstr {
                     ($l:literal) => {
@@ -906,35 +878,37 @@ impl MailBackend for NotmuchDb {
                     }};
                 }
 
-                for (f, v) in flags.iter() {
-                    let value = *v;
-                    debug!(&f);
-                    debug!(&value);
-                    match f {
-                        Ok(Flag::DRAFT) if value => add_tag!(b"draft\0"),
-                        Ok(Flag::DRAFT) => remove_tag!(b"draft\0"),
-                        Ok(Flag::FLAGGED) if value => add_tag!(b"flagged\0"),
-                        Ok(Flag::FLAGGED) => remove_tag!(b"flagged\0"),
-                        Ok(Flag::PASSED) if value => add_tag!(b"passed\0"),
-                        Ok(Flag::PASSED) => remove_tag!(b"passed\0"),
-                        Ok(Flag::REPLIED) if value => add_tag!(b"replied\0"),
-                        Ok(Flag::REPLIED) => remove_tag!(b"replied\0"),
-                        Ok(Flag::SEEN) if value => remove_tag!(b"unread\0"),
-                        Ok(Flag::SEEN) => add_tag!(b"unread\0"),
-                        Ok(Flag::TRASHED) if value => add_tag!(b"trashed\0"),
-                        Ok(Flag::TRASHED) => remove_tag!(b"trashed\0"),
-                        Ok(_) => debug!("flags is {:?} value = {}", f, value),
-                        Err(tag) if value => {
+                let mut has_seen_changes = false;
+                for op in flags.iter() {
+                    has_seen_changes |=
+                        matches!(op, FlagOp::Set(Flag::SEEN) | FlagOp::UnSet(Flag::SEEN));
+                    match op {
+                        FlagOp::Set(Flag::DRAFT) => add_tag!(b"draft\0"),
+                        FlagOp::UnSet(Flag::DRAFT) => remove_tag!(b"draft\0"),
+                        FlagOp::Set(Flag::FLAGGED) => add_tag!(b"flagged\0"),
+                        FlagOp::UnSet(Flag::FLAGGED) => remove_tag!(b"flagged\0"),
+                        FlagOp::Set(Flag::PASSED) => add_tag!(b"passed\0"),
+                        FlagOp::UnSet(Flag::PASSED) => remove_tag!(b"passed\0"),
+                        FlagOp::Set(Flag::REPLIED) => add_tag!(b"replied\0"),
+                        FlagOp::UnSet(Flag::REPLIED) => remove_tag!(b"replied\0"),
+                        FlagOp::Set(Flag::SEEN) => remove_tag!(b"unread\0"),
+                        FlagOp::UnSet(Flag::SEEN) => add_tag!(b"unread\0"),
+                        FlagOp::Set(Flag::TRASHED) => add_tag!(b"trashed\0"),
+                        FlagOp::UnSet(Flag::TRASHED) => remove_tag!(b"trashed\0"),
+                        FlagOp::SetTag(tag) => {
                             let c_tag = CString::new(tag.as_str()).unwrap();
                             add_tag!(&c_tag.as_ref());
                         }
-                        Err(tag) => {
+                        FlagOp::UnSetTag(tag) => {
                             let c_tag = CString::new(tag.as_str()).unwrap();
                             remove_tag!(&c_tag.as_ref());
                         }
+                        _ => log::debug!("flag_op is {:?}", op),
                     }
                 }
 
+                message.thaw();
+
                 /* Update message filesystem path. */
                 message.tags_to_maildir_flags()?;
 
@@ -942,13 +916,26 @@ impl MailBackend for NotmuchDb {
                 if let Some(p) = index_lck.get_mut(&env_hash) {
                     *p = msg_id.into();
                 }
+
+                if has_seen_changes {
+                    let mailbox_index_lck = mailbox_index.read().unwrap();
+                    has_seen_changes_mailboxes_set.insert(mailbox_hash);
+                    has_seen_changes_mailboxes_set
+                        .extend(mailbox_index_lck.values().flat_map(|hs| hs.iter().cloned()));
+                }
             }
-            for (f, v) in flags.iter() {
-                if let (Err(tag), true) = (f, v) {
+            for op in flags.iter() {
+                if let FlagOp::SetTag(tag) = op {
                     let hash = TagHash::from_bytes(tag.as_bytes());
                     tag_index.write().unwrap().insert(hash, tag.to_string());
                 }
             }
+            if !has_seen_changes_mailboxes_set.is_empty() {
+                let mailboxes_lck = mailboxes.write().unwrap();
+                for mh in has_seen_changes_mailboxes_set {
+                    mailboxes_lck[&mh].update_counts(&database)?;
+                }
+            }
 
             Ok(())
         }))
@@ -1073,7 +1060,7 @@ struct NotmuchOp {
     database: Arc<DbConnection>,
     bytes: Option<Vec<u8>>,
     #[allow(dead_code)]
-    lib: Arc<libloading::Library>,
+    lib: Arc<NotmuchLibrary>,
 }
 
 impl BackendOp for NotmuchOp {
@@ -1088,209 +1075,3 @@ impl BackendOp for NotmuchOp {
         Ok(Box::pin(async move { ret }))
     }
 }
-
-pub struct Query<'s> {
-    #[allow(dead_code)]
-    lib: Arc<libloading::Library>,
-    ptr: *mut notmuch_query_t,
-    query_str: &'s str,
-}
-
-impl<'s> Query<'s> {
-    fn new(database: &DbConnection, query_str: &'s str) -> Result<Self> {
-        let lib: Arc<libloading::Library> = database.lib.clone();
-        let query_cstr = std::ffi::CString::new(query_str)?;
-        let query: *mut notmuch_query_t = unsafe {
-            call!(lib, notmuch_query_create)(
-                database.inner.lock().unwrap().as_mut(),
-                query_cstr.as_ptr(),
-            )
-        };
-        if query.is_null() {
-            return Err(Error::new("Could not create query. Out of memory?"));
-        }
-        Ok(Query {
-            lib,
-            ptr: query,
-            query_str,
-        })
-    }
-
-    fn count(&self) -> Result<u32> {
-        let mut count = 0_u32;
-        unsafe {
-            try_call!(
-                self.lib,
-                call!(self.lib, notmuch_query_count_messages)(
-                    self.ptr,
-                    std::ptr::addr_of_mut!(count)
-                )
-            )
-            .map_err(|err| err.0)?;
-        }
-        Ok(count)
-    }
-
-    fn search(&'s self) -> Result<MessageIterator<'s>> {
-        let mut messages: *mut notmuch_messages_t = std::ptr::null_mut();
-        let status = unsafe {
-            call!(self.lib, notmuch_query_search_messages)(
-                self.ptr,
-                std::ptr::addr_of_mut!(messages),
-            )
-        };
-        if status != 0 {
-            return Err(Error::new(format!(
-                "Search for {} returned {}",
-                self.query_str, status,
-            )));
-        }
-        assert!(!messages.is_null());
-        Ok(MessageIterator {
-            messages,
-            lib: self.lib.clone(),
-            _ph: std::marker::PhantomData,
-            is_from_thread: false,
-        })
-    }
-}
-
-impl Drop for Query<'_> {
-    fn drop(&mut self) {
-        unsafe {
-            call!(self.lib, notmuch_query_destroy)(self.ptr);
-        }
-    }
-}
-
-pub trait MelibQueryToNotmuchQuery {
-    fn query_to_string(&self, ret: &mut String);
-}
-
-impl MelibQueryToNotmuchQuery for crate::search::Query {
-    fn query_to_string(&self, ret: &mut String) {
-        use crate::search::Query::*;
-        match self {
-            Before(timestamp) => {
-                ret.push_str("date:..@");
-                ret.push_str(&timestamp.to_string());
-            }
-            After(timestamp) => {
-                ret.push_str("date:@");
-                ret.push_str(&timestamp.to_string());
-                ret.push_str("..");
-            }
-            Between(a, b) => {
-                ret.push_str("date:@");
-                ret.push_str(&a.to_string());
-                ret.push_str("..@");
-                ret.push_str(&b.to_string());
-            }
-            On(timestamp) => {
-                ret.push_str("date:@");
-                ret.push_str(&timestamp.to_string());
-            }
-            /* * * * */
-            From(s) => {
-                ret.push_str("from:\"");
-                for c in s.chars() {
-                    if c == '"' {
-                        ret.push_str("\\\"");
-                    } else {
-                        ret.push(c);
-                    }
-                }
-                ret.push('"');
-            }
-            To(s) | Cc(s) | Bcc(s) => {
-                ret.push_str("to:\"");
-                for c in s.chars() {
-                    if c == '"' {
-                        ret.push_str("\\\"");
-                    } else {
-                        ret.push(c);
-                    }
-                }
-                ret.push('"');
-            }
-            InReplyTo(_s) | References(_s) | AllAddresses(_s) => {}
-            /* * * * */
-            Body(s) => {
-                ret.push_str("body:\"");
-                for c in s.chars() {
-                    if c == '"' {
-                        ret.push_str("\\\"");
-                    } else {
-                        ret.push(c);
-                    }
-                }
-                ret.push('"');
-            }
-            Subject(s) => {
-                ret.push_str("subject:\"");
-                for c in s.chars() {
-                    if c == '"' {
-                        ret.push_str("\\\"");
-                    } else {
-                        ret.push(c);
-                    }
-                }
-                ret.push('"');
-            }
-            AllText(s) => {
-                ret.push('"');
-                for c in s.chars() {
-                    if c == '"' {
-                        ret.push_str("\\\"");
-                    } else {
-                        ret.push(c);
-                    }
-                }
-                ret.push('"');
-            }
-            /* * * * */
-            Flags(v) => {
-                for f in v {
-                    ret.push_str("tag:\"");
-                    for c in f.chars() {
-                        if c == '"' {
-                            ret.push_str("\\\"");
-                        } else {
-                            ret.push(c);
-                        }
-                    }
-                    ret.push_str("\" ");
-                }
-                if !v.is_empty() {
-                    ret.pop();
-                }
-            }
-            HasAttachment => {
-                ret.push_str("tag:attachment");
-            }
-            And(q1, q2) => {
-                ret.push('(');
-                q1.query_to_string(ret);
-                ret.push_str(") AND (");
-                q2.query_to_string(ret);
-                ret.push(')');
-            }
-            Or(q1, q2) => {
-                ret.push('(');
-                q1.query_to_string(ret);
-                ret.push_str(") OR (");
-                q2.query_to_string(ret);
-                ret.push(')');
-            }
-            Not(q) => {
-                ret.push_str("(NOT (");
-                q.query_to_string(ret);
-                ret.push_str("))");
-            }
-            Answered => todo!(),
-            AnsweredBy { .. } => todo!(),
-            Larger { .. } => todo!(),
-            Smaller { .. } => todo!(),
-        }
-    }
-}
