@@ -52,8 +52,7 @@ pub use iterators::*;
 use smallvec::SmallVec;
 use uuid::Uuid;
 
-#[cfg(feature = "text-processing")]
-use crate::text_processing::grapheme_clusters::*;
+use crate::text::grapheme_clusters::*;
 
 type Envelopes = Arc<RwLock<HashMap<EnvelopeHash, Envelope>>>;
 
@@ -74,8 +73,9 @@ macro_rules! uuid_hash_type {
             }
         }
 
-        impl From<&[u8]> for $n {
-            fn from(val: &[u8]) -> Self {
+        impl<B: AsRef<[u8]>> From<B> for $n {
+            fn from(val: B) -> Self {
+                let val = val.as_ref();
                 $n(Uuid::new_v5(&Uuid::NAMESPACE_URL, val))
             }
         }
@@ -465,7 +465,7 @@ impl SubjectPrefix for &str {
     }
 }
 
-#[derive(Default, Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct Thread {
     pub root: ThreadNodeHash,
     pub date: UnixTimestamp,
@@ -814,7 +814,7 @@ impl Threads {
         }
     }
 
-    /// Update show_subject details of ThreadNode
+    /// Update `show_subject` details of [`ThreadNode`].
     pub fn update_show_subject(
         &mut self,
         id: ThreadNodeHash,
@@ -898,6 +898,9 @@ impl Threads {
 
         {
             let envelopes_lck = envelopes.read().unwrap();
+            if !envelopes_lck.contains_key(&env_hash) {
+                return false;
+            }
             let message_id = envelopes_lck[&env_hash].message_id().raw();
             if self.message_ids.contains_key(message_id)
                 && !self.missing_message_ids.contains(message_id)
@@ -1219,16 +1222,11 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     ma.subject()
                         .split_graphemes()
                         .cmp(&mb.subject().split_graphemes())
                 }
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    ma.subject().cmp(&mb.subject())
-                }
             }
             (SortField::Subject, SortOrder::Asc) => {
                 let a = &self.thread_nodes[&self.thread_ref(*a).root()].message();
@@ -1248,18 +1246,12 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     mb.subject()
                         .as_ref()
                         .split_graphemes()
                         .cmp(&ma.subject().split_graphemes())
                 }
-
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    mb.subject().as_ref().cmp(&ma.subject())
-                }
             }
         });
     }
@@ -1299,16 +1291,11 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     ma.subject()
                         .split_graphemes()
                         .cmp(&mb.subject().split_graphemes())
                 }
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    ma.subject().cmp(&mb.subject())
-                }
             }
             (SortField::Subject, SortOrder::Asc) => {
                 let a = &self.thread_nodes[a].message();
@@ -1328,18 +1315,12 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     mb.subject()
                         .as_ref()
                         .split_graphemes()
                         .cmp(&ma.subject().split_graphemes())
                 }
-
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    mb.subject().as_ref().cmp(&ma.subject())
-                }
             }
         });
     }
@@ -1375,16 +1356,11 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     ma.subject()
                         .split_graphemes()
                         .cmp(&mb.subject().split_graphemes())
                 }
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    ma.subject().cmp(&mb.subject())
-                }
             }
             (SortField::Subject, SortOrder::Asc) => {
                 let a = &self.thread_nodes[a].message();
@@ -1404,18 +1380,12 @@ impl Threads {
                 }
                 let ma = &envelopes[&a.unwrap()];
                 let mb = &envelopes[&b.unwrap()];
-                #[cfg(feature = "text-processing")]
                 {
                     mb.subject()
                         .as_ref()
                         .split_graphemes()
                         .cmp(&ma.subject().split_graphemes())
                 }
-
-                #[cfg(not(feature = "text-processing"))]
-                {
-                    mb.subject().as_ref().cmp(&ma.subject())
-                }
             }
         });
     }
@@ -1510,28 +1480,28 @@ fn print_threadnodes(
 }
 */
 
-#[derive(Serialize)]
-struct Node {
-    id: String,
-    subject: String,
-    from: String,
-    to: String,
-    date: UnixTimestamp,
-    references: String,
-    in_reply_to: String,
-}
+// #[derive(Serialize)]
+// struct Node {
+//     id: String,
+//     subject: String,
+//     from: String,
+//     to: String,
+//     date: UnixTimestamp,
+//     references: String,
+//     in_reply_to: String,
+// }
 
-#[derive(Serialize)]
-struct Link {
-    source: String,
-    target: String,
-}
+// #[derive(Serialize)]
+// struct Link {
+//     source: String,
+//     target: String,
+// }
 
-#[derive(Serialize)]
-struct Graph {
-    nodes: Vec<Node>,
-    links: Vec<Link>,
-}
+// #[derive(Serialize)]
+// struct Graph {
+//     nodes: Vec<Node>,
+//     links: Vec<Link>,
+// }
 
 /*
 fn save_graph(
