@@ -20,7 +20,7 @@
  */
 
 //! Connections layers (TCP/fd/TLS/Deflate) to use with remote backends.
-use std::{os::unix::io::AsRawFd, time::Duration};
+use std::{borrow::Cow, os::unix::io::AsRawFd, time::Duration};
 
 use flate2::{read::DeflateDecoder, write::DeflateEncoder, Compression};
 #[cfg(any(target_os = "openbsd", target_os = "netbsd", target_os = "haiku"))]
@@ -132,6 +132,66 @@ macro_rules! syscall {
     }};
 }
 
+/// Hardcoded `setsockopt` arguments for type safety when calling
+/// [`Connection::setsockopt`] in an `unsafe` block.
+///
+/// Add new variants when you need to call `setsockopt` with new arguments.
+pub enum SockOpts {
+    /// Set TCP Keep Alive.
+    ///
+    /// Following text is sourced from <https://tldp.org/HOWTO/html_single/TCP-Keepalive-HOWTO/>.
+    ///
+    /// ```text
+    /// 4.2. The setsockopt function call
+    ///
+    /// All you need to enable keepalive for a specific socket is to set the specific socket option
+    /// on the socket itself. The prototype of the function is as follows:
+    ///
+    ///
+    ///   int setsockopt(int s, int level, int optname,
+    ///                  const void *optval, socklen_t optlen)
+    ///
+    ///
+    /// The first parameter is the socket, previously created with the socket(2); the second one
+    /// must be SOL_SOCKET, and the third must be SO_KEEPALIVE . The fourth parameter must be a
+    /// boolean integer value, indicating that we want to enable the option, while the last is the
+    /// size of the value passed before.
+    ///
+    /// According to the manpage, 0 is returned upon success, and -1 is returned on error (and
+    /// errno is properly set).
+    ///
+    /// There are also three other socket options you can set for keepalive when you write your
+    /// application. They all use the SOL_TCP level instead of SOL_SOCKET, and they override
+    /// system-wide variables only for the current socket. If you read without writing first, the
+    /// current system-wide parameters will be returned.
+    ///
+    ///     TCP_KEEPCNT: overrides tcp_keepalive_probes
+    ///
+    ///     TCP_KEEPIDLE: overrides tcp_keepalive_time
+    ///
+    ///     TCP_KEEPINTVL: overrides tcp_keepalive_intvl
+    /// ```
+    ///
+    /// Field `duration` overrides `tcp_keepalive_time`:
+    ///
+    /// ```text
+    /// tcp_keepalive_time
+    ///
+    ///    the interval between the last data packet sent (simple ACKs are not considered data) and the
+    ///    first keepalive probe; after the connection is marked to need keepalive, this counter is not
+    ///    used any further
+    /// ```
+    ///
+    /// The default value in the Linux kernel is 7200 seconds (2 hours).
+    KeepAlive {
+        enable: bool,
+        duration: Option<Duration>,
+    },
+    TcpNoDelay {
+        enable: bool,
+    },
+}
+
 impl Connection {
     pub const IO_BUF_SIZE: usize = 64 * 1024;
 
@@ -160,11 +220,14 @@ impl Connection {
     }
 
     pub fn new_tcp(inner: std::net::TcpStream) -> Self {
-        Self::Tcp {
+        let ret = Self::Tcp {
             inner,
             id: None,
             trace: false,
-        }
+        };
+        _ = ret.setsockopt(SockOpts::TcpNoDelay { enable: true });
+
+        ret
     }
 
     pub fn trace(mut self, val: bool) -> Self {
@@ -288,11 +351,11 @@ impl Connection {
             return Ok(None);
         }
         unsafe {
-            let raw: c_int = self.getsockopt(libc::SOL_SOCKET, libc::SO_KEEPALIVE)?;
+            let raw: c_int = self.__getsockopt(libc::SOL_SOCKET, libc::SO_KEEPALIVE)?;
             if raw == 0 {
                 return Ok(None);
             }
-            let secs: c_int = self.getsockopt(libc::IPPROTO_TCP, KEEPALIVE_OPTION)?;
+            let secs: c_int = self.__getsockopt(libc::IPPROTO_TCP, KEEPALIVE_OPTION)?;
             Ok(Some(Duration::new(secs as u64, 0)))
         }
     }
@@ -312,21 +375,13 @@ impl Connection {
         if matches!(self, Fd { .. }) {
             return Ok(());
         }
-        unsafe {
-            self.setsockopt(
-                libc::SOL_SOCKET,
-                libc::SO_KEEPALIVE,
-                keepalive.is_some() as c_int,
-            )?;
-            if let Some(dur) = keepalive {
-                // [ref:TODO]: checked cast here
-                self.setsockopt(libc::IPPROTO_TCP, KEEPALIVE_OPTION, dur.as_secs() as c_int)?;
-            }
-            Ok(())
-        }
+        self.setsockopt(SockOpts::KeepAlive {
+            enable: keepalive.is_some(),
+            duration: keepalive,
+        })
     }
 
-    unsafe fn setsockopt<T>(&self, opt: c_int, val: c_int, payload: T) -> std::io::Result<()>
+    unsafe fn inner_setsockopt<T>(&self, opt: c_int, val: c_int, payload: T) -> std::io::Result<()>
     where
         T: Copy,
     {
@@ -341,7 +396,67 @@ impl Connection {
         Ok(())
     }
 
-    unsafe fn getsockopt<T: Copy>(&self, opt: c_int, val: c_int) -> std::io::Result<T> {
+    fn setsockopt(&self, option: SockOpts) -> std::io::Result<()> {
+        match option {
+            SockOpts::KeepAlive {
+                enable: true,
+                duration,
+            } => {
+                unsafe {
+                    self.inner_setsockopt(libc::SOL_SOCKET, libc::SO_KEEPALIVE, <c_int>::from(true))
+                }?;
+                if let Some(dur) = duration {
+                    unsafe {
+                        self.inner_setsockopt(
+                            libc::IPPROTO_TCP,
+                            KEEPALIVE_OPTION,
+                            dur.as_secs() as c_int,
+                        )
+                    }?;
+                }
+                Ok(())
+            }
+            SockOpts::KeepAlive {
+                enable: false,
+                duration: _,
+            } => unsafe {
+                self.inner_setsockopt(libc::SOL_SOCKET, libc::SO_KEEPALIVE, <c_int>::from(false))
+            },
+            SockOpts::TcpNoDelay { enable } => unsafe {
+                #[cfg(any(
+                    target_os = "openbsd",
+                    target_os = "netbsd",
+                    target_os = "haiku",
+                    target_os = "macos",
+                    target_os = "ios"
+                ))]
+                {
+                    self.inner_setsockopt(
+                        libc::IPPROTO_TCP,
+                        libc::TCP_NODELAY,
+                        if enable { c_int::from(1_u8) } else { 0 },
+                    )
+                }
+                #[cfg(not(any(
+                    target_os = "openbsd",
+                    target_os = "netbsd",
+                    target_os = "haiku",
+                    target_os = "macos",
+                    target_os = "ios"
+                )))]
+                {
+                    self.inner_setsockopt(
+                        libc::SOL_TCP,
+                        libc::TCP_NODELAY,
+                        if enable { c_int::from(1_u8) } else { 0 },
+                    )
+                }
+            },
+        }
+    }
+
+    #[inline]
+    unsafe fn __getsockopt<T: Copy>(&self, opt: c_int, val: c_int) -> std::io::Result<T> {
         let mut slot: T = std::mem::zeroed();
         let mut len = std::mem::size_of::<T>() as libc::socklen_t;
         syscall!(getsockopt(
@@ -399,25 +514,33 @@ impl std::io::Read for Connection {
         };
         if self.is_trace_enabled() {
             let id = self.id();
-            if let Ok(len) = &res {
-                log::trace!(
-                    "{}{}{}{:?} read {:?} bytes:{:?}",
-                    if id.is_some() { "[" } else { "" },
-                    if let Some(id) = id.as_ref() { id } else { "" },
-                    if id.is_some() { "]: " } else { "" },
-                    self,
-                    len,
-                    String::from_utf8_lossy(&buf[..*len])
-                );
-            } else {
-                log::trace!(
-                    "{}{}{}{:?} could not read {:?}",
-                    if id.is_some() { "[" } else { "" },
-                    if let Some(id) = id.as_ref() { id } else { "" },
-                    if id.is_some() { "]: " } else { "" },
-                    self,
-                    &res
-                );
+            match &res {
+                Ok(len) => {
+                    let slice = &buf[..*len];
+                    log::trace!(
+                        "{}{}{}{:?} read {:?} bytes:{}",
+                        if id.is_some() { "[" } else { "" },
+                        if let Some(id) = id.as_ref() { id } else { "" },
+                        if id.is_some() { "]: " } else { "" },
+                        self,
+                        len,
+                        std::str::from_utf8(slice)
+                            .map(Cow::Borrowed)
+                            .or_else(|_| crate::text::hex::bytes_to_hex(slice).map(Cow::Owned))
+                            .unwrap_or(Cow::Borrowed("Could not convert to hex."))
+                    );
+                }
+                Err(err) if matches!(err.kind(), std::io::ErrorKind::WouldBlock) => {}
+                Err(err) => {
+                    log::trace!(
+                        "{}{}{}{:?} could not read {:?}",
+                        if id.is_some() { "[" } else { "" },
+                        if let Some(id) = id.as_ref() { id } else { "" },
+                        if id.is_some() { "]: " } else { "" },
+                        self,
+                        err,
+                    );
+                }
             }
         }
         res
@@ -429,13 +552,16 @@ impl std::io::Write for Connection {
         if self.is_trace_enabled() {
             let id = self.id();
             log::trace!(
-                "{}{}{}{:?} writing {} bytes:{:?}",
+                "{}{}{}{:?} writing {} bytes:{}",
                 if id.is_some() { "[" } else { "" },
                 if let Some(id) = id.as_ref() { id } else { "" },
                 if id.is_some() { "]: " } else { "" },
                 self,
                 buf.len(),
-                String::from_utf8_lossy(buf)
+                std::str::from_utf8(buf)
+                    .map(Cow::Borrowed)
+                    .or_else(|_| crate::text::hex::bytes_to_hex(buf).map(Cow::Owned))
+                    .unwrap_or(Cow::Borrowed("Could not convert to hex."))
             );
         }
         match self {
