@@ -90,29 +90,27 @@ extern "C" {
     fn gettimeofday(tv: *mut libc::timeval, tz: *mut libc::timezone) -> i32;
 }
 
-#[cfg(not(target_os = "openbsd"))]
+#[cfg(target_os = "openbsd")]
+const _LC_LAST: c_int = 7;
+#[cfg(all(target_os = "openbsd", target_env = "musl"))]
+const PLATFORM_ALL_MASK: i32 = 0x7fffffff;
+#[cfg(all(target_os = "openbsd", not(target_env = "musl")))]
+const PLATFORM_ALL_MASK: i32 = (1 << _LC_LAST) - 2;
+#[cfg(all(not(target_os = "openbsd"), target_env = "musl"))]
+const PLATFORM_ALL_MASK: i32 = 0x7fffffff;
+#[cfg(all(not(target_os = "openbsd"), not(target_env = "musl")))]
+const PLATFORM_ALL_MASK: i32 = libc::LC_ALL_MASK;
+
 #[repr(i32)]
-#[derive(Copy, Clone)]
+#[derive(Clone, Copy)]
 #[allow(dead_code)]
 enum LocaleCategoryMask {
     Time = libc::LC_TIME_MASK,
-    All = libc::LC_ALL_MASK,
-}
-
-#[cfg(target_os = "openbsd")]
-const _LC_LAST: c_int = 7;
-
-#[cfg(target_os = "openbsd")]
-#[repr(i32)]
-#[derive(Copy, Clone)]
-#[allow(dead_code)]
-enum LocaleCategoryMask {
-    Time = 1 << libc::LC_TIME,
-    All = (1 << _LC_LAST) - 2,
+    All = PLATFORM_ALL_MASK,
 }
 
 #[repr(i32)]
-#[derive(Copy, Clone)]
+#[derive(Clone, Copy)]
 #[allow(dead_code)]
 enum LocaleCategory {
     Time = libc::LC_TIME,
@@ -124,8 +122,8 @@ enum LocaleCategory {
 struct Locale {
     mask: LocaleCategoryMask,
     category: LocaleCategory,
-    new_locale: libc::locale_t,
-    old_locale: libc::locale_t,
+    new: libc::locale_t,
+    old: libc::locale_t,
 }
 
 #[cfg(target_os = "netbsd")]
@@ -133,19 +131,19 @@ struct Locale {
 struct Locale {
     mask: LocaleCategoryMask,
     category: LocaleCategory,
-    old_locale: *const std::os::raw::c_char,
+    old: *const std::os::raw::c_char,
 }
 
 impl Drop for Locale {
     fn drop(&mut self) {
         #[cfg(not(target_os = "netbsd"))]
         unsafe {
-            let _ = libc::uselocale(self.old_locale);
-            libc::freelocale(self.new_locale);
+            let _ = libc::uselocale(self.old);
+            libc::freelocale(self.new);
         }
         #[cfg(target_os = "netbsd")]
         unsafe {
-            let _ = libc::setlocale(self.category as c_int, self.old_locale);
+            let _ = libc::setlocale(self.category as c_int, self.old);
         }
     }
 }
@@ -160,22 +158,23 @@ impl Locale {
         locale: *const std::os::raw::c_char,
         base: libc::locale_t,
     ) -> Result<Self> {
-        let new_locale = unsafe { libc::newlocale(mask as c_int, locale, base) };
-        if new_locale.is_null() {
+        let new = unsafe { libc::newlocale(mask as c_int, locale, base) };
+        if new.is_null() {
             return Err(nix::Error::last().into());
         }
-        let old_locale = unsafe { libc::uselocale(new_locale) };
-        if old_locale.is_null() {
-            unsafe { libc::freelocale(new_locale) };
+        let old = unsafe { libc::uselocale(new) };
+        if old.is_null() {
+            unsafe { libc::freelocale(new) };
             return Err(nix::Error::last().into());
         }
         Ok(Self {
             mask,
             category,
-            new_locale,
-            old_locale,
+            new,
+            old,
         })
     }
+
     #[cfg(target_os = "netbsd")]
     fn new(
         mask: LocaleCategoryMask,
@@ -183,18 +182,18 @@ impl Locale {
         locale: *const std::os::raw::c_char,
         _base: libc::locale_t,
     ) -> Result<Self> {
-        let old_locale = unsafe { libc::setlocale(category as c_int, std::ptr::null_mut()) };
-        if old_locale.is_null() {
+        let old = unsafe { libc::setlocale(category as c_int, std::ptr::null_mut()) };
+        if old.is_null() {
             return Err(nix::Error::last().into());
         }
-        let new_locale = unsafe { libc::setlocale(category as c_int, locale) };
-        if new_locale.is_null() {
+        let new = unsafe { libc::setlocale(category as c_int, locale) };
+        if new.is_null() {
             return Err(nix::Error::last().into());
         }
         Ok(Locale {
             mask,
             category,
-            old_locale,
+            old,
         })
     }
 }
@@ -790,7 +789,9 @@ mod tests {
 
     #[test]
     fn test_datetime_rfcs() {
-        if unsafe { libc::setlocale(libc::LC_ALL, b"\0".as_ptr() as *const i8) }.is_null() {
+        if unsafe { libc::setlocale(libc::LC_ALL, b"\0".as_ptr() as *const core::ffi::c_char) }
+            .is_null()
+        {
             eprintln!("Unable to set locale.");
         }
         /* Some tests were lazily stolen from https://rachelbythebay.com/w/2013/06/11/time/ */
